
/***************************************************************************
*   Copyright (C) 2004 by Christoph Thielecke                             *
*   crissi99@gmx.de                                                       *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

// BEGIN includes
#include <qcheckbox.h>
#include <klineedit.h>
#include <kcombobox.h>
#include <qspinbox.h>
#include <qpushbutton.h>
#include <klocale.h>
#include "advancedsettingsdialog.h"
// END includes

AdvancedSettingsDialog::AdvancedSettingsDialog(QWidget *parent, const QString& caption, VpnAccountData *currentProfile, VpnAccountData::ConnectionType connectionType) : KDialogBase( parent, i18n("Advanced settings"), true, caption, KDialogBase::Ok|KDialogBase::Cancel, KDialogBase::Ok, true )
{

	main = new AdvancedProfileDialogBase(this);
	setMainWidget(main);
	main->setMinimumSize(main->sizeHint());

	this->currentProfile = currentProfile;
	this->connectionType = connectionType;


	connect (main->AdvancedSettingsCheckbox, SIGNAL(toggled(bool)),
	         this, SLOT(advancedSettingsToggled(bool)));


	connect (main->NtDomainNameCheckbox, SIGNAL(toggled(bool)),
	         this, SLOT(domainNameToggled(bool)));


	connect (main->SetDefaultRouteCheckbox, SIGNAL(toggled(bool)),
	         this, SLOT(setDefaultRouteToggled(bool)));


	connect (main->DnsServerCheckbox, SIGNAL(toggled(bool)),
	         this, SLOT(useDnsServerToggled(bool)));


	connect (main->ApplicationVersionCheckbox, SIGNAL(toggled(bool)),
	         this, SLOT(applicationVersionToggled(bool)));

	connect (main->PerfectForwardSecurityCheckbox, SIGNAL(toggled(bool)),
	         this, SLOT(perfectForwardSecurityToggled(bool)));

	connect (main->MtuCheckbox, SIGNAL(toggled(bool)),
	         this, SLOT(mtuToggled(bool)));

	connect (main->IkeGroupCheckbox, SIGNAL(toggled(bool)),
	         this, SLOT(IkeGroupToggled(bool)));


	connect (main->SingleDesCheckbox, SIGNAL(toggled(bool)),
	         this, SLOT(singleDesToggled(bool)));


	connect (main->LocalPortCheckbox, SIGNAL(toggled(bool)),
	         this, SLOT(localPortToggled(bool)));


	main->LocalPortCheckbox->setEnabled( false );
	main->IkeGroupCheckbox->setEnabled( false );
	main->PerfectForwardSecrecyCombobox->setEnabled( false );
	main->SingleDesCheckbox->setEnabled( false );
	main->NtDomainNameCheckbox->setEnabled( false );
	main->LocalPortSpinbox->setEnabled( false );
	main->ApplicationVersionLineedit->setEnabled( false );
	main->NtDomainNameLineedit->setEnabled( false );
	main->PerfectForwardSecurityCheckbox->setEnabled( false );
	main->IkeGroupCombobox->setEnabled( false );
	main->ApplicationVersionCheckbox->setEnabled( false );
	main->UseGlobalIpsecSecretCheckbox->setEnabled( false );
	main->RefuseEapCheckbox->setEnabled( false );
	main->requireMppeCheckbox->setEnabled( false );
	main->Refuse128BitEncryptionCheckbox->setEnabled(false );
	main->Refuse40BitEncryptionCheckbox->setEnabled( false );
	main->DnsServerCheckbox->setEnabled( false );
	main->DnsServerLineedit->setEnabled( false );
	main->AllowStatefulModeCheckbox->setEnabled( false );
	main->GetDnsFromPeerCheckbox->setEnabled( false );
	main->IkeGroupCheckbox->setEnabled( false );
	main->UseNoBsdCompCheckbox->setEnabled( false );
	main->UseNoDeflateCheckbox->setEnabled( false );
	main->ReplaceDefaultRouteCheckbox->setEnabled( false );
	main->SetDefaultRouteCheckbox->setEnabled( false );
	main->MtuSpinbox->setEnabled( false );
	main->MtuCheckbox->setEnabled( false );

	main->IkeGroupCombobox->insertItem( "dh1", 0 );
	main->IkeGroupCombobox->insertItem( "dh2", 1 );
	main->IkeGroupCombobox->insertItem( "dh5", 2 );

	main->PerfectForwardSecrecyCombobox->insertItem( "server", 0 );
	main->PerfectForwardSecrecyCombobox->insertItem( "nopfs", 1 );
	main->PerfectForwardSecrecyCombobox->insertItem( "dh1", 2 );
	main->PerfectForwardSecrecyCombobox->insertItem( "dh2", 3 );
	main->PerfectForwardSecrecyCombobox->insertItem( "dh5", 4 );

	main->ApplicationVersionLineedit->setText( "Cisco Systems VPN Client 0.2-rm+zomb.1:Linux" );

	main->AdvancedSettingsCheckbox->setChecked(currentProfile->getUseAdvancedSettings());
	this->advancedSettingsToggled(true);
}


AdvancedSettingsDialog::~AdvancedSettingsDialog() {}

void AdvancedSettingsDialog::advancedSettingsToggled(bool)
{
	main->LocalPortCheckbox->setEnabled( false );
	main->IkeGroupCheckbox->setEnabled( false );
	main->PerfectForwardSecrecyCombobox->setEnabled( false );
	main->SingleDesCheckbox->setEnabled( false );
	main->NtDomainNameCheckbox->setEnabled( false );
	main->LocalPortSpinbox->setEnabled( false );
	main->ApplicationVersionLineedit->setEnabled( false );
	main->NtDomainNameLineedit->setEnabled( false );
	main->PerfectForwardSecurityCheckbox->setEnabled( false );
	main->IkeGroupCombobox->setEnabled( false );
	main->ApplicationVersionCheckbox->setEnabled( false );
	main->UseGlobalIpsecSecretCheckbox->setEnabled( false );
	main->RefuseEapCheckbox->setEnabled( false );
	main->requireMppeCheckbox->setEnabled( false );
	main->Refuse128BitEncryptionCheckbox->setEnabled(false );
	main->Refuse40BitEncryptionCheckbox->setEnabled( false );
	main->DnsServerCheckbox->setEnabled( false );
	main->DnsServerLineedit->setEnabled( false );
	main->AllowStatefulModeCheckbox->setEnabled( false );
	main->GetDnsFromPeerCheckbox->setEnabled( false );
	main->ReplaceDefaultRouteCheckbox->setEnabled( false );
	main->SetDefaultRouteCheckbox->setEnabled( false );
	if ( this->main->AdvancedSettingsCheckbox->isChecked() )
	{
		if (connectionType == VpnAccountData::ConnectionType(VpnAccountData::cisco))
		{
			main->RefuseEapCheckbox->setEnabled( false );
			main->requireMppeCheckbox->setEnabled( false );
			main->Refuse128BitEncryptionCheckbox->setEnabled(false );
			main->Refuse40BitEncryptionCheckbox->setEnabled( false );
			main->DnsServerCheckbox->setEnabled( false );
			main->DnsServerLineedit->setEnabled( false );

			main->AllowStatefulModeCheckbox->setEnabled( false );
			main->GetDnsFromPeerCheckbox->setEnabled( false );
			//if (currentProfile->getUseAdvancedSettings())
			//{
			main->LocalPortCheckbox->setEnabled( true );
			main->SingleDesCheckbox->setEnabled( true );
			main->NtDomainNameCheckbox->setEnabled( true );
			main->PerfectForwardSecurityCheckbox->setEnabled( true );
			main->ApplicationVersionCheckbox->setEnabled( true );
			main->UseGlobalIpsecSecretCheckbox->setEnabled( true );
			main->PerfectForwardSecurityCheckbox->setEnabled ( true );
		}
		else if (connectionType == VpnAccountData::ConnectionType(VpnAccountData::racoon))
		{
			main->LocalPortCheckbox->setEnabled( false );
			main->IkeGroupCheckbox->setEnabled( false );
			main->PerfectForwardSecrecyCombobox->setEnabled( false );
			main->SingleDesCheckbox->setEnabled( false );
			main->NtDomainNameCheckbox->setEnabled( false );
			main->LocalPortSpinbox->setEnabled( false );
			main->ApplicationVersionLineedit->setEnabled( false );
			main->NtDomainNameLineedit->setEnabled( false );
			main->PerfectForwardSecurityCheckbox->setEnabled( false );
			main->IkeGroupCombobox->setEnabled( false );
			main->ApplicationVersionCheckbox->setEnabled( false );
			main->UseGlobalIpsecSecretCheckbox->setEnabled( false );
			main->RefuseEapCheckbox->setEnabled( false );
			main->requireMppeCheckbox->setEnabled( false );
			main->Refuse128BitEncryptionCheckbox->setEnabled(false );
			main->Refuse40BitEncryptionCheckbox->setEnabled( false );
			main->DnsServerCheckbox->setEnabled( false );
			main->DnsServerLineedit->setEnabled( false );
			main->AllowStatefulModeCheckbox->setEnabled( false );
			main->GetDnsFromPeerCheckbox->setEnabled( false );

			main->IkeGroupCheckbox->setEnabled( true );
		}

		else if (connectionType == VpnAccountData::ConnectionType( VpnAccountData::pptp))
		{
			main->RefuseEapCheckbox->setEnabled( true );
			main->requireMppeCheckbox->setEnabled( true );
			main->Refuse128BitEncryptionCheckbox->setEnabled( true );
			main->Refuse40BitEncryptionCheckbox->setEnabled( true );
			main->DnsServerCheckbox->setEnabled( true );
			//main->DnsServerLineedit->setEnabled( true );
			main->NtDomainNameCheckbox->setEnabled( true );
			main->NtDomainNameLineedit->setEnabled( true );
			main->AllowStatefulModeCheckbox->setEnabled( true );
			main->GetDnsFromPeerCheckbox->setEnabled( true );
			main->UseNoBsdCompCheckbox->setEnabled( true );
			main->UseNoDeflateCheckbox->setEnabled( true );
			main->ReplaceDefaultRouteCheckbox->setEnabled( true );
			main->SetDefaultRouteCheckbox->setEnabled( true );
			main->MtuCheckbox->setEnabled( true );
		}
	}

	else
	{
		main->LocalPortCheckbox->setEnabled( false );
		main->IkeGroupCheckbox->setEnabled( false );
		main->PerfectForwardSecrecyCombobox->setEnabled( false );
		main->SingleDesCheckbox->setEnabled( false );
		main->NtDomainNameCheckbox->setEnabled( false );
		main->LocalPortSpinbox->setEnabled( false );
		main->ApplicationVersionLineedit->setEnabled( false );
		main->NtDomainNameLineedit->setEnabled( false );
		main->PerfectForwardSecurityCheckbox->setEnabled( false );
		main->IkeGroupCombobox->setEnabled( false );
		main->ApplicationVersionCheckbox->setEnabled( false );
		main->UseGlobalIpsecSecretCheckbox->setEnabled( false );
		main->RefuseEapCheckbox->setEnabled( false );
		main->requireMppeCheckbox->setEnabled( false );
		main->Refuse128BitEncryptionCheckbox->setEnabled(false );
		main->Refuse40BitEncryptionCheckbox->setEnabled( false );
		main->DnsServerCheckbox->setEnabled( false );
		main->DnsServerLineedit->setEnabled( false );
		main->AllowStatefulModeCheckbox->setEnabled( false );
		main->GetDnsFromPeerCheckbox->setEnabled( false );
	}
}

void AdvancedSettingsDialog::domainNameToggled(bool)
{
	if ( this->main->NtDomainNameCheckbox->isChecked() )
		this->main->NtDomainNameLineedit->setEnabled( true );
	else
		this->main->NtDomainNameLineedit->setEnabled( false );
}

void AdvancedSettingsDialog::applicationVersionToggled(bool)
{
	if ( this->main->ApplicationVersionCheckbox->isChecked() )
		this->main->ApplicationVersionLineedit->setEnabled( true );
	else
		this->main->ApplicationVersionLineedit->setEnabled( false );
}

void AdvancedSettingsDialog::IkeGroupToggled(bool)
{
	if ( this->main->IkeGroupCheckbox->isChecked() )
		this->main->IkeGroupCombobox->setEnabled( true );
	else
		this->main->IkeGroupCombobox->setEnabled( false );
}

void AdvancedSettingsDialog::perfectForwardSecurityToggled(bool)
{
	if ( this->main->PerfectForwardSecurityCheckbox->isChecked() )
		this->main->PerfectForwardSecrecyCombobox->setEnabled( true );
	else
		this->main->PerfectForwardSecrecyCombobox->setEnabled( false );
}

void AdvancedSettingsDialog::singleDesToggled(bool) {}

void AdvancedSettingsDialog::helpClicked() {}

void AdvancedSettingsDialog::localPortToggled(bool)
{
	if ( this->main->LocalPortCheckbox->isChecked() )
		this->main->LocalPortSpinbox->setEnabled( true );
	else
		this->main->LocalPortSpinbox->setEnabled( false );
}

void AdvancedSettingsDialog::useDnsServerToggled(bool)
{
	if ( this->main->DnsServerCheckbox->isChecked() )
		this->main->DnsServerLineedit->setEnabled( true );
	else
		this->main->DnsServerLineedit->setEnabled( false );
}


void AdvancedSettingsDialog::accept()
{
	currentProfile->setUseAdvancedSettings( this->main->AdvancedSettingsCheckbox->isChecked() );

	currentProfile->setNtDomainName( this->main->NtDomainNameLineedit->text() );

	currentProfile->setUseNtDomainName( this->main->NtDomainNameCheckbox->isChecked() );

	currentProfile->setApplicationVersion( this->main->ApplicationVersionLineedit->text() );
	currentProfile->setUseApplicationVersion( this->main->ApplicationVersionCheckbox->isChecked() );

	currentProfile->setIkeGroup( this->main->IkeGroupCombobox->currentText() );
	currentProfile->setUseIkeGroup( this->main->IkeGroupCheckbox->isChecked() );

	currentProfile->setPerfectForwardSecrety( this->main->PerfectForwardSecrecyCombobox->currentText() );
	currentProfile->setUsePerfectForwardSecrety( this->main->PerfectForwardSecurityCheckbox->isChecked() );

	currentProfile->setUseSingleDes( this->main->SingleDesCheckbox->isChecked() );

	currentProfile->setUseGlobalIpsecSecret(this->main->UseGlobalIpsecSecretCheckbox->isChecked() );

	currentProfile->setLocalPort( this->main->LocalPortSpinbox->value() );

	currentProfile->setUseLocalPort( this->main->LocalPortCheckbox->isChecked() );

	currentProfile->setRefuseEap(this->main->RefuseEapCheckbox->isChecked( ) );
	currentProfile->setRequireMppe(this->main->requireMppeCheckbox->isChecked()  );
	currentProfile->setRefuse128BitEncryption(this->main->Refuse128BitEncryptionCheckbox->isChecked( ) );
	currentProfile->setRefuse40BitEncryption(this->main->Refuse40BitEncryptionCheckbox->isChecked( ) );
	currentProfile->setUseGetDnsFromPeer(this->main->GetDnsFromPeerCheckbox->isChecked());
	currentProfile->setUseDnsServer(this->main->DnsServerCheckbox->isChecked( ) );
	currentProfile->setDnsServer(this->main->DnsServerLineedit->text( ) );

	currentProfile->setUseNtDomainName(this->main->NtDomainNameCheckbox->isChecked( ) );
	currentProfile->setNtDomainName(this->main->NtDomainNameLineedit->text( ) );

	currentProfile->setUseNoBsdComp(this->main->UseNoBsdCompCheckbox->isChecked());
	currentProfile->setUseNoDeflate(this->main->UseNoDeflateCheckbox->isChecked());

	currentProfile->setSetDefaultRoute(this->main->SetDefaultRouteCheckbox->isChecked());
	currentProfile->setReplaceDefaultRoute(this->main->ReplaceDefaultRouteCheckbox->isChecked());

	currentProfile->setUseMtu(this->main->MtuCheckbox->isChecked());
	currentProfile->setMtu(this->main->MtuSpinbox->value());

	QDialog::accept();
}

void AdvancedSettingsDialog::setDefaultRouteToggled(bool)
{
	if (main->SetDefaultRouteCheckbox->isChecked())
		main->ReplaceDefaultRouteCheckbox->setEnabled(true);
	else
		main->ReplaceDefaultRouteCheckbox->setEnabled(false);
}

void AdvancedSettingsDialog::mtuToggled(bool)
{
	if (main->MtuCheckbox->isChecked())
		main->MtuSpinbox->setEnabled(true);
	else
		main->MtuSpinbox->setEnabled(false);
}

