/***************************************************************************
*   Copyright (C) 2004 by Christoph Thielecke                             *
*   crissi99@gmx.de                                                       *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
// BEGIN includes
#include "newprofiledialog.h"
#include <kcombobox.h>
#include <klineedit.h>
#include <qcheckbox.h>
#include <kurlrequester.h>
#include <kmessagebox.h>
#include <qlistbox.h>
#include <klineeditdlg.h>
#include <klocale.h>
#include <knuminput.h>
#include <qobject.h>
#include <qtextedit.h>
#include <qlabel.h>
#include "utils.h"


#include "networkinterface.h"
#include "importcertificatedialog.h"
#include "advancedsettingsdialog.h"
#include "vpnaccountdata.h"

// END includes

NewProfileDialog::NewProfileDialog( QApplication *app,QWidget *parent, const QString& caption, KVpncConfig *GlobalConfig, QTextEdit *LogOutput)
		: KDialogBase( parent, i18n("New profile"), true, caption, 
		KDialogBase::Ok|KDialogBase::Cancel, KDialogBase::Ok, true )  	
{

	this->AccountList = GlobalConfig->AccountList;
	this->LogOutput = LogOutput;
	this->GlobalConfig = GlobalConfig;
	this->app = app;

	main = new NewProfileDialogbase(this);
	setMainWidget(main);
	main->setMinimumSize(main->sizeHint());

	connect( main->ConnectionTypeComboBox, SIGNAL( activated( const QString& ) ), this, SLOT( connectiontypeChanged( const QString& ) ) );
	connect( main->AuthTypeComboBox , SIGNAL( activated( const QString& ) ), this, SLOT( authTypeChanged( const QString& ) ) );
//	connect( main->buttonCancel , SIGNAL( clicked () ), this, SLOT( reject() ) );
	connect( main->ImportCertificatePushButton, SIGNAL( clicked () ), this, SLOT( importCertificate() ) );
	connect( main->AdvancedSettingsPushButton, SIGNAL( clicked () ), this, SLOT( advancedSettingsClicked() ) );
	newProfileCreated = FALSE;
	Name = "";

	// currently static for new profile
	VpnAccountData::ConnectionType ConnType = VpnAccountData::ConnectionType(VpnAccountData::other);
	Port = 500;
	Gateway = "";
	ID = "";
	Username = "";
	UserPassword = "";
	PreSharedKey = "";
	ApplicationVersion = "";
	NtDomainName = "";
	PerfectForwardSecrecy = "";
	IkeGroup = "";
	useApplicationVersion = FALSE;
	useGlobalIpsecSecret = FALSE;
	useIkeGroup = FALSE;
	useLocalPort = FALSE;
	useNtDomainName = FALSE;
	useSingleDes = FALSE;
	useAdvancedSettings = FALSE;
	usePerfectSecrecy = FALSE;
	saveUserPassword = FALSE;
	savePsk = FALSE;
	useUdp=false;
	useUdpPort=false;
	UdpPort=4500;
	useDnsServer=false;
	DnsServer="";
	useGetDnsFromPeer=true;
	refuse40BitEncryption=false;
	refuse128BitEncryption=false;
	allowStatefulMode=false;
	requireMppe=false;
	refuseEap=false;
	useNoBsdComp=true;
	useNoDeflate=true;
	savePsk=false;


	profileData = new VpnAccountData( ConnType, Name );
	profileData->setGateway ( Gateway );
	profileData->setID( ID );
	profileData->setPreSharedKey( PreSharedKey);
	profileData->setSavePsk( savePsk );
	profileData->setUserName( Username );
	profileData->setUserPassword( UserPassword );
	profileData->setSaveUserPassword( saveUserPassword );
	profileData->setSavePsk(  savePsk );
	profileData->setApplicationVersion( ApplicationVersion );
	profileData->setUseLocalPort( useLocalPort );
	profileData->setLocalPort( Port );
	profileData->setUseSingleDes ( useSingleDes );
	profileData->setPerfectForwardSecrety ( PerfectForwardSecrecy );
	profileData->setIkeGroup( IkeGroup );
	profileData->setUseGlobalIpsecSecret (useGlobalIpsecSecret );
	profileData->setUseIkeGroup ( useIkeGroup );
	profileData->setNtDomainName (NtDomainName );
	profileData->setUseAdvancedSettings ( useAdvancedSettings );
	profileData->setUsePerfectForwardSecrety (usePerfectSecrecy );
	



	NetworkInterface iface(app);
	QStringList list = iface.getAllNetworkInterfaces();
	main->NetworkDeviceComboBox->insertItem( "default" );
	for ( QStringList::Iterator it = list.begin(); it != list.end(); ++it )
	{
		main->NetworkDeviceComboBox->insertItem( QString( *it ) );
	}
	main->NetworkDeviceComboBox->setEnabled( TRUE );
	main->NetworkDeviceComboBox->setCurrentText( "default" );

	main->certpathURLRequester->setURL( "/etc/racoon/certs" );

	main->ConnectionTypeComboBox->setCurrentItem( VpnAccountData::ConnectionType(VpnAccountData::other) ); // other
//	buttonOk->setEnabled( FALSE );

	main->ConnectionTypeComboBox->insertItem( i18n( "Cisco (vpnc)" ) );
	main->ConnectionTypeComboBox->insertItem( i18n( "IPSec (racoon)" ) );
	main->ConnectionTypeComboBox->insertItem( i18n( "IPSec (FreeS/WAN)" ) );
	main->ConnectionTypeComboBox->insertItem( i18n( "PPTP" ) );
	main->ConnectionTypeComboBox->insertItem( i18n( "OpenVPN" ) );
	//main->ConnectionTypeComboBox->insertItem( i18n( "-" ) );
	main->AuthTypeComboBox->setCurrentItem(VpnAccountData::ConnectionType(VpnAccountData::cisco)); // cisco

	authTypeChanged( "" );
	connectiontypeChanged( "" );
}

NewProfileDialog::~NewProfileDialog() {}

void NewProfileDialog::accept()
{
	bool canSave = TRUE;
	if ( main->ConnectionTypeComboBox->currentItem() == 0 )
		profileData->setConnectionType( VpnAccountData::cisco );
	else if ( main->ConnectionTypeComboBox->currentItem() == 1 )
	{
		profileData->setConnectionType( VpnAccountData::racoon );

		bool validAddr = TRUE;
		if ( main->RemoteNetAddrLineEdit->text().contains( '.' ) != 3 )
		{
			KMessageBox::error ( this, i18n( "No IP address for remote network entered!" ), i18n( "No IP address" ) );
			validAddr = canSave = FALSE;
		}
		else
		{
			if (!Utils().isValidIPv4Adress(main->RemoteNetAddrLineEdit->text()))
			{
				KMessageBox::error ( this, i18n( "IP address of remote network is invalid!" ), i18n( "Invalid IP address" ) );
				validAddr = canSave = FALSE;
			}
			if ( validAddr )
			{
				profileData->setRemoteNet( main->RemoteNetAddrLineEdit->text() + "/" +  main->RemoteNetMaskComboBox->currentText() );
			}

		}

		if (main->AuthTypeComboBox->currentItem() == 0)
			profileData->setAuthType( VpnAccountData::cert );
		else
			profileData->setAuthType( VpnAccountData::psk );

		profileData->setX509Certificate( main->x509certURLRequester->url() );
		profileData->setCertPath( main->certpathURLRequester->url() );

	}
	else if ( main->ConnectionTypeComboBox->currentItem() == 2 )
	{
		profileData->setConnectionType( VpnAccountData::freeswan );
	}
	else if ( main->ConnectionTypeComboBox->currentItem() == 3 )
	{
		profileData->setConnectionType( VpnAccountData::pptp );
	}
	else if ( main->ConnectionTypeComboBox->currentItem() == 4 )
	{
		profileData->setConnectionType( VpnAccountData::openvpn );
	}
	else
		profileData->setConnectionType( VpnAccountData::other );

	profileData->setSavePsk( main->CheckGroupPass->isChecked() );
	profileData->setSaveUserPassword( main->CheckUserPass->isChecked () );
	profileData->setID( main->IDLineEdit->text() );
	profileData->setUserName( main->UsernameLineEdit->text () );
	profileData->setUserPassword( main->PasswordEdit->text ( ) );
	profileData->setGateway( main->gatewayLineEdit->text() );
	profileData->setPreSharedKey( main->PSKLineEdit->text() );
	profileData->setX509Certificate( main->x509certURLRequester->url() );
	profileData->setCertPath( main->certpathURLRequester->url() );
	profileData->setNetworkDevice( main->NetworkDeviceComboBox->currentText() );
	if (main->AuthTypeComboBox->currentItem() == 0)
		profileData->setAuthType( VpnAccountData::cert );
	else
		profileData->setAuthType( VpnAccountData::psk );

	if ( canSave )
	{

		bool nameOk = TRUE;
		Name = main->ProfileLineEdit->text();

		if (Name.contains(' '))
		{
			KMessageBox::error ( this, i18n( "Profile name cant contain spaces!" ), i18n( "Spaces not allowed" ) );
			nameOk = FALSE;
		}

		if (Name.isEmpty())
		{
			KMessageBox::error ( this, i18n( "Profile name cant be empty!" ), i18n( "No name entered" ) );
			nameOk = FALSE;
		}

		VpnAccountData * it;
		for ( it = AccountList->first(); it; it = AccountList->next() )
		{
			if ( it->getName() == Name )
			{
				KMessageBox::error ( this, i18n( "Profile name exists!" ), i18n( "Name exist" ) );
				nameOk = FALSE;
				break;
			}
		}

		if ( nameOk )
		{
			profileData->setName(Name);
			AccountList->append( profileData );
			newProfileCreated = TRUE;
			QDialog::accept();
		}

	}
}

void NewProfileDialog::importCertificate()
{
	//FIXME make dynamiclly
	ImportCertificateDialog importdlg(  this, i18n("Import certificate..."), GlobalConfig, LogOutput );
	//int result =
	importdlg.exec();
}


void NewProfileDialog::connectiontypeChanged( const QString& )
{

	// First we should set a type
	//buttonOk->setEnabled( FALSE );
	main->AdvancedSettingsPushButton->setEnabled( FALSE );

	int item = main->ConnectionTypeComboBox->currentItem();

	if ( item == VpnAccountData::ConnectionType(VpnAccountData::cisco) )
	{
		LogOutput->append( i18n("New type: %1").arg("cisco") );
		
		profileData->setConnectionType( VpnAccountData::cisco );
		
		main->CheckGroupPass->setEnabled( TRUE );
		main->CheckUserPass->setEnabled( TRUE );

		main->IDLineEdit->setEnabled( TRUE );
		main->LabelID->setEnabled( TRUE );

		main->UsernameLineEdit->setEnabled( TRUE );
		main->LabelUsername->setEnabled( TRUE );

		main->PasswordEdit->setEnabled( TRUE );
		main->LabelPassword->setEnabled( TRUE );

		main->AdvancedSettingsPushButton->setEnabled( TRUE );

		main->x509certURLRequester->setEnabled( FALSE );
		main->LabelCertificate->setEnabled( FALSE );

		main->certpathURLRequester->setEnabled( FALSE );
		main->LabelCertificatePath->setEnabled( FALSE );

		main->NetworkDeviceComboBox->setEnabled( FALSE );
		main->LabelNetworkDevice->setEnabled( FALSE );
		main->AuthTypeComboBox->setEnabled( FALSE );
		main->LabelAuthType->setEnabled( FALSE );

		main->PSKLineEdit->setEnabled( FALSE );
		main->LabelPsk->setEnabled( FALSE );

		//buttonCancel->setEnabled( TRUE );
		//buttonOk->setEnabled( TRUE );

		main->RemoteNetAddrLineEdit->setEnabled( FALSE );
		main->LabelRemoteNetwork->setEnabled( FALSE );

		main->RemoteNetMaskComboBox->setEnabled( FALSE );

	} // vpnc

	else if ( item == VpnAccountData::ConnectionType(VpnAccountData::racoon) )
	{
		LogOutput->append( i18n("New type: %1").arg("racoon") );
		//     KURLRequester* main->x509certURLRequester;
		//     KComboBox* main->AuthTypeComboBox;
		//     KPushButton* ImportCertificatePushButton;
		//     KLineEdit* main->PSKLineEdit;

		profileData->setConnectionType( VpnAccountData::racoon );
		
		main->x509certURLRequester->setEnabled( TRUE );
		main->LabelCertificate->setEnabled( TRUE );

		main->certpathURLRequester->setEnabled( TRUE );
		main->LabelCertificatePath->setEnabled( TRUE );

		main->NetworkDeviceComboBox->setEnabled ( TRUE );
		main->LabelNetworkDevice->setEnabled( TRUE );

		main->AuthTypeComboBox->setEnabled( TRUE );
		main->LabelAuthType->setEnabled( TRUE );

		main->PSKLineEdit->setEnabled( TRUE );
		main->LabelPsk->setEnabled( TRUE );

		main->RemoteNetAddrLineEdit->setEnabled( TRUE );
		main->LabelRemoteNetwork->setEnabled( TRUE );

		main->RemoteNetMaskComboBox->setEnabled( TRUE );

		main->PasswordEdit->setEnabled( FALSE );
		main->LabelPassword->setEnabled( FALSE );

		main->CheckGroupPass->setEnabled( FALSE );
		main->CheckUserPass->setEnabled( FALSE );

		main->IDLineEdit->setEnabled( FALSE );
		main->LabelID->setEnabled( FALSE );

		main->UsernameLineEdit->setEnabled( FALSE );
		main->LabelUsername->setEnabled( FALSE );

		//buttonCancel->setEnabled( TRUE );
		//buttonOk->setEnabled( TRUE );
		main->AdvancedSettingsPushButton->setEnabled( TRUE );
	} // racoon

	else if ( item == VpnAccountData::ConnectionType(VpnAccountData::freeswan) )
	{
		LogOutput->append( i18n("New type: %1").arg("freeswan") );
		//     KURLRequester* main->x509certURLRequester;
		//     KComboBox* main->AuthTypeComboBox;
		//     KPushButton* ImportCertificatePushButton;
		//     KLineEdit* main->PSKLineEdit;

		profileData->setConnectionType( VpnAccountData::freeswan );
		
		main->x509certURLRequester->setEnabled( TRUE );
		main->LabelCertificate->setEnabled( TRUE );

		main->certpathURLRequester->setEnabled( TRUE );
		main->LabelCertificatePath->setEnabled( TRUE );

		main->NetworkDeviceComboBox->setEnabled ( TRUE );
		main->LabelNetworkDevice->setEnabled( TRUE );

		main->AuthTypeComboBox->setEnabled( TRUE );
		main->LabelAuthType->setEnabled( TRUE );

		main->PSKLineEdit->setEnabled( TRUE );
		main->LabelPsk->setEnabled( TRUE );

		main->RemoteNetAddrLineEdit->setEnabled( TRUE );
		main->LabelRemoteNetwork->setEnabled( TRUE );

		main->RemoteNetMaskComboBox->setEnabled( TRUE );

		main->CheckGroupPass->setEnabled( FALSE );
		main->CheckUserPass->setEnabled( FALSE );

		main->IDLineEdit->setEnabled( FALSE );
		main->LabelID->setEnabled( FALSE );

		main->PasswordEdit->setEnabled( FALSE );
		main->LabelPassword->setEnabled( FALSE );

		main->UsernameLineEdit->setEnabled( FALSE );
		main->LabelUsername->setEnabled( FALSE );

//		buttonCancel->setEnabled( TRUE );
//		buttonOk->setEnabled( TRUE );
		main->AdvancedSettingsPushButton->setEnabled( TRUE );
	} // freeswan

	else if ( item == VpnAccountData::ConnectionType(VpnAccountData::pptp) )
	{
		LogOutput->append( i18n("New type: %1").arg("pptp") );
		//     KURLRequester* main->x509certURLRequester;
		//     KComboBox* main->AuthTypeComboBox;
		//     KPushButton* ImportCertificatePushButton;
		//     KLineEdit* main->PSKLineEdit;

		profileData->setConnectionType( VpnAccountData::pptp );
		
		main->x509certURLRequester->setEnabled( FALSE );
		main->LabelCertificate->setEnabled( FALSE );

		main->certpathURLRequester->setEnabled( FALSE );
		main->LabelCertificatePath->setEnabled( FALSE );

		main->NetworkDeviceComboBox->setEnabled ( FALSE );
		main->LabelNetworkDevice->setEnabled( FALSE );

		main->AuthTypeComboBox->setEnabled( FALSE );
		main->LabelAuthType->setEnabled( FALSE );

		main->PSKLineEdit->setEnabled( FALSE );
		main->LabelPsk->setEnabled( FALSE );

		main->RemoteNetAddrLineEdit->setEnabled( FALSE );
		main->LabelRemoteNetwork->setEnabled( FALSE );

		main->RemoteNetMaskComboBox->setEnabled( FALSE );

		main->CheckGroupPass->setEnabled( FALSE );
		main->CheckUserPass->setEnabled( TRUE );

		main->IDLineEdit->setEnabled( FALSE );
		main->LabelID->setEnabled( FALSE );

		main->PasswordEdit->setEnabled( TRUE );
		main->LabelPassword->setEnabled( TRUE );

		main->UsernameLineEdit->setEnabled( TRUE );
		main->LabelUsername->setEnabled( TRUE );

		//buttonCancel->setEnabled( TRUE );
		//buttonOk->setEnabled( TRUE );
		main->AdvancedSettingsPushButton->setEnabled( TRUE );
	} // racoon
	else if ( item == VpnAccountData::ConnectionType(VpnAccountData::openvpn) )
	{
		LogOutput->append( i18n("New type: %1").arg("openvpn") );
		//     KURLRequester* main->x509certURLRequester;
		//     KComboBox* main->AuthTypeComboBox;
		//     KPushButton* ImportCertificatePushButton;
		//     KLineEdit* main->PSKLineEdit;

		profileData->setConnectionType( VpnAccountData::openvpn );
		
		main->x509certURLRequester->setEnabled( TRUE );
		main->LabelCertificate->setEnabled( TRUE );

		main->certpathURLRequester->setEnabled( TRUE );
		main->LabelCertificatePath->setEnabled( TRUE );

		main->NetworkDeviceComboBox->setEnabled ( TRUE );
		main->LabelNetworkDevice->setEnabled( TRUE );

		main->AuthTypeComboBox->setEnabled( TRUE );
		main->LabelAuthType->setEnabled( TRUE );

		main->PSKLineEdit->setEnabled( TRUE );
		main->LabelPsk->setEnabled( TRUE );

		main->RemoteNetAddrLineEdit->setEnabled( TRUE );
		main->LabelRemoteNetwork->setEnabled( TRUE );

		main->RemoteNetMaskComboBox->setEnabled( TRUE );

		main->PasswordEdit->setEnabled( FALSE );
		main->LabelPassword->setEnabled( FALSE );

		main->CheckGroupPass->setEnabled( FALSE );
		main->CheckUserPass->setEnabled( FALSE );

		main->IDLineEdit->setEnabled( FALSE );
		main->LabelID->setEnabled( FALSE );


		main->UsernameLineEdit->setEnabled( FALSE );
		main->LabelUsername->setEnabled( FALSE );

		//buttonCancel->setEnabled( TRUE );
		//buttonOk->setEnabled( TRUE );
		main->AdvancedSettingsPushButton->setEnabled( TRUE );
	} // openvpn

	// other
	else
	{
		LogOutput->append( "new type: other" );
		// 		main->x509certURLRequester->setEnabled( FALSE );
		// 		main->AuthTypeComboBox->setEnabled( FALSE );
		// 		main->PSKLineEdit->setEnabled( FALSE );
		//
		// 		main->CheckGroupPass->setEnabled( FALSE );
		// 		main->CheckUserPass->setEnabled( FALSE );
		// 		main->IDLineEdit->setEnabled( FALSE );
		// 		main->UsernameLineEdit->setEnabled( FALSE );
		// 		main->AdvancedSettingsPushButton->setEnabled( FALSE );
		//		buttonCancel->setEnabled( FALSE );
		//buttonOk->setEnabled( FALSE );
	}
}

void NewProfileDialog::authTypeChanged( const QString& )
{
	if ( main->AuthTypeComboBox->currentItem() == VpnAccountData::AuthenticationType(VpnAccountData::cert ))
	{
		// certificate
		main->PSKLineEdit->setEnabled( FALSE );
		main->LabelPsk->setEnabled( FALSE );

		main->x509certURLRequester->setEnabled( TRUE );
		main->LabelCertificate->setEnabled( TRUE );

		main->certpathURLRequester->setEnabled( TRUE );
		main->LabelCertificatePath->setEnabled( TRUE );
	}
	if ( main->AuthTypeComboBox->currentItem() == VpnAccountData::AuthenticationType(VpnAccountData::psk ))
	{
		// psk
		main->PSKLineEdit->setEnabled( TRUE );
		main->LabelPsk->setEnabled( TRUE );

		main->x509certURLRequester->setEnabled( FALSE );
		main->LabelCertificate->setEnabled( FALSE );

		main->certpathURLRequester->setEnabled( FALSE );
		main->LabelCertificatePath->setEnabled( FALSE );
	}

}

void NewProfileDialog::reject()
{
	QDialog::reject();
}

void NewProfileDialog::profileNameChanged( const QString& )
{
	// 	LogOutput->append ( "text changed!" );
// 	if ( main->ProfileLineEdit->text().isEmpty() )
// 		buttonOk->setEnabled( FALSE );
// 	else
// 		buttonOk->setEnabled( TRUE );
}

void NewProfileDialog::advancedSettingsClicked()
{
	VpnAccountData::ConnectionType connectionType;
	if (main->ConnectionTypeComboBox->currentItem() == VpnAccountData::ConnectionType(VpnAccountData::cisco))
		connectionType = VpnAccountData::ConnectionType(VpnAccountData::cisco);
	else if (main->ConnectionTypeComboBox->currentItem() == VpnAccountData::ConnectionType(VpnAccountData::racoon))
		connectionType = VpnAccountData::ConnectionType(VpnAccountData::racoon);
	else if (main->ConnectionTypeComboBox->currentItem() == VpnAccountData::ConnectionType(VpnAccountData::freeswan))
		connectionType = VpnAccountData::ConnectionType(VpnAccountData::freeswan);
	else if (main->ConnectionTypeComboBox->currentItem() == VpnAccountData::ConnectionType(VpnAccountData::pptp))
		connectionType = VpnAccountData::ConnectionType(VpnAccountData::pptp);
	else
		connectionType = VpnAccountData::ConnectionType(VpnAccountData::other);

	AdvancedSettingsDialog d1(this, i18n("Advanced settings..."),profileData,connectionType);

	d1.main->UseGlobalIpsecSecretCheckbox->setChecked ( profileData->getUseGlobalIpsecSecret() );
	d1.main->SingleDesCheckbox->setChecked( profileData->getUseSpecialRemoteID() );
	d1.main->IkeGroupCheckbox->setChecked ( profileData->getUseIkeGroup() );

	d1.main->PerfectForwardSecurityCheckbox->setChecked( profileData->getUsePerfectForwardSecrety() );
	d1.main->NtDomainNameCheckbox->setEnabled ( profileData->getUseNtDomainName() );
	d1.main->ApplicationVersionCheckbox->setEnabled ( profileData->getUseApplicationVersion() );
	d1.main->UseGlobalIpsecSecretCheckbox->setEnabled ( profileData->getUseGlobalIpsecSecret() );

	d1.main->LocalPortSpinbox->setValue( profileData->getLocalPort() );
	d1.main->IkeGroupCombobox->setCurrentText( profileData->getIkeGroup() );
	d1.main->PerfectForwardSecrecyCombobox->setCurrentText( profileData->getPerfectForwardSecrety() );
	d1.main->NtDomainNameLineedit->setText( profileData->getNtDomainName() );
	d1.main->ApplicationVersionLineedit->setText( profileData->getApplicationVersion() );

	if (profileData->connectiontype == VpnAccountData::pptp)
	{
		d1.main->RefuseEapCheckbox->setChecked( profileData->getRefuseEap() );
		d1.main->requireMppeCheckbox->setChecked( profileData->getRequireMppe() );
		d1.main->Refuse128BitEncryptionCheckbox->setChecked( profileData->getRefuse128BitEncryption() );
		d1.main->Refuse40BitEncryptionCheckbox->setChecked( profileData->getRefuse40BitEncryption() );
		d1.main->DnsServerCheckbox->setChecked( profileData->getUseDnsServer() );
		d1.main->DnsServerLineedit->setText( profileData->getDnsServer() );
	}
	d1.main->NtDomainNameCheckbox->setChecked( profileData->getUseNtDomainName() );
	d1.main->NtDomainNameLineedit->setText( profileData->getNtDomainName() );
	d1.main->AllowStatefulModeCheckbox->setChecked( profileData->getAllowStatefulMode() );
	d1.main->GetDnsFromPeerCheckbox->setChecked( profileData->getUseGetDnsFromPeer() );
	d1.main->AdvancedSettingsCheckbox->setChecked ( profileData->getUseAdvancedSettings());
	d1.main->UseNoBsdCompCheckbox->setChecked(profileData->getUseNoBsdComp());
	d1.main->UseNoDeflateCheckbox->setChecked(profileData->getUseNoDeflate());
	d1.main->SetDefaultRouteCheckbox->setChecked(profileData->getSetDefaultRoute());
	d1.main->ReplaceDefaultRouteCheckbox->setChecked(profileData->getReplaceDefaultRoute());

	if ( profileData->getUseAdvancedSettings() )
	{
		if (profileData->connectiontype == VpnAccountData::cisco)
		{
			d1.main->LocalPortCheckbox->setEnabled ( true );
			d1.main->PerfectForwardSecurityCheckbox->setEnabled ( true );
			d1.main->SingleDesCheckbox->setEnabled ( true );
			d1.main->NtDomainNameCheckbox->setEnabled ( true );
			d1.main->ApplicationVersionCheckbox->setEnabled ( true );
			d1.main->UseGlobalIpsecSecretCheckbox->setEnabled ( true );
		}

		else if (profileData->connectiontype == VpnAccountData::pptp)
		{
			d1.main->NtDomainNameCheckbox->setEnabled ( true );
			d1.main->RefuseEapCheckbox->setEnabled( true );
			d1.main->requireMppeCheckbox->setEnabled( true );
			d1.main->Refuse128BitEncryptionCheckbox->setEnabled( true );
			d1.main->Refuse40BitEncryptionCheckbox->setEnabled( true );
			d1.main->DnsServerCheckbox->setEnabled( true );
			d1.main->DnsServerLineedit->setEnabled( true );
			d1.main->NtDomainNameCheckbox->setEnabled( true );
			//d1.main->NtDomainNameLineedit->setEnabled( true );
			d1.main->AllowStatefulModeCheckbox->setEnabled( true );
			d1.main->GetDnsFromPeerCheckbox->setEnabled( true );
			d1.main->SetDefaultRouteCheckbox->setEnabled( true );
			if (profileData->getSetDefaultRoute())
				d1.main->ReplaceDefaultRouteCheckbox->setEnabled( true );
		}


		d1.main->IkeGroupCheckbox->setChecked ( profileData->getUseIkeGroup() );

		d1.main->SingleDesCheckbox->setChecked( profileData->getUseSingleDes() );

		d1.main->IkeGroupCheckbox->setChecked ( profileData->getUseIkeGroup() );

		d1.main->UseGlobalIpsecSecretCheckbox->setChecked ( profileData->getUseGlobalIpsecSecret() );

		d1.main->PerfectForwardSecurityCheckbox->setChecked( profileData->getUsePerfectForwardSecrety() );

		if ( profileData->getUseNtDomainName() )
		{
			d1.main->NtDomainNameLineedit->setEnabled ( true );
			d1.main->NtDomainNameCheckbox->setChecked( true );
		}

		if ( profileData->getUseApplicationVersion() )
		{
			d1.main->ApplicationVersionLineedit->setEnabled ( true );
			d1.main->ApplicationVersionCheckbox->setChecked( true );
		}

		if ( profileData->getUseLocalPort() )
		{
			d1.main->LocalPortSpinbox->setEnabled ( true );
			d1.main->LocalPortCheckbox->setChecked( true );
		}

		d1.main->IkeGroupCombobox->setEnabled (  profileData->getUseIkeGroup() );
		d1.main->UseNoBsdCompCheckbox->setChecked(profileData->getUseNoBsdComp());
		
		d1.main->UseNoDeflateCheckbox->setChecked(profileData->getUseNoDeflate());
	}

	d1.exec();
	d1.connectionType = connectionType;
}

QString NewProfileDialog::getName(){
	return Name;
}



#include "newprofiledialog.moc"
