/***************************************************************************
*   Copyright (C) 2004 by Christoph Thielecke                             *
*   crissi99@gmx.de                                                       *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include "preferencesdialog.h"
#include <qtabwidget.h>
#include <qpushbutton.h>
#include <klocale.h>
#include <qcheckbox.h>
#include <qspinbox.h>
#include <kurlrequester.h>
#include <kcolordialog.h>
#include <qlabel.h>

PreferencesDialog::PreferencesDialog( QWidget *parent, const QString& caption, QApplication *app,QTextEdit *LogOutput, KVpncConfig *GlobalConfig, VpnAccountData *currentProfile ) : KDialogBase( parent, i18n("Preferences"), true, caption,
		        KDialogBase::Ok|KDialogBase::Cancel, KDialogBase::Ok, true )
{
	main = new PreferencesDialogBase(this);
	setMainWidget(main);
	main->setMinimumSize(main->sizeHint());

	this->AccountList = GlobalConfig->AccountList;
	this->LogOutput = LogOutput;
	this->currentProfile = currentProfile;
	this->KvpncDebugLevel = GlobalConfig->KvpncDebugLevel;
	this->app = app;

	// 	main->ErrorLogLabel->setBackgroundMode(Qt::FixedColor );
	// 	main->ErrorLogLabel->setBackgroundMode(Qt::FixedColor );
	// 	main->ErrorLogLabel->setBackgroundMode(Qt::FixedColor );
	// 	main->ErrorLogLabel->setBackgroundMode(Qt::FixedColor );

	MyprofileWidget = new ProfileWidget(app, LogOutput, GlobalConfig, currentProfile, this );
	main->tabWidget3->addTab( MyprofileWidget, i18n( "Connection profiles" ) );
	//	MyprofileWidget->show();

	connect( main->ProgramsInPathcheckBox, SIGNAL ( toggled( bool ) ), this, SLOT( programsInPathToggled( bool ) ) );

	connect( main->UseColorizedLogOutputcheckBox, SIGNAL ( toggled( bool ) ), this, SLOT( useColorizedLogOutputToggled(bool)));

	connect( main->DebugLogFontPushButton , SIGNAL ( clicked() ), this, SLOT( debugLogColorClicked() ));

	connect( main->RemoteLogFontPushButton , SIGNAL ( clicked() ), this, SLOT( remoteLogColorClicked() ));

	connect( main->ErrorLogFontPushButton , SIGNAL ( clicked() ), this, SLOT( errorLogColorClicked() ));

	connect( main->InfoLogFontPushButton , SIGNAL ( clicked() ), this, SLOT( infoLogColorClicked() ));

	connect( main->SuccessLogFontPushButton , SIGNAL ( clicked() ), this, SLOT( successLogColorClicked() ));

	programsInPathToggled(GlobalConfig->programsInPath);
}

void PreferencesDialog::okClicked()
{
	MyprofileWidget->saveSessionClicked();
	close();
}

void PreferencesDialog::programsInPathToggled( bool )
{
	if ( main->ProgramsInPathcheckBox->isChecked() )
	{
		//main->spinVpncDebugLevel->setEnabled( false );
		main->VpncPathInput->setEnabled( false );
		main->FreeswanPathInput->setEnabled( false );
		//main->spinRacoonDebugLevel->setEnabled( false );
		main->SetkeyPathInput->setEnabled( false );
		main->RacoonPathInput->setEnabled( false );
		main->OpenSSLPathInput->setEnabled( false );
		main->IptablesPathInput->setEnabled( false );
		main->PppdPathInput->setEnabled( false );
	}
	else
	{
		//main->spinVpncDebugLevel->setEnabled( true );
		main->VpncPathInput->setEnabled( true );
		main->FreeswanPathInput->setEnabled( true );
		//main->spinRacoonDebugLevel->setEnabled( true );
		main->SetkeyPathInput->setEnabled( true );
		main->RacoonPathInput->setEnabled( true );
		main->OpenSSLPathInput->setEnabled( true );
		main->IptablesPathInput->setEnabled( true );
		main->PppdPathInput->setEnabled( true );
	}
}


void PreferencesDialog::errorLogColorClicked()
{
	int result = KColorDialog::getColor( ErrorLogColor );
	if ( result == KColorDialog::Accepted )
		main->ErrorLogLabel->setPaletteBackgroundColor (ErrorLogColor);
}

void PreferencesDialog::debugLogColorClicked()
{
	int result = KColorDialog::getColor( DebugLogColor );
	if ( result == KColorDialog::Accepted )
		main->DebugLogLabel->setPaletteBackgroundColor (DebugLogColor);
}


void PreferencesDialog::infoLogColorClicked()
{
	int result = KColorDialog::getColor( InfoLogColor );
	if ( result == KColorDialog::Accepted )
		main->InfoLogLabel->setPaletteBackgroundColor (InfoLogColor);
}

void PreferencesDialog::remoteLogColorClicked()
{
	int result = KColorDialog::getColor( RemoteLogColor );
	if ( result == KColorDialog::Accepted )
		main->RemoteLogLabel->setPaletteBackgroundColor (RemoteLogColor);
}

void PreferencesDialog::successLogColorClicked()
{
	int result = KColorDialog::getColor( SuccessLogColor );
	if ( result == KColorDialog::Accepted )
		main->SuccessLogLabel->setPaletteBackgroundColor (SuccessLogColor);
}

void PreferencesDialog::useColorizedLogOutputToggled(bool)
{
	if (main->UseColorizedLogOutputcheckBox->isChecked())
		main->ColorGroupBox->setEnabled(true);
	else
		main->ColorGroupBox->setEnabled(false);
}






#include "preferencesdialog.moc"
