/***************************************************************************
*   Copyright (C) 2004 by Christoph Thielecke                             *
*   crissi99@gmx.de                                                       *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
//BEGIN Includes
#include "profilewidget.h"
#include "importcertificatedialog.h"
#include <qlistview.h>
#include <klistview.h>
#include <kcombobox.h>
#include <klineedit.h>
#include <qcheckbox.h>
#include <qtabwidget.h>
#include <kurlrequester.h>
#include <kmessagebox.h>
#include <qlistbox.h>
#include <qcheckbox.h>
#include <qlabel.h>
#include <klineeditdlg.h>
#include <klocale.h>
#include <knuminput.h>
#include <kinputdialog.h>
#include <ktextedit.h>
#include "networkinterface.h"
#include "advancedsettingsdialog.h"
#include "newprofiledialog.h"
#include "utils.h"
#include "addnetworkroutedialog.h"
#include "kdialogbase.h"
#include <klistview.h>
#include <qstringlist.h>
//END Includes

ProfileWidget::ProfileWidget( QApplication *app, QTextEdit *LogOutput, KVpncConfig *GlobalConfig, VpnAccountData *currentProfile,QWidget * parent, const char * name, WFlags f ) : ProfileWidgetBase( parent, name, f)
{
	//resize(QSize(minimumSizeHint().width()+50, minimumSizeHint().height()+50));

	this->AccountList = GlobalConfig->AccountList;
	this->LogOutput = LogOutput;
	this->currentProfile = currentProfile;
	this->KvpncDebugLevel = GlobalConfig->KvpncDebugLevel;
	this->GlobalConfig = GlobalConfig;
	this->app = app;

	// TODO read types from list
	ConnectionTypeComboBox->insertItem( i18n( "Cisco (vpnc)" ), VpnAccountData::cisco );
	ConnectionTypeComboBox->insertItem( i18n( "IPSec (racoon)" ), VpnAccountData::racoon );
	ConnectionTypeComboBox->insertItem( i18n( "IPSec (FreeSWAN)" ), VpnAccountData::freeswan );
	ConnectionTypeComboBox->insertItem( i18n( "PPTP" ), VpnAccountData::pptp );
	ConnectionTypeComboBox->insertItem( i18n( "OpenVPN" ), VpnAccountData::openvpn );
	//ConnectionTypeComboBox->insertItem( "-", 5 );

	profileAddedOrDeleted = false;
	NewProfileName="";

	// profile data
	VpnAccountData * it;
	for ( it = AccountList->first(); it; it = AccountList->next() )
	{
		//KMessageBox::information(0,"item found",it->getName());

		ProfileListBox->insertItem( it->getName() );
	}

	connect( ProfileListBox, SIGNAL( highlighted ( QListBoxItem * ) ), this, SLOT( profileChanged( QListBoxItem * ) ) );

	connect( PingCheckBox, SIGNAL( toggled( bool ) ), this, SLOT( pingIpToggled( bool ) ) );

	connect( UseUdpCheckbox, SIGNAL( toggled( bool ) ), this, SLOT( useUdpToggled( bool ) ) );

	connect( UdpPortCheckbox, SIGNAL( toggled( bool ) ), this, SLOT( udpPortToggled( bool ) ) );

	connect( UseSpecialServerCertificateCheckBox, SIGNAL( toggled( bool ) ), this, SLOT( useSpecialServerCertificateToggled ( bool ) ) );

	if ( !AccountList->isEmpty() )
	{
		if ( currentProfile != 0 )
			ProfileListBox->setCurrentItem( ProfileListBox->findItem( currentProfile->getName() ) );
		else
		{
			ProfileListBox->setCurrentItem( 0 );
		}
	}
	else
	{
		AdvancedSettingsPushButton->setEnabled( false );
		RenameSessionPushButton->setEnabled( false );
		DeleteSessionPushButton->setEnabled( false );
		SaveSessionPushButton->setEnabled( false );
		CiscoPptpGroupBox->setEnabled( false );
		//		FreeswanRacoonGroupBox->setEnabled( false );
		OpenvpnGroupBox->setEnabled( false );
		GeneralGroupbox->setEnabled( false );
		CommandExecutionGroupBox->setEnabled( false );
	}


	NetworkInterface iface(app);
	QStringList list = iface.getAllNetworkInterfaces();
	NetworkDeviceComboBox->insertItem( "default" );
	for ( QStringList::Iterator it = list.begin(); it != list.end(); ++it )
	{
		NetworkDeviceComboBox->insertItem( QString( *it ) );
	}
	NetworkDeviceComboBox->setEnabled( true );
	LabelNetworkDevice->setEnabled( true );

	if ( currentProfile != 0 )
	{
		if ( currentProfile->getAuthType() == VpnAccountData::cert )
			AuthTypeComboBox->setCurrentText( i18n( "X.509 certificate" ) );

		if ( currentProfile->getAuthType() == VpnAccountData::psk )
			AuthTypeComboBox->setCurrentText( i18n( "Pre shared key" ) );

		if (currentProfile->getConnectionType() != VpnAccountData::ConnectionType(VpnAccountData::openvpn))
			PSKLineEdit->setText( currentProfile->getPreSharedKey() );
		else
			PSKFileURLRequester->setURL(currentProfile->getPreSharedKeyFile());

		if ( currentProfile->getDoPingIP() )
			PingIPLineEdit->setEnabled( true );
		else
			PingIPLineEdit->setEnabled( false );


		RemoteIDLineEdit->setEnabled( false );

		UseExtraNetworkRoutesCheckbox->setChecked(currentProfile->getUseAdditionalNetworkRoutes());

	NetworkListView->clear();
	QStringList AdditionalNetworkRoutes = currentProfile->getAdditionalNetworkRoutes();

	for (QStringList::Iterator it2 = AdditionalNetworkRoutes.begin() ; it2 != AdditionalNetworkRoutes.end(); ++it2 )
	{
		QString networkstring = *it2;
		NetworkListView->insertItem(new QListViewItem(NetworkListView,networkstring.section('/',0,0),networkstring.section('/',1,1)));

	}

	UseSpecialServerCertificateCheckBox->setChecked( currentProfile->getUseSpecialServerCertificate() );
	if ( currentProfile->getUseSpecialServerCertificate())
			SpecialServerCertificateURLRequester->setURL(currentProfile->getSpecialServerCertificate());

		connectionTypeChanged( ConnectionTypeComboBox->currentItem() );
		authTypeChanged( AuthTypeComboBox->currentItem() );
		profileChanged( ProfileListBox->item( ProfileListBox->currentItem() ) );
		
	}

	connect (ProfileListBox, SIGNAL(doubleClicked ( QListBoxItem *)),this,SLOT(doubleClicked(QListBoxItem *)));

	//FIXME -> not virtual IP's yet
	UseVirtualIPCheckBox->setChecked(false);
	UseVirtualIPCheckBox->setEnabled(false);
	RemoteVirtualIpLineEdit->setEnabled(false);
	LocalVirtualIpEdit->setEnabled(false);
	LabelRemoteVirtualIp->setEnabled(false);
	LabelLocalVirtualIp->setEnabled(false);

	//FIXME
	OpenvpnGroupBox->hide(); // no content
	
}


ProfileWidget::~ProfileWidget() {}


void ProfileWidget::importCertificate()
{
	//FIXME make path dynamiclly
	ImportCertificateDialog dlg( this, i18n("Import certificate..."),GlobalConfig, LogOutput );
	//int result =
	dlg.exec();
}

void ProfileWidget::profileChanged( QListBoxItem * )
{

	if (!AccountList->isEmpty() && currentProfile != 0)
	{

		//int item = ProfileListBox->currentItem();
		QString Name = ProfileListBox->currentText();
		if ( KvpncDebugLevel > 0 )
			LogOutput->append( i18n( "New profile: %1" ).arg( Name ) );

		//	KMessageBox::information(0,"item clicked",Name);
		// profile data
		VpnAccountData * it;
		for ( it = AccountList->first(); it; it = AccountList->next() )
		{
			if ( it->getName() == Name )
			{
				currentProfile = it;
				break;
			}
		}

		x509certURLRequester->clear();
		certpathURLRequester->clear();
		UsernameLineEdit->clear();
		PSKLineEdit->clear();
		RemoteIDLineEdit->clear();
		RemoteNetAddrLineEdit->clear();
		gatewayLineEdit->clear();
		NetworkListView->clear();
		SpecialServerCertificateURLRequester->clear();

		CheckSavePsk->setChecked( false );
		CheckUserPass->setChecked( false );
		IDLineEdit->clear();
		UsernameLineEdit->clear();
		PasswordEdit->clear();
		CmdConnectTextEdit->setEnabled(false);
		CmdDisconnectTextEdit->setEnabled(false);
		ExcuteCmdAfterConnectCheckBox->setChecked(false);
		ExcuteCmdAfterDisconnectCheckBox->setChecked(false);
		UseVirtualIPCheckBox->setChecked(false);
		UseSpecialServerCertificateCheckBox->setChecked(false);

		if ( currentProfile == 0 )
		{
			RenameSessionPushButton->setEnabled( false );
			DeleteSessionPushButton->setEnabled( false );
			SaveSessionPushButton->setEnabled( false );
			return;
		}
		else
		{
			AdvancedSettingsPushButton->setEnabled( true );
			DeleteSessionPushButton->setEnabled( true );
			SaveSessionPushButton->setEnabled( true );
		}

		RemoteIDLineEdit->clear();
		RemoteNetMaskComboBox->setCurrentText("24");

		gatewayLineEdit->setText(currentProfile->getGateway());

		// 	AdvancedSettingsPushButton->setEnabled( false );
		if ( ( currentProfile->getNetworkDevice() ).isEmpty() )
		{
			NetworkDeviceComboBox->setCurrentText( "default" );
			if ( KvpncDebugLevel > 0 )
				LogOutput->append( i18n( "profile \"%1\": no network device definied, using \"default\"." ).arg( currentProfile->getName() ) );
		}
		else
		{
			NetworkDeviceComboBox->setCurrentText( currentProfile->getNetworkDevice() );
			if ( KvpncDebugLevel > 0 )
				LogOutput->append( i18n( "profile \"%1\": network device definied, using \"%2\"." ).arg( currentProfile->getName() ).arg( currentProfile->getNetworkDevice() ) );
		}

		// global
		if ( it->getConnectionType() == VpnAccountData::ConnectionType(VpnAccountData::cisco ))
			ConnectionTypeComboBox->setCurrentItem( 0 );
		else if ( it->getConnectionType() == VpnAccountData::ConnectionType(VpnAccountData::racoon ))
			ConnectionTypeComboBox->setCurrentItem( 1 );
		else if ( it->getConnectionType() == VpnAccountData::ConnectionType(VpnAccountData::freeswan ))
			ConnectionTypeComboBox->setCurrentItem( 2 );
		else if ( it->getConnectionType() == VpnAccountData::ConnectionType(VpnAccountData::pptp ))
			ConnectionTypeComboBox->setCurrentItem( 3 );
		else if ( it->getConnectionType() == VpnAccountData::ConnectionType(VpnAccountData::openvpn ))
			ConnectionTypeComboBox->setCurrentItem( 4 );
		else
			ConnectionTypeComboBox->setCurrentItem( VpnAccountData::other ); // must be last item -> other

		gatewayLineEdit->setText( it->getGateway() );

		PSKLineEdit->setText( currentProfile->getPreSharedKey() );

		if ( it->getConnectionType() == VpnAccountData::cisco )
		{
			CheckSavePsk->setChecked( it->getSavePsk() );
			CheckUserPass->setChecked ( it->getSaveUserPassword() );
			IDLineEdit->setText( it->getID() );
			UsernameLineEdit->setText ( it->getUserName() );
			PSKLineEdit->setText( it->getPreSharedKey() );
			PasswordEdit->setText ( it->getUserPassword() );

			UseUdpCheckbox->setChecked(it->getUseUdp());
			UdpPortSpinbox->setValue(it->getUdpPort());
			if (it->getUseUdpPort() )
				UdpPortCheckbox->setChecked(true);
			else
				UdpPortCheckbox->setChecked(false);

			UseUdpCheckbox->setEnabled( true );
			UdpPortCheckbox->setEnabled( true );
			if (it->getUseUdp())
				UdpPortCheckbox->setEnabled(true);
			else
				UdpPortCheckbox->setEnabled(false);


			PSKLineEdit->setText( it->getPreSharedKey() );


			IDLineEdit->setEnabled( true );
			LabelID->setEnabled( true );
			AdvancedSettingsPushButton->setEnabled( true );

			RemoteNetAddrLineEdit->setEnabled( false );
			LabelRemoteNetwork->setEnabled( false );
			RemoteNetMaskComboBox->setEnabled( false );
			RemoteIDLineEdit->setEnabled( false );

		} // vpnc

		else if ( it->getConnectionType() == VpnAccountData::ConnectionType(VpnAccountData::racoon) || it->getConnectionType() == VpnAccountData::ConnectionType(VpnAccountData::freeswan ))
		{
			QString RemoteNetAddr = it->getRemoteNet().section( '/', 0, 0 );
			QString RemoteNetMask = it->getRemoteNet().section( '/', 1, 1 );
			RemoteNetAddrLineEdit->setText( RemoteNetAddr );
			RemoteNetMaskComboBox->setCurrentText( RemoteNetMask );
			if ( it->getAuthType() == VpnAccountData::cert )
			{
				AuthTypeComboBox->setCurrentItem( 0 );
			}
			else
			{
				AuthTypeComboBox->setCurrentItem( 1 );
			}


			x509certURLRequester->setURL( currentProfile->getX509Certificate() );
			CaCertpathURLRequester->setURL( currentProfile->getCaCertificate() );
			certpathURLRequester->setURL( currentProfile->getCertPath() );
			PrivkeyPasswordEdit->setText( currentProfile->getPrivateKeyPass());
			privkeypathURLRequester->setURL( currentProfile->getPrivateKey() );

			PingCheckBox->setChecked( currentProfile->getDoPingIP() );
			PingIPLineEdit->setText( currentProfile->getPingHostIP() );
			RemoteIDLineEdit->setText( currentProfile->getSpecialRemoteID() );
			UseSpecialRemoteIDcheckBox->setChecked( currentProfile->getUseSpecialRemoteID() );

			// TODO made it useable for racoon
			if (currentProfile->getConnectionType() == VpnAccountData::freeswan) {
				SpecialServerCertificateURLRequester->setEnabled(false);
				UseSpecialServerCertificateCheckBox->setChecked( 	currentProfile->getUseSpecialServerCertificate() );
				if ( currentProfile->getUseSpecialServerCertificate())
					SpecialServerCertificateURLRequester->setURL(currentProfile->getSpecialServerCertificate());
			}

			UseVirtualIPCheckBox->setChecked( currentProfile->getUseVirtualIP() );
			UseVirtualIPCheckBox->setEnabled( true);

			UseUdpCheckbox->setChecked(it->getUseUdp());
			UdpPortSpinbox->setValue(it->getUdpPort());
			if (it->getUseUdpPort() )
				UdpPortCheckbox->setChecked(true);
			else
				UdpPortCheckbox->setChecked(false);

			UseUdpCheckbox->setEnabled( true );
			UdpPortCheckbox->setEnabled( true );
			if (it->getUseUdp())
				UdpPortCheckbox->setEnabled(true);
			else
				UdpPortCheckbox->setEnabled(false);

			RemoteNetAddrLineEdit->setEnabled( true );
			LabelRemoteNetwork->setEnabled( true );
			RemoteNetMaskComboBox->setEnabled( true );

			ImportCertificatePushButton->setEnabled( true );

			IDLineEdit->setEnabled( false );
			LabelID->setEnabled( false );

			if (currentProfile->getConnectionType() == VpnAccountData::racoon)
				AdvancedSettingsPushButton->setEnabled( true );

		} // racoon or freeswan

		else if ( it->getConnectionType() == VpnAccountData::ConnectionType(VpnAccountData::pptp ))
		{
			CheckSavePsk->setChecked( it->getSavePsk() );
			CheckUserPass->setChecked ( it->getSaveUserPassword() );
			UsernameLineEdit->setText ( it->getUserName() );
			PasswordEdit->setText ( it->getUserPassword() );
			PingCheckBox->setChecked( currentProfile->getDoPingIP() );
			PingIPLineEdit->setText( currentProfile->getPingHostIP() );

			if (!it->getRemoteNet().isEmpty())
			{
				QString RemoteNetAddr = it->getRemoteNet().section( '/', 0, 0 );
				QString RemoteNetMask = it->getRemoteNet().section( '/', 1, 1 );
				RemoteNetAddrLineEdit->setText( RemoteNetAddr );
				RemoteNetMaskComboBox->setCurrentText( RemoteNetMask );
			}
			AdvancedSettingsPushButton->setEnabled( true );

			IDLineEdit->setEnabled( false );
			LabelID->setEnabled( false );

			RemoteNetAddrLineEdit->setEnabled( true );
			LabelRemoteNetwork->setEnabled( true );
			RemoteNetMaskComboBox->setEnabled(true );
			ImportCertificatePushButton->setEnabled( false );
			NetworkDeviceComboBox->setEnabled( false );
		} // pptp

		else if ( it->getConnectionType() == VpnAccountData::ConnectionType(VpnAccountData::openvpn ))
		{

			if ( it->getAuthType() == VpnAccountData::cert )
			{
				AuthTypeComboBox->setCurrentItem( 0 );
			}
			else
			{
				AuthTypeComboBox->setCurrentItem( 1 );
			}

			x509certURLRequester->setURL( currentProfile->getX509Certificate() );
			CaCertpathURLRequester->setURL( currentProfile->getCaCertificate() );
			certpathURLRequester->setURL( currentProfile->getCertPath() );
			PrivkeyPasswordEdit->setText( currentProfile->getPrivateKeyPass());
			privkeypathURLRequester->setURL( currentProfile->getPrivateKey() );
			PingCheckBox->setChecked( currentProfile->getDoPingIP() );
			PingIPLineEdit->setText( currentProfile->getPingHostIP() );

			LocalVirtualIpEdit->setText( currentProfile->getLocalVirtualIP());
			RemoteVirtualIpLineEdit->setText( currentProfile->getRemoteVirtualIP());

			if (!it->getRemoteNet().isEmpty())
			{
				QString RemoteNetAddr = it->getRemoteNet().section( '/', 0, 0 );
				QString RemoteNetMask = it->getRemoteNet().section( '/', 1, 1 );
				RemoteNetAddrLineEdit->setText( RemoteNetAddr );
				RemoteNetMaskComboBox->setCurrentText( RemoteNetMask );
			}
			RemoteNetMaskComboBox->setEnabled(true);
			RemoteNetAddrLineEdit->setEnabled(true);


			UseUdpCheckbox->setChecked(it->getUseUdp());
			UdpPortSpinbox->setValue(it->getUdpPort());
			if (it->getUseUdpPort() )
				UdpPortCheckbox->setChecked(true);
			else
				UdpPortCheckbox->setChecked(false);

			UseUdpCheckbox->setEnabled( true );
			UdpPortCheckbox->setEnabled( true );
			if (it->getUseUdp())
				UdpPortCheckbox->setEnabled(true);
			else
				UdpPortCheckbox->setEnabled(false);

			IDLineEdit->setEnabled( false );
			LabelID->setEnabled( false );
		} // openvpn

		ExcuteCmdAfterConnectCheckBox->setChecked(currentProfile->getExecuteCmdAfterConnect());
		ExcuteCmdAfterDisconnectCheckBox->setChecked(currentProfile->getExecuteCmdAfterDisconnect());
		CmdConnectTextEdit->setText(currentProfile->getCommandAfterConnect());
		CmdDisconnectTextEdit->setText (currentProfile->getCommandAfterDisconnect());
		CmdConnectTextEdit->setEnabled(currentProfile->getExecuteCmdAfterConnect());
		CmdDisconnectTextEdit->setEnabled(currentProfile->getExecuteCmdAfterDisconnect());

		UseExtraNetworkRoutesCheckbox->setChecked(currentProfile->getUseAdditionalNetworkRoutes());

		QStringList AdditionalNetworkRoutes = currentProfile->getAdditionalNetworkRoutes();

		for (QStringList::Iterator it2 = AdditionalNetworkRoutes.begin() ; it2 != AdditionalNetworkRoutes.end(); ++it2 )
		{
			QString networkstring = *it2;
			NetworkListView->insertItem(new QListViewItem(NetworkListView,networkstring.section('/',0,0),networkstring.section('/',1,1)));

		}

		connectionTypeChanged( ConnectionTypeComboBox->currentItem() );
		AuthTypeComboBox->setCurrentItem(currentProfile->getAuthType());
		x509certURLRequester->setFilter( "*.pem" );
		certpathURLRequester->setMode( KFile::Directory );
	}
}

void ProfileWidget::connectionTypeChanged( int )
{
	if ( KvpncDebugLevel > 0 )
		LogOutput->append( "connectionTypeChanged()" );

	if (currentProfile == 0)
		return;

	DeleteSessionPushButton->setEnabled ( TRUE );
	NewSessionPushButton->setEnabled ( TRUE );
	SaveSessionPushButton->setEnabled ( TRUE );

	// First we should set a type
	// 	parent->buttonCancel->setEnabled( FALSE );
	// 	parent->buttonOk->setEnabled( FALSE );
	AdvancedSettingsPushButton->setEnabled( false );

	if ( currentProfile->getConnectionType() ==  VpnAccountData::cisco )
	{
		if ( KvpncDebugLevel > 0 )
			LogOutput->append( "new type: cisco" );

		IDLineEdit->setEnabled( true );
		LabelID->setEnabled( true );

		AdvancedSettingsPushButton->setEnabled( true );

		NetworkDeviceComboBox->setEnabled( true );
		LabelNetworkDevice->setEnabled( true );
		RemoteNetAddrLineEdit->setEnabled( false );
		RemoteNetMaskComboBox->setEnabled( false );

		UseUdpCheckbox->setEnabled( true );
		UdpPortCheckbox->setEnabled( true );
		if (currentProfile->getUseUdp())
			UdpPortCheckbox->setEnabled(true);
		else
			UdpPortCheckbox->setEnabled(false);
		emit authTypeChanged(currentProfile->getAuthType());
	} // vpnc

	else if ( currentProfile->getConnectionType() == VpnAccountData::racoon || currentProfile->getConnectionType() == VpnAccountData::freeswan )
	{

		if ( currentProfile->getConnectionType() == VpnAccountData::racoon )
		{
			if ( KvpncDebugLevel > 0 )
				LogOutput->append( i18n( "New type: %1" ).arg("racoon") );

			AdvancedSettingsPushButton->setEnabled( true );
		}
		else
		{
			if ( KvpncDebugLevel > 0 )
				LogOutput->append( i18n( "New type: %1" ).arg("freeswan") );

			if (currentProfile != 0)
			{
				privkeypathURLRequester->setURL( currentProfile->getPrivateKey() );
				PrivkeyPasswordEdit->setText( currentProfile->getPrivateKeyPass() );
			}
			AdvancedSettingsPushButton->setEnabled( false );
		}
		NetworkDeviceComboBox->setEnabled ( true );
		LabelNetworkDevice->setEnabled( true );
		RemoteNetAddrLineEdit->setEnabled( true );
		RemoteNetMaskComboBox->setEnabled( true );
		IDLineEdit->setEnabled( false );
		LabelID->setEnabled( false );
		OpenvpnGroupBox->setEnabled( false );

		// 		buttonCancel->setEnabled( TRUE );
		// 		buttonOk->setEnabled( TRUE );
		if ( UseSpecialRemoteIDcheckBox->isChecked() )
			RemoteIDLineEdit->setEnabled( true );
		else
			RemoteIDLineEdit->setEnabled( false );

		UseUdpCheckbox->setEnabled( true );
		UdpPortCheckbox->setEnabled( true );
		if (currentProfile->getUseUdp())
			UdpPortCheckbox->setEnabled(true);
		else
			UdpPortCheckbox->setEnabled(false);

		emit authTypeChanged(currentProfile->getAuthType());
	} // racoon/freeswan

	else if ( currentProfile->getConnectionType() == VpnAccountData::pptp )
	{
		if ( KvpncDebugLevel > 0 )
			LogOutput->append( "new type: pptp" );

		AdvancedSettingsPushButton->setEnabled( true );
		OpenvpnGroupBox->setEnabled( false );

		authTypeChanged(VpnAccountData::pptp);
	} // pptp
	else if ( currentProfile->getConnectionType() == VpnAccountData::openvpn )
	{

		if ( KvpncDebugLevel > 0 )
			LogOutput->append( i18n( "new type: %1" ).arg("openvpn") );

		AdvancedSettingsPushButton->setEnabled( false );
		RemoteNetAddrLineEdit->setEnabled( true );
		LabelRemoteNetwork->setEnabled( true );
		RemoteNetMaskComboBox->setEnabled( true );
		IDLineEdit->setEnabled( false );
		LabelID->setEnabled( false );
		// 		buttonCancel->setEnabled( TRUE );
		// 		buttonOk->setEnabled( TRUE );

		OpenvpnGroupBox->setEnabled( true );
		UseUdpCheckbox->setEnabled( true );
		UdpPortCheckbox->setEnabled( true );
		if (currentProfile->getUseUdp())
			UdpPortCheckbox->setEnabled(true);
		else
			UdpPortCheckbox->setEnabled(false);

		emit authTypeChanged(currentProfile->getAuthType());
	} // openvpn

	// other
	else if ( currentProfile->getConnectionType() == VpnAccountData::other )
	{
		if ( KvpncDebugLevel > 0 )
			LogOutput->append( i18n( "new type: none specified" ) );
	}
	else
	{
		LogOutput->append( i18n( "unknown type" ) );
		OpenvpnGroupBox->setEnabled( false );
	}

}

void ProfileWidget::authTypeChanged( int authtype )
{
	if (currentProfile ==0)
		return;

	PskInFileCheckBox->setEnabled(true);

	if (GlobalConfig->KvpncDebugLevel > 0)
	{

		QString type;
		if ( currentProfile->getAuthType() == VpnAccountData::cert )
			type = "cert";
		else if ( currentProfile->getAuthType() == VpnAccountData::psk )
			type = "psk";


		LogOutput->append( i18n( "New authtype: \"%1\"." ).arg( type ) );
	}

	PasswordGroupBox->setEnabled(false);
	CertificateGroupBox->setEnabled( false );
	PskGroupBox->setEnabled( false );
	AuthTypeComboBox->setEnabled( false );
	LabelAuthType->setEnabled( false );
	PasswordGroupBox->setEnabled( false );
	CaCertpathURLRequester->setEnabled( false );

	int connectiontype = ConnectionTypeComboBox->currentItem();


	if (connectiontype == VpnAccountData::cisco || connectiontype == VpnAccountData::pptp )
	{
		PasswordGroupBox->setEnabled(true);
		if (connectiontype == VpnAccountData::cisco)
		{
			PskGroupBox->setEnabled( true );
			PSKLineEdit->setEnabled( true );
			LabelPsk->setEnabled( true );
			PskInFileCheckBox->setChecked(false);
			PskInFileCheckBox->setEnabled(false);
			PSKFileURLRequester->setEnabled(false);
			LabelPskFile->setEnabled(false);
		}

	}

	// racoon or freeswan
	else if ( connectiontype == VpnAccountData::freeswan || connectiontype == VpnAccountData::racoon )
	{

		AuthTypeComboBox->setEnabled( true );
		LabelAuthType->setEnabled( true );

		// cert
		if ( authtype == VpnAccountData::cert )
		{
			// certificate
			CertificateGroupBox->setEnabled( true );
			x509certURLRequester->setEnabled( true );
			LabelCertificate->setEnabled( true );

			certpathURLRequester->setEnabled( true );
			LabelCertificatePath->setEnabled( true );

			ImportCertificatePushButton->setEnabled( true );

			PskGroupBox->setEnabled( false );
			PSKLineEdit->setEnabled( false );
			LabelPsk->setEnabled( false );

		}

		// psk
		if ( authtype == VpnAccountData::psk )
		{
			// psk
			PskGroupBox->setEnabled( true );
			PSKLineEdit->setEnabled( true );
			LabelPsk->setEnabled( true );

			CertificateGroupBox->setEnabled( false );
			x509certURLRequester->setEnabled( false );
			LabelCertificate->setEnabled( false );

			certpathURLRequester->setEnabled( false );
			LabelCertificatePath->setEnabled( false );

			ImportCertificatePushButton->setEnabled( false );

		}
	}
	else if (connectiontype == VpnAccountData::openvpn)
	{

		AuthTypeComboBox->setEnabled( true );
		LabelAuthType->setEnabled( true );
		// cert
		if ( authtype  == VpnAccountData::cert )
		{
			// certificate
			CertificateGroupBox->setEnabled( true );
			x509certURLRequester->setEnabled( true );
			CaCertpathURLRequester->setEnabled( true );
			LabelCertificate->setEnabled( true );

			certpathURLRequester->setEnabled( true );
			LabelCertificatePath->setEnabled( true );
			privkeypathURLRequester->setEnabled( true );
			LabelPrivateKeyPath->setEnabled( true );

			ImportCertificatePushButton->setEnabled( true );

			PskGroupBox->setEnabled( false );
			PSKFileURLRequester->setEnabled( false );
			LabelPrivateKeyPath->setEnabled( true );
		}

		// psk
		if ( authtype == VpnAccountData::psk)
		{
			// psk
			PskGroupBox->setEnabled( true );
			PSKFileURLRequester->setEnabled( true );
			PSKLineEdit->setEnabled( true );
			LabelPsk->setEnabled( true );

			CertificateGroupBox->setEnabled( false );
			LabelPrivateKeyPath->setEnabled( false );
			x509certURLRequester->setEnabled( false );
			LabelCertificate->setEnabled( false );

			certpathURLRequester->setEnabled( false );
			LabelCertificatePath->setEnabled( false );
			ImportCertificatePushButton->setEnabled( false );
		}

	}
}

void ProfileWidget::newSessionClicked()
{

	// call new profile dialog...
	NewProfileDialog newdlg( app,this, i18n("New profile"),GlobalConfig, LogOutput );
	newdlg.exec();
	if ( newdlg.newProfileCreated )
	{
		QString Name = newdlg.profileData->getName();
		VpnAccountData * it;
		for ( it = AccountList->first(); it; it = AccountList->next() )
		{
			if ( it->getName() == Name )
			{
				ProfileListBox->insertItem( Name, 0 );
				ProfileListBox->setCurrentItem ( 0 );
				currentProfile = it;
				break;
			}
		}

		AdvancedSettingsPushButton->setEnabled( TRUE );
		RenameSessionPushButton->setEnabled( TRUE );
		DeleteSessionPushButton->setEnabled( TRUE );
		SaveSessionPushButton->setEnabled ( TRUE );

		CiscoPptpGroupBox->setEnabled( true );
		//		FreeswanRacoonGroupBox->setEnabled( true );
		OpenvpnGroupBox->setEnabled( true );
		GeneralGroupbox->setEnabled( true );
		CommandExecutionGroupBox->setEnabled( true );

		KMessageBox::information ( this, i18n( "Profile \"%1\" added.\nNow set the right type,\n fill in the fields\nand press \"Save\" to complete the profile." ).arg( Name ), i18n( "Complete profile" ) );
		//slotStatusMsg ( i18n( "Profile \"%1\" added." ).arg( Name ), ID_FLASH_MSG );
		LogOutput->append ( i18n( "Profile \"%1\" added." ).arg( Name ) );
		saveSessionClicked();
		profileAddedOrDeleted=true;
		NewProfileName=Name;
	}
	else
	{
		LogOutput->append ( i18n( "Creating new profile canceled." ) );
	}

}


void ProfileWidget::saveSessionClicked()
{
	if ( currentProfile== 0 )
		return;
	//QString itemString=ConnectionTypeComboBox->currentText();
	int item = ConnectionTypeComboBox->currentItem();
	// cisco
	if ( item == VpnAccountData::cisco ){
		currentProfile->setConnectionType( VpnAccountData::cisco );
		if (CheckUserPass->isChecked()) {
			currentProfile->setSaveUserPassword(true);
			currentProfile->setUserPassword(UsernameLineEdit->text());
		}
		else {
			currentProfile->setSaveUserPassword(false);
		}

		if (CheckSavePsk->isChecked()) {
			currentProfile->setSavePsk(true);
			currentProfile->setPreSharedKey(PSKLineEdit->text());
		}
		else {
			currentProfile->setSavePsk(false);
		}
	}

	else if ( item == VpnAccountData::racoon || item == VpnAccountData::freeswan )
	{
		currentProfile->setSpecialRemoteID( RemoteIDLineEdit->text() );
		currentProfile->setUseSpecialRemoteID( UseSpecialRemoteIDcheckBox->isChecked() );

		if ( item == VpnAccountData::racoon )
			currentProfile->setConnectionType( VpnAccountData::racoon );
		else
			currentProfile->setConnectionType( VpnAccountData::freeswan );

		currentProfile->setPrivateKey( privkeypathURLRequester->url() );
		currentProfile->setPrivateKeyPass( PrivkeyPasswordEdit->text() );


		bool validAddr = true;
		if ( RemoteNetAddrLineEdit->text().contains( '.' ) != 3 )
		{
			KMessageBox::error ( this, i18n( "No IP address entered!" ), i18n( "No IP address" ) );
			validAddr = false;
		}
		else
		{
			QString addr = RemoteNetAddrLineEdit->text();
			int part0 = addr.section( '.', 0, 0 ).toInt();
			int part1 = addr.section( '.', 1, 1 ).toInt();
			int part2 = addr.section( '.', 2, 2 ).toInt();
			int part3 = addr.section( '.', 3, 3 ).toInt();

			/*
						LogOutput->append ("part0: "+QString().setNum(part0));
						LogOutput->append ("part1: "+QString().setNum(part1));
						LogOutput->append ("part2: "+QString().setNum(part2));
						LogOutput->append ("part3: "+QString().setNum(part3));
			*/
			if ( ( part0 < 1 || part0 > 254 ) || ( part1 < 0 || part1 > 254 ) || ( part2 < 0 || part2 > 254 ) || ( part3 < 0 || part3 > 254 ) )
			{
				KMessageBox::error ( this, i18n( "IP address (remote net) is invalid!" ), i18n( "Invalid IP address" ) );
				validAddr = false;
			}
			if ( validAddr == true )
			{
				currentProfile->setRemoteNet( addr + "/" + RemoteNetMaskComboBox->currentText() );
			}

		}

		if ( AuthTypeComboBox->currentItem() == 0 )
			currentProfile->setAuthType( VpnAccountData::cert );
		else
			currentProfile->setAuthType( VpnAccountData::psk );

		currentProfile->setX509Certificate( x509certURLRequester->url() );
		currentProfile->setCertPath( certpathURLRequester->url() );

		currentProfile->setPreSharedKey( PSKLineEdit->text());


	}

	else if ( item == VpnAccountData::pptp )
	{
		currentProfile->setConnectionType( VpnAccountData::pptp );


		bool validAddr = true;
		if (!RemoteNetAddrLineEdit->text().isEmpty())
		{
			if ( RemoteNetAddrLineEdit->text().contains( '.' ) != 3 )
			{
				//KMessageBox::error ( this, i18n( "No IP address entered!" ), i18n( "No IP address" ) );
				validAddr = false;
			}
			else
			{
				QString addr = RemoteNetAddrLineEdit->text();
				int part0 = addr.section( '.', 0, 0 ).toInt();
				int part1 = addr.section( '.', 1, 1 ).toInt();
				int part2 = addr.section( '.', 2, 2 ).toInt();
				int part3 = addr.section( '.', 3, 3 ).toInt();

				/*
							LogOutput->append ("part0: "+QString().setNum(part0));
							LogOutput->append ("part1: "+QString().setNum(part1));
							LogOutput->append ("part2: "+QString().setNum(part2));
							LogOutput->append ("part3: "+QString().setNum(part3));
				*/

				if ( ( part0 < 1 || part0 > 254 ) || ( part1 < 0 || part1 > 254 ) || ( part2 < 0 || part2 > 254 ) || ( part3 < 0 || part3 > 254 ) )
				{
					KMessageBox::error ( this, i18n( "Invalid values in IP address (remote net)!" ), i18n( "Invalid values in IP address" ) );
					validAddr = false;
				}


				if ( validAddr == true )
				{
					currentProfile->setRemoteNet( addr + "/" + RemoteNetMaskComboBox->currentText() );
				}

			}
		}
		else
		{
			currentProfile->setRemoteNet("");
		}

	}

	else if ( item == VpnAccountData::ConnectionType(VpnAccountData::openvpn) )
	{
		currentProfile->setConnectionType( VpnAccountData::openvpn );

		bool validAddr = true;
		if ( RemoteNetAddrLineEdit->text().contains( '.' ) != 3 )
		{
			KMessageBox::error ( this, i18n( "No IP address entered!" ), i18n( "No IP address" ) );
			validAddr = false;
		}
		else
		{
			QString addr = RemoteNetAddrLineEdit->text();
			int part0 = addr.section( '.', 0, 0 ).toInt();
			int part1 = addr.section( '.', 1, 1 ).toInt();
			int part2 = addr.section( '.', 2, 2 ).toInt();
			int part3 = addr.section( '.', 3, 3 ).toInt();

			/*
						LogOutput->append ("part0: "+QString().setNum(part0));
						LogOutput->append ("part1: "+QString().setNum(part1));
						LogOutput->append ("part2: "+QString().setNum(part2));
						LogOutput->append ("part3: "+QString().setNum(part3));
			*/
			if ( ( part0 < 1 || part0 > 254 ) || ( part1 < 0 || part1 > 254 ) || ( part2 < 0 || part2 > 254 ) || ( part3 < 0 || part3 > 254 ) )
			{
				KMessageBox::error ( this, i18n( "IP address (remote net) is invalid!" ), i18n( "Invalid IP address" ) );
				validAddr = false;
			}
			if ( validAddr == true )
			{
				currentProfile->setRemoteNet( addr + "/" + RemoteNetMaskComboBox->currentText() );
			}

		}

		// 		QString LocalVirtualIp = LocalVirtualIpEdit->text();
		// 		if (Utils().isValidIPv4Adress(LocalVirtualIp) == false)
		// 			KMessageBox::error ( this, i18n( "No valid IP address entered!" ), i18n( "No valid IP address" ) );
		// 		else
		// 		{
		// 			currentProfile->setLocalVirtualIP( LocalVirtualIp);
		// 		}
		//
		// 		QString RemoteVirtualIp = RemoteVirtualIpLineEdit->text();
		// 		if (Utils().isValidIPv4Adress(RemoteVirtualIp) == false)
		// 			KMessageBox::error ( this, i18n( "No valid IP address entered!" ), i18n( "No valid IP address" ) );
		// 		else
		// 		{
		// 			currentProfile->setRemoteVirtualIP( RemoteVirtualIp);
		// 		}


		if ( AuthTypeComboBox->currentItem() == 0 )
			currentProfile->setAuthType( VpnAccountData::cert );
		else
			currentProfile->setAuthType( VpnAccountData::psk );


		currentProfile->setPreSharedKeyFile(PSKFileURLRequester->url());
		currentProfile->setX509Certificate( x509certURLRequester->url() );
		currentProfile->setCaCertificate( CaCertpathURLRequester->url() );
		currentProfile->setCertPath( certpathURLRequester->url() );
		currentProfile->setPrivateKey ( privkeypathURLRequester->url());
		currentProfile->setPrivateKeyPass ( PrivkeyPasswordEdit->text());



	} // openvpn

	else
		currentProfile->setConnectionType( VpnAccountData::other );

	if ( PingCheckBox->isChecked() && !PingIPLineEdit->text().isEmpty() )
	{
		// do check for vailid IP
		if (!Utils().isValidIPv4Adress(PingIPLineEdit->text()))
			KMessageBox::error ( this, i18n( "No valid IP address entered!" ), i18n( "No valid IP address" ) );
		else
		{
			currentProfile->setPingHostIP( PingIPLineEdit->text() );
			currentProfile->setDoPingIP( PingCheckBox->isChecked());
		}
	}
	else
	{
		if ( PingCheckBox->isChecked() && PingIPLineEdit->text() == QString( "" ) )
			KMessageBox::error ( this, i18n( "No IP address (ping host) entered!" ), i18n( "No IP address" ) );
		else
			currentProfile->setPingHostIP( "" );
	}

	if ( item == VpnAccountData::ConnectionType(VpnAccountData::freeswan) ||
	        item == VpnAccountData::ConnectionType(VpnAccountData::racoon) )
	{

		if ( UseVirtualIPCheckBox->isChecked() && !LocalVirtualIpEdit->text().isEmpty() )
		{
			// do check for vailid IP
			if (!Utils().isValidIPv4Adress(LocalVirtualIpEdit->text()))
				KMessageBox::error ( this, i18n( "No valid IP address entered!" ), i18n( "No valid IP address" ) );
			else
			{
				currentProfile->setVirtualIP( LocalVirtualIpEdit->text() );
				currentProfile->setUseVirtualIP( UseVirtualIPCheckBox->isChecked());
			}
		}
		else
		{
			if ( UseVirtualIPCheckBox->isChecked() && LocalVirtualIpEdit->text() == QString( "" ) )
				KMessageBox::error ( this, i18n( "No IP address (virtual IP) entered!" ), i18n( "No IP address" ) );
			else
			{
				currentProfile->setVirtualIP( "" );
				currentProfile->setUseVirtualIP(false);
			}
		}
	}

	currentProfile->setSavePsk( CheckSavePsk->isChecked() );
	currentProfile->setSaveUserPassword( CheckUserPass->isChecked () );
	currentProfile->setID( IDLineEdit->text() );
	currentProfile->setUserName( UsernameLineEdit->text () );

	if (CheckSavePsk->isChecked())
		currentProfile->setPreSharedKey( PSKLineEdit->text( ) );
	else
		currentProfile->setPreSharedKey( "" );

	if (CheckUserPass->isChecked())
		currentProfile->setUserPassword( PasswordEdit->text ( ) );
	else
		currentProfile->setUserPassword( "" );

	currentProfile->setGateway( gatewayLineEdit->text() );
	currentProfile->setPreSharedKey( PSKLineEdit->text() );
	currentProfile->setX509Certificate( x509certURLRequester->url() );
	currentProfile->setCertPath( certpathURLRequester->url() );
	currentProfile->setNetworkDevice( NetworkDeviceComboBox->currentText() );

	currentProfile->setDoPingIP( PingCheckBox->isChecked() );

	if ( AuthTypeComboBox->currentItem() == 0 )
		currentProfile->setAuthType( VpnAccountData::cert );
	if ( AuthTypeComboBox->currentItem() == 1 )
		currentProfile->setAuthType( VpnAccountData::psk );

	if (UseUdpCheckbox->isChecked())
		currentProfile->setUseUdp(true);
	else
		currentProfile->setUseUdp(false);

	if (UdpPortCheckbox->isChecked())
	{
		currentProfile->setUdpPort(UdpPortSpinbox->value());
		currentProfile->setUseUdpPort(true);
	}
	else
	{
		currentProfile->setUdpPort(0);
		currentProfile->setUseUdpPort(false);
	}



	currentProfile->setExecuteCmdAfterConnect( (ExcuteCmdAfterConnectCheckBox->isChecked()));
	currentProfile->setExecuteCmdAfterDisconnect( (ExcuteCmdAfterDisconnectCheckBox->isChecked()));

	currentProfile->setCommandAfterConnect(CmdConnectTextEdit->text());
	currentProfile->setCommandAfterDisconnect(CmdDisconnectTextEdit->text());

	currentProfile->setUseAdditionalNetworkRoutes(UseExtraNetworkRoutesCheckbox->isChecked());

	if (UseExtraNetworkRoutesCheckbox->isChecked() && NetworkListView->childCount() > 0)
	{
		QStringList AdditionalNetworkRoutes;
		QListViewItemIterator it( NetworkListView );
		for ( ; it.current(); ++it )
		{
			QString network = QString(it.current()->text(0)+"/"+it.current()->text(1));
			AdditionalNetworkRoutes.append(network);
		}
		currentProfile->setAdditionalNetworkRoutes(AdditionalNetworkRoutes);
	}

		currentProfile->setUseSpecialServerCertificate(UseSpecialServerCertificateCheckBox->isChecked());
		if ( UseSpecialServerCertificateCheckBox->isChecked()){
			if (!SpecialServerCertificateURLRequester->url().isEmpty())
				currentProfile->setSpecialServerCertificate(SpecialServerCertificateURLRequester->url());
			else
				KMessageBox::error ( this, i18n( "Special server certificate can't be empty!" ), i18n( "No server certificate" ) );
		}

	LogOutput->append( i18n( "Profile \"%1\" saved." ).arg( currentProfile->getName() ) );
}


void ProfileWidget::deleteSessionClicked()
{
	QString Name = ProfileListBox->currentText();
	if ( !ProfileListBox->currentText().isEmpty() )
	{
		int result = KMessageBox::questionYesNo ( this, i18n( "Really delete \"%1\"?" ).arg( Name ), i18n( "Delete?" ) );
		//LogOutput->append( "Result: "+ QString().setNum(result) ) ;
		if ( result == 3) // Yes
		{


			VpnAccountData * it;
			if (!AccountList->isEmpty()) {
				for ( it = AccountList->first(); it; it = AccountList->next() )
				{
					if ( ProfileListBox->currentText().isEmpty() )
					{
						// 					this->deleteSessionAction->setEnabled ( FALSE );
						//			this->advancedSessionAction->setEnabled ( FALSE );
						// 					this->saveSessionAction->setEnabled ( FALSE );
					}
					else
					{
						if ( it->getName() == Name )
						{
							currentProfile=0L;
							AccountList->remove( it );
							ProfileListBox->removeItem( ProfileListBox->currentItem() );
							break;
						}
					}
				}
			}

			if ( !AccountList->isEmpty() )
			{
				currentProfile=AccountList->first();
				ProfileListBox->setCurrentItem( 0 );
				
				connectionTypeChanged( ConnectionTypeComboBox->currentItem() );
				authTypeChanged( AuthTypeComboBox->currentItem() );
				ProfileListBox->setCurrentItem(ProfileListBox->findItem(GlobalConfig->AccountList->first()->getName()));
				ProfileListBox->setSelected(0, true);
				profileChanged ( ProfileListBox->selectedItem ());
				// 				buttonCancel->setEnabled( true );
				// 				buttonOk->setEnabled( true );
			}
			else
			{
				this->DeleteSessionPushButton->setEnabled ( FALSE );
				this->RenameSessionPushButton->setEnabled ( FALSE );
				this->SaveSessionPushButton->setEnabled ( FALSE );
				AdvancedSettingsPushButton->setEnabled( false );
				CiscoPptpGroupBox->setEnabled( false );
				//				FreeswanRacoonGroupBox->setEnabled( false );
				OpenvpnGroupBox->setEnabled( false );
				GeneralGroupbox->setEnabled( false );
				CommandExecutionGroupBox->setEnabled( false );
			}

			//		slotStatusMsg ( i18n( "Profile \"%1\" deleted." ).arg( Name ), ID_FLASH_MSG );
			LogOutput->append ( i18n( "Profile \"%1\" deleted." ).arg( Name ) );
			saveSessionClicked();
			profileAddedOrDeleted=true;
			//saveOptions(); // TODO save, currently after closing dialog
			//			sessionToggled();
			//if ( ProfileListBox->currentText().isEmpty() )
			//	ProfileListBox->setEnabled ( FALSE );
		}
	}

}

void ProfileWidget::slotCancelClicked()
{
	cancelClicked = true;
}

void ProfileWidget::advancedSettingsClicked()
{
	if (currentProfile ==0)
		return;

	AdvancedSettingsDialog d1(this, i18n("Advanced settings..."),currentProfile,currentProfile->getConnectionType());

	d1.main->LocalPortCheckbox->setEnabled( false );
	d1.main->IkeGroupCheckbox->setEnabled( false );
	d1.main->PerfectForwardSecrecyCombobox->setEnabled( false );
	d1.main->SingleDesCheckbox->setEnabled( false );
	d1.main->NtDomainNameCheckbox->setEnabled( false );
	d1.main->LocalPortSpinbox->setEnabled( false );
	d1.main->ApplicationVersionLineedit->setEnabled( false );
	d1.main->NtDomainNameLineedit->setEnabled( false );
	d1.main->PerfectForwardSecurityCheckbox->setEnabled( false );
	d1.main->IkeGroupCombobox->setEnabled( false );
	d1.main->ApplicationVersionCheckbox->setEnabled( false );
	d1.main->UseGlobalIpsecSecretCheckbox->setEnabled( false );
	d1.main->RefuseEapCheckbox->setEnabled( false );
	d1.main->requireMppeCheckbox->setEnabled( false );
	d1.main->Refuse128BitEncryptionCheckbox->setEnabled(false );
	d1.main->Refuse40BitEncryptionCheckbox->setEnabled( false );
	d1.main->DnsServerCheckbox->setEnabled( false );
	d1.main->DnsServerLineedit->setEnabled( false );
	d1.main->AllowStatefulModeCheckbox->setEnabled( false );
	d1.main->GetDnsFromPeerCheckbox->setEnabled( false );
	d1.main->IkeGroupCheckbox->setEnabled( false );

	d1.main->SetDefaultRouteCheckbox->setEnabled( false );
	d1.main->ReplaceDefaultRouteCheckbox->setEnabled( false );
	d1.main->MtuCheckbox->setEnabled(false);
	d1.main->MtuSpinbox->setEnabled(false);


	d1.main->UseGlobalIpsecSecretCheckbox->setChecked ( currentProfile->getUseGlobalIpsecSecret() );
	d1.main->SingleDesCheckbox->setChecked( currentProfile->getUseSpecialRemoteID() );
	d1.main->IkeGroupCheckbox->setChecked ( currentProfile->getUseIkeGroup() );

	d1.main->PerfectForwardSecurityCheckbox->setChecked( currentProfile->getUsePerfectForwardSecrety() );
	d1.main->NtDomainNameCheckbox->setEnabled ( currentProfile->getUseNtDomainName() );
	d1.main->ApplicationVersionCheckbox->setEnabled ( currentProfile->getUseApplicationVersion() );
	d1.main->UseGlobalIpsecSecretCheckbox->setEnabled ( currentProfile->getUseGlobalIpsecSecret() );

	d1.main->LocalPortSpinbox->setValue( currentProfile->getLocalPort() );
	if (!currentProfile->getIkeGroup().isEmpty())
		d1.main->IkeGroupCombobox->setCurrentText( currentProfile->getIkeGroup() );
	d1.main->PerfectForwardSecrecyCombobox->setCurrentText( currentProfile->getPerfectForwardSecrety() );
	d1.main->NtDomainNameLineedit->setText( currentProfile->getNtDomainName() );
	d1.main->ApplicationVersionLineedit->setText( currentProfile->getApplicationVersion() );

	if (currentProfile->connectiontype == VpnAccountData::pptp)
	{
		d1.main->RefuseEapCheckbox->setChecked( currentProfile->getRefuseEap() );
		d1.main->requireMppeCheckbox->setChecked( currentProfile->getRequireMppe() );
		d1.main->Refuse128BitEncryptionCheckbox->setChecked( currentProfile->getRefuse128BitEncryption() );
		d1.main->Refuse40BitEncryptionCheckbox->setChecked( currentProfile->getRefuse40BitEncryption() );
		d1.main->DnsServerCheckbox->setChecked( currentProfile->getUseDnsServer() );
		d1.main->DnsServerLineedit->setText( currentProfile->getDnsServer() );
		d1.main->MtuCheckbox->setChecked(currentProfile->getUseMtu());
		d1.main->MtuSpinbox->setValue(currentProfile->getMtu());
	}
	d1.main->NtDomainNameCheckbox->setChecked( currentProfile->getUseNtDomainName() );
	d1.main->NtDomainNameLineedit->setText( currentProfile->getNtDomainName() );
	d1.main->AllowStatefulModeCheckbox->setChecked( currentProfile->getAllowStatefulMode() );
	d1.main->GetDnsFromPeerCheckbox->setChecked( currentProfile->getUseGetDnsFromPeer() );
	d1.main->AdvancedSettingsCheckbox->setChecked ( currentProfile->getUseAdvancedSettings());
	d1.main->SetDefaultRouteCheckbox->setChecked(currentProfile->getSetDefaultRoute());
	d1.main->ReplaceDefaultRouteCheckbox->setChecked(currentProfile->getReplaceDefaultRoute());

	if ( currentProfile->getUseAdvancedSettings() )
	{
		if (currentProfile->connectiontype == VpnAccountData::cisco)
		{
			d1.main->LocalPortCheckbox->setEnabled ( true );
			d1.main->PerfectForwardSecurityCheckbox->setEnabled ( true );
			d1.main->SingleDesCheckbox->setEnabled ( true );
			d1.main->NtDomainNameCheckbox->setEnabled ( true );
			d1.main->ApplicationVersionCheckbox->setEnabled ( true );
			d1.main->UseGlobalIpsecSecretCheckbox->setEnabled ( true );
			d1.main->PerfectForwardSecurityCheckbox->setEnabled ( true );
		}

		if (currentProfile->connectiontype == VpnAccountData::pptp)
		{
			d1.main->NtDomainNameCheckbox->setEnabled ( true );
			d1.main->RefuseEapCheckbox->setEnabled( true );
			d1.main->requireMppeCheckbox->setEnabled( true );
			d1.main->Refuse128BitEncryptionCheckbox->setEnabled( true );
			d1.main->Refuse40BitEncryptionCheckbox->setEnabled( true );
			d1.main->DnsServerCheckbox->setEnabled( true );
			d1.main->DnsServerLineedit->setEnabled( true );
			d1.main->NtDomainNameCheckbox->setEnabled( true );
			//d1.main->NtDomainNameLineedit->setEnabled( true );
			d1.main->AllowStatefulModeCheckbox->setEnabled( true );
			d1.main->GetDnsFromPeerCheckbox->setEnabled( true );
			d1.main->SetDefaultRouteCheckbox->setEnabled( true );
			if (currentProfile->getSetDefaultRoute())
				d1.main->ReplaceDefaultRouteCheckbox->setEnabled( true );

			d1.main->IkeGroupCheckbox->setEnabled( true );
			if (!currentProfile->getIkeGroup().isEmpty())
				d1.main->IkeGroupCombobox->setCurrentText(currentProfile->getIkeGroup());

			d1.main->MtuCheckbox->setEnabled(true);
		}


		d1.main->IkeGroupCheckbox->setChecked ( currentProfile->getUseIkeGroup() );

		d1.main->SingleDesCheckbox->setChecked( currentProfile->getUseSingleDes() );

		d1.main->IkeGroupCheckbox->setChecked ( currentProfile->getUseIkeGroup() );

		d1.main->UseGlobalIpsecSecretCheckbox->setChecked ( currentProfile->getUseGlobalIpsecSecret() );

		d1.main->PerfectForwardSecurityCheckbox->setChecked( currentProfile->getUsePerfectForwardSecrety() );

		if ( currentProfile->getUseNtDomainName() )
		{
			d1.main->NtDomainNameLineedit->setEnabled ( true );
			d1.main->NtDomainNameCheckbox->setChecked( true );
		}

		if ( currentProfile->getUseApplicationVersion() )
		{
			d1.main->ApplicationVersionLineedit->setEnabled ( true );
			d1.main->ApplicationVersionCheckbox->setChecked( true );
		}

		if ( currentProfile->getUseLocalPort() )
		{
			d1.main->LocalPortSpinbox->setEnabled ( true );
			d1.main->LocalPortCheckbox->setChecked( true );
		}

		d1.main->IkeGroupCombobox->setEnabled (  currentProfile->getUseIkeGroup() );

		d1.main->UseNoBsdCompCheckbox->setChecked(currentProfile->getUseNoBsdComp());

		d1.main->UseNoDeflateCheckbox->setChecked(currentProfile->getUseNoDeflate());
	}

	d1.exec();

	saveSessionClicked();

}

void ProfileWidget::pingIpToggled( bool )
{
	if ( PingCheckBox->isChecked() )
	{
		PingIPLineEdit->setEnabled( true );
	}
	else
	{
		PingIPLineEdit->setEnabled( false );
	}
}

void ProfileWidget::useSpecialRemoteIDClicked()
{
	if ( UseSpecialRemoteIDcheckBox->isChecked() )
		RemoteIDLineEdit->setEnabled( true );
	else
		RemoteIDLineEdit->setEnabled( false );
}

void ProfileWidget::doubleClicked(QListBoxItem *)
{
	advancedSettingsClicked();

}

void ProfileWidget::renameSessionClicked()
{
	bool ok=false;
	QString oldName= currentProfile->getName();
	QString newName = KInputDialog::getText ( i18n("New name"),i18n("Enter new name for profile:"), oldName, &ok,this);

	if (ok)
	{
		currentProfile->setName(newName);
		int currIndex = ProfileListBox->currentItem();
		ProfileListBox->removeItem(currIndex);
		ProfileListBox->insertItem(newName, currIndex);
		ProfileListBox->setCurrentItem(currIndex);


		saveSessionClicked();

		LogOutput->append( i18n( "Profile \"%1\" renamed to \"%2\"." ).arg(oldName).arg( currentProfile->getName() ) );
	}

}

void ProfileWidget::executeCmdAfterDisconnectClicked()
{
	if (ExcuteCmdAfterDisconnectCheckBox->isChecked())
		CmdDisconnectTextEdit->setEnabled( TRUE );
	else
		CmdDisconnectTextEdit->setEnabled( FALSE );
}


void ProfileWidget::executeCmdAfterConnectClicked()
{
	if (ExcuteCmdAfterConnectCheckBox->isChecked())
		CmdConnectTextEdit->setEnabled( TRUE );
	else
		CmdConnectTextEdit->setEnabled( FALSE );
}

void ProfileWidget::useVirtualIPClicked()
{
	if (UseVirtualIPCheckBox->isChecked())
		LocalVirtualIpEdit->setEnabled( TRUE );
	else
		LocalVirtualIpEdit->setEnabled( FALSE );
}

void ProfileWidget::PskInFileToggled()
{
	if (PskInFileCheckBox->isChecked())
	{
		LabelPskFile->setEnabled(TRUE);
		PSKFileURLRequester->setEnabled(TRUE);
		LabelPsk->setEnabled(FALSE);
		PSKLineEdit->setEnabled(FALSE);
	}
	else
	{
		LabelPskFile->setEnabled(FALSE);
		PSKFileURLRequester->setEnabled(FALSE);
		LabelPsk->setEnabled(TRUE);
		PSKLineEdit->setEnabled(TRUE);

	}
}

void ProfileWidget::additionalNetworkRoutesToggled(bool)
{
	if (UseExtraNetworkRoutesCheckbox->isChecked())
	{
		NetworkListView->setEnabled(TRUE);
		DeleteNetworkRoutePushButton->setEnabled(TRUE);
		AddRoutePushButton->setEnabled(TRUE);
	}
	else
	{
		NetworkListView->setEnabled(FALSE);
		DeleteNetworkRoutePushButton->setEnabled(FALSE);
		AddRoutePushButton->setEnabled(FALSE);
	}
}

void ProfileWidget::addNetworkRouteClicked()
{
	AddNetworkRouteDialog dlg(this, i18n("Add network route..."));
	if (dlg.exec())
	{
		NetworkListView->insertItem( new QListViewItem(NetworkListView, dlg.getNetwork(),dlg.getNetmask()));
	}
}


void ProfileWidget::delNetworkRouteClicked()
{
	if ( NetworkListView->childCount() > 0 && NetworkListView->currentItem()!= 0 )
	{
		QListViewItem *item = NetworkListView->currentItem();
		QString Network = item->text(0);
		QString Netmask = item->text(1);
		int result = KMessageBox::questionYesNo ( this, i18n( "Really delete network \"%1/%2\"?" ).arg( Network ).arg(Netmask), i18n( "Delete?" ) );
		//LogOutput->append( "Result: "+ QString().setNum(result) ) ;
		if ( result == 3) // Yes
		{
			NetworkListView->removeItem(item);
		}
	}
}

void ProfileWidget::useSpecialServerCertificateToggled(bool)
{
	if (UseSpecialServerCertificateCheckBox->isChecked())
	{
		SpecialServerCertificateURLRequester->setEnabled(TRUE);
	}
	else
	{
		SpecialServerCertificateURLRequester->setEnabled(FALSE);
	}
}

void ProfileWidget::udpPortToggled(bool)
{
	if (UdpPortCheckbox->isChecked())
	{
		UdpPortSpinbox->setEnabled(TRUE);
	}
	else
	{
		UdpPortSpinbox->setEnabled(FALSE);
	}
}

void ProfileWidget::useUdpToggled (bool)
{
	if (UseUdpCheckbox->isChecked())
	{
		UdpPortCheckbox->setEnabled(TRUE);
		UdpPortSpinbox->setEnabled(TRUE);
	}
	else
	{
		UdpPortCheckbox->setEnabled(FALSE);
		UdpPortSpinbox->setEnabled(FALSE);
	}


}





#include "profilewidget.moc"
