/***************************************************************************
*  Copyright (C) 2004 by Christoph Thielecke               *
*  crissi99@gmx.de                            *
*                                     *
*  Contains parts of code from klcc project               *
*  (Peter Simonsson <psn@linux.se>.                   *
*                                     *
*  This program is free software; you can redistribute it and/or modify *
*  it under the terms of the GNU General Public License as published by *
*  the Free Software Foundation; either version 2 of the License, or   *
*  (at your option) any later version.                  *
*                                     *
*  This program is distributed in the hope that it will be useful,    *
*  but WITHOUT ANY WARRANTY; without even the implied warranty of    *
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the     *
*  GNU General Public License for more details.             *
*                                     *
*  You should have received a copy of the GNU General Public License   *
*  along with this program; if not, write to the             *
*  Free Software Foundation, Inc.,                    *
*  59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.       *
***************************************************************************/
//BEGIN includes
#include <time.h>
#include <iomanip>
#include <stdlib.h>
#include <qstringlist.h>
#include <qcstring.h>
#include <qptrlist.h>
#include <qcolor.h>
#include <qcheckbox.h>
#include <qspinbox.h>
#include <qtextedit.h>
#include <qpushbutton.h>
#include <qtooltip.h>
#include <qtimer.h>
#include <qdir.h>
#include <qspinbox.h>
#include <qdns.h>
#include <qstringlist.h>
#include <qcombobox.h>
#include <qfile.h>
#include <qdatetime.h>
#include <qtextstream.h>
#include <qsocket.h>
#include <qstatusbar.h>
#include <qstringlist.h>
#include <qcolor.h>
#include <qregexp.h>
#include <kaboutdata.h>
#include <kstdaction.h>
#include <klocale.h>
#include <kiconloader.h>
#include <kmessagebox.h>
#include <kpassdlg.h>
#include <knotifyclient.h>
#include <kstatusbar.h>
#include <kdebug.h>
#include <krun.h>
#include <kkeydialog.h>
#include <kedittoolbar.h>
#include <knotifydialog.h>
#include <kapplication.h>
#include <klineedit.h>
#include <kpassdlg.h>
#include <kurlrequester.h>
#include <kcombobox.h>
#include <kstddirs.h>
#include <klineeditdlg.h>
#include <kmessagebox.h>
#include <kio/netaccess.h>
#include <ktempfile.h>
#include <klistview.h>
#include <kglobal.h>
#include <kstandarddirs.h>
#include <knuminput.h>

#include "kvpnc.h"
#include "mainview.h"
#include "mainviewbase.h"
#include "preferencesdialog.h"
#include "helpdialog.h"
#include "advancedsettingsdialog.h"
#include "importprofiledialog.h"
#include "networkinterface.h"
#include "newprofiledialog.h"
#include "importcertificatedialog.h"
#include "vpnaccountdata.h"
#include "profilemanager.h"
#include "enterpassworddialog.h"
#include <ktempfile.h>
#include <iostream>
#include "utils.h"
#include "toolsinfodialog.h"
#include "logviewerdialog.h"

//END includes

KVpnc::KVpnc( KApplication *parent, QWidget*, const char *name ) : KMainWindow( 0, name ),   DCOPObject("KVpncInterface")
{

	config = kapp->config();
	this->parent = parent;
	setCaption( i18n( "KVpnc" ) );

	disconnectedIcon = KGlobal::iconLoader()->loadIcon( "kvpnc-disconnected", KIcon::NoGroup, 22 );
	connectedIcon = KGlobal::iconLoader()->loadIcon( "kvpnc-connected", KIcon::NoGroup, 22 );
	connectingIcon = KGlobal::iconLoader()->loadIcon( "kvpnc-connecting", KIcon::NoGroup, 22 );
	setIcon( disconnectedIcon );

	KStandardDirs *dirs = KGlobal::dirs();
	logfileName = dirs->saveLocation( "data" );
	logfileName += "/kvpnc/kvpnc.log";
	logfile.setName( logfileName );
	s.setDevice( &logfile );

	GlobalConfig = new KVpncConfig();

	this->initStatusbar();
	this->initAction();
	this->initDockWindow();
	this->initView();

	status = disconnected;
	mw->buttonConnect->setEnabled( false );
	mw->buttonDisconnect->setEnabled( false );


	InfoLogColor=Qt::black;
	ErrorLogColor=Qt::red;
	SuccessLogColor=Qt::darkGreen;
	DebugLogColor=Qt::yellow;


	Vpnc_pid_file = "";
	VpncConnectScript = "";
	VpncDisconnectScript = "";
	OldDefaultroute = "";
	lastProfile = "";
	TunnelInterfaceIP="";

	connectCounter = 0;
	connectSleepDelay = 15;
	connectSleepRetry = 0;
	tryConnectTimeout = 10;
	runningCheckCounter = 0;
	runningCheckTimeout = 60;
	currentProfile = 0;
	ConnectProcess = 0;
	DisconnectProcess = 0;
	RacoonProcess = 0;
	FreeswanProcess = 0;
	PingProcess = 0;
	TailProcess = 0;
	X509CertificateExtractProcess = 0;

	AuthRetry=false;

	this->loadOptions();
}

KVpnc::~KVpnc()
{}

void KVpnc::initAction()
{
	( void* ) KStdAction::quit( this, SLOT( quitCalled() ), actionCollection() );

	saveSessionAction = new KAction( i18n( "&Save profile..." ), "fileexport", Key_S, this, SLOT( saveSessionClicked() ), actionCollection(), "saveSession" );
	deleteSessionAction = new KAction( i18n( "&Delete profile..." ), "editdelete", Key_D, this, SLOT( deleteSessionClicked() ), actionCollection(), "deleteSession" );
	newSessionAction = new KAction( i18n( "&New profile..." ), "kvpnc-profilenew", Key_N, this, SLOT( newSessionClicked() ), actionCollection(), "newSession" );
	//advancedSessionAction = new KAction( i18n( "&Advanced..." ), "wizard", Key_A, this, SLOT( advancedSettingsClicked() ), actionCollection(), "advancedSettingsSession" );
	importSessionAction = new KAction( i18n( "&Import Cisco pcf file..." ), "kvpnc-pcfimport", Key_I, this, SLOT( importProfileClicked() ), actionCollection(), "importSession" );

	importCertAction = new KAction( i18n( "&Import certificate..." ), "kvpnc-certimport", Key_C, this, SLOT( importCertClicked() ), actionCollection(), "importCert" );

	connectAction = new KAction( i18n( "&Connect" ), "kvpnc-connected", CTRL + Key_Up, this, SLOT( slotConnecting() ), actionCollection(), "serverConnect" );
	disconnectAction = new KAction( i18n( "&Disconnect" ), "kvpnc-disconnected", CTRL + Key_Down, this, SLOT( disconnectClicked() ), actionCollection(), "serverDisconnect" );

	profileManagerAction = new KAction( i18n( "&Manage profiles" ), "view_detailed", CTRL + Key_M, this, SLOT( showProfileManagerClicked() ), actionCollection(), "profileManager" );

	donateAction = new KAction ( i18n("&Donate..."), "kvpnc-donate", CTRL + Key_D, this, SLOT(
	                                 donateClicked() ), actionCollection(), "donate" );

	debugconsoleAction = new KAction ( i18n("Toggle debug &console"), "text_center", CTRL + Key_C, this, SLOT(toggleDebugConsole() ), actionCollection(), "debugconsole" );

	toolsInfoAction = new KAction ( i18n("Show &tools info"), "info", CTRL + Key_T, this, SLOT(toolsInfoClicked() ), actionCollection(), "toolsinfo" );

  //		toolsInfoAction = new KAction ( i18n("Show &tools info"), "info", CTRL + Key_T, this, SLOT(helpClicked() ), actionCollection(), "toolsinfo" );

	LogViewerAction = new KAction ( i18n("Show &log"), "log", CTRL + Key_L, this, SLOT(showLogViewer() ), actionCollection(), "logviewer" );

	KStdAction::preferences( this, SLOT( slotSettings() ), actionCollection(), "preferences" );

	KStdAction::helpContents(this, SLOT( helpClicked () ), actionCollection(), "help" );

	KStdAction::configureNotifications(this, SLOT(slotConfigNotifications()), actionCollection());

	this->connectAction->setEnabled( false );
	this->disconnectAction->setEnabled( false );

	showToolbarAction = KStdAction::showToolbar( this, SLOT( slotShowToolbar() ), actionCollection() );
	showStatusbarAction = KStdAction::showStatusbar( this, SLOT( slotShowStatusbar() ), actionCollection() );
	preferencesAction = KStdAction::preferences( this, SLOT( slotSettings() ), actionCollection() );

	( void* ) KStdAction::keyBindings( this, SLOT( slotKeyBindings() ), actionCollection() );
	( void* ) KStdAction::configureToolbars( this, SLOT( slotConfigToolbars() ), actionCollection() );
	( void* ) KStdAction::configureNotifications( this, SLOT( slotConfigNotifications() ), actionCollection() );

	createGUI();

}

void KVpnc::initView()
{

	mw = new MainView( this, "mainView" );
	setCentralWidget( mw );


	connect ( mw->buttonConnect, SIGNAL( clicked() ), this, SLOT( slotConnecting() ) );
	connect ( mw->buttonDisconnect, SIGNAL( clicked() ), this, SLOT( disconnectClicked() ) );

	connect ( mw->SessionCombo, SIGNAL ( activated ( const QString & ) ), this, SLOT ( sessionToggled( const QString& ) ) );

	mw->SessionCombo->setEnabled( FALSE );

	statusColorLabel = new QLabel(this);
	disconnectedStatusPixmap = new QPixmap(15,10);
	disconnectedStatusPixmap->fill(QColor(Qt::red));
	connectedStatusPixmap  = new QPixmap(15,10);
	connectedStatusPixmap->fill(QColor(Qt::green));
	connectingStatusPixmap = new QPixmap(15,10);
	connectingStatusPixmap->fill(QColor(Qt::yellow));


	statusColorLabel->setPixmap(*disconnectedStatusPixmap);
	statusBar()->addWidget(statusColorLabel,0,true);

	KNotifyClient::startDaemon();

	
}

void KVpnc::initStatusbar()
{
	statusBar() ->insertItem( i18n( "Ready." ), 1, 100 );
	statusBar() ->setItemAlignment( 1, AlignLeft | AlignVCenter );
	statusBar() ->insertItem( i18n( "Disconnected" ), 2, 0, true );
}

void KVpnc::initDockWindow()
{
	this->kickerMenu = new KVpncKicker( this, "kickerIcon" );
	this->connectAction->plug( this->kickerMenu->menu(),1 );
	this->disconnectAction->plug( this->kickerMenu->menu(),2 );
	this->kickerMenu->menu()->insertSeparator(3);
	this->preferencesAction->plug( this->kickerMenu->menu(),4 );
}

void KVpnc::saveOptions()
{
	KStandardDirs * dirs = KGlobal::dirs();
	QString filePath = dirs->findResource ( "config", "kvpncrc" );
	QFile::remove
		( filePath );
	this->config->setGroup( "General Options" );

	this->config->writeEntry( "Minimize after connect", GlobalConfig->minimizeAfterConnect );
	this->config->writeEntry( "Use silent quit", GlobalConfig->useSilentQuit );
	this->config->writeEntry( "Show debugconsole", GlobalConfig->showDebugConsole );
	this->config->writeEntry( "Vpnc debuglevel", GlobalConfig->VpncDebugLevel );
	this->config->writeEntry( "Pid file for vpnc ", Vpnc_pid_file );
	this->config->writeEntry( "Racoon debuglevel", GlobalConfig->RacoonDebugLevel );
	this->config->writeEntry( "Kvpnc debuglevel", GlobalConfig->VpncDebugLevel );
	this->config->writeEntry( "Path to vpnc-bin", GlobalConfig->pathToVpnc );
	this->config->writeEntry( "Path to racoon", GlobalConfig->pathToRacoon );
	this->config->writeEntry( "Path to setkey", GlobalConfig->pathToSetkey );
	this->config->writeEntry( "Path to iptables", GlobalConfig->pathToIptables );
	this->config->writeEntry( "Path to openssl", GlobalConfig->pathToOpenssl );
	this->config->writeEntry( "Path to freeswan", GlobalConfig->pathToFreeswan );
	this->config->writeEntry( "Path to pppd", GlobalConfig->pathToPppd );
	this->config->writeEntry( "Path to kill", GlobalConfig->pathToKill );
	this->config->writeEntry( "Path to kill-all", GlobalConfig->pathToKillall );
	this->config->writeEntry( "Path to openvpn", GlobalConfig->pathToOpenvpn );
	this->config->writeEntry( "Pppd debuglevel", GlobalConfig->PppdDebugLevel );
	this->config->writeEntry( "Openvpn debuglevel", GlobalConfig->OpenvpnDebugLevel );
	this->config->writeEntry( "Show Statusbar", showStatusbarAction->isChecked() );
	this->config->writeEntry( "Try connect timeout", tryConnectTimeout );
	this->config->writeEntry( "Last Profile", lastProfile );
	this->config->writeEntry( "Write log file", GlobalConfig->writeLogFile );
	this->config->writeEntry( "Hold generated scripts", GlobalConfig->holdGeneratedScripts );

	toolBar( "mainToolBar" ) ->saveSettings( this->config, "ToolBar Settings" );

	this->config->writeEntry( "Programs in path", GlobalConfig->programsInPath );

	this->config->writeEntry( "Mainwindow width", QWidget::width() );
	this->config->writeEntry( "Mainwindow height", QWidget::height() );
	this->config->writeEntry( "Mainwindow position x", QWidget::x() );
	this->config->writeEntry( "Mainwindow position y", QWidget::y() );

	this->config->writeEntry( "Info logwindow color", InfoLogColor );
	this->config->writeEntry( "Remote logwindow color", RemoteLogColor );
	this->config->writeEntry( "Error logwindow color", ErrorLogColor );
	this->config->writeEntry( "Success  logwindow color", SuccessLogColor );
	this->config->writeEntry( "Debug logwindow color", DebugLogColor );
	this->config->writeEntry( "Use colorized log output", GlobalConfig->useColorizedLogOutput );
	this->config->writeEntry( "Show interface IP in tooltip", GlobalConfig->showInterfaceIPinTooltip );

	/* = user data = */
	/* old style: one profile: will be removed in future */
	/*
	this->config->setGroup( "Client Options" );
	this->config->writeEntry( "VPN Gateway", this->mw->gatewayLineEdit->text() );
	this->config->writeEntry( "VPN ID", this->mw->IDLineEdit->text() );
	this->config->writeEntry( "Username", this->mw->UsernameLineEdit->text() );
	if ( this->mw->CheckUserPass->isChecked() ) {
	 this->config->writeEntry( "Password", this->mw->PasswordEdit->text() );
	}
	this->config->writeEntry( "Save user password", this->mw->CheckUserPass->isChecked() );
	this->config->writeEntry( "Save group password", this->mw->CheckGroupPass->isChecked() );
	*/
	/* new style: multiple profiles: not implemented */
	VpnAccountData *it;
	if ( !GlobalConfig->AccountList->isEmpty() )
	{
		for ( it = GlobalConfig->AccountList->first(); it; it = GlobalConfig->AccountList->next() )
		{
			if ( !it->getName().isEmpty() )
			{
				QString name = it->getName();
				QString ProfileName = "Profile_";
				ProfileName += name;
				this->config->setGroup( ProfileName );

				if ( it->getConnectionType() == VpnAccountData::cisco )
					this->config->writeEntry( "Connection type", "cisco" );
				else if ( it->getConnectionType() == VpnAccountData::racoon )
					this->config->writeEntry( "Connection type", "racoon" );
				else if ( it->getConnectionType() == VpnAccountData::freeswan )
					this->config->writeEntry( "Connection type", "freeswan" );
				else if ( it->getConnectionType() == VpnAccountData::pptp )
					this->config->writeEntry( "Connection type", "pptp" );
				else if ( it->getConnectionType() == VpnAccountData::openvpn )
					this->config->writeEntry( "Connection type", "openvpn" );
				else
					this->config->writeEntry( "Connection type", "other" );
				this->config->writeEntry( "VPN Gateway", it->getGateway() );
				this->config->writeEntry( "VPN ID", it->getID() );
				this->config->writeEntry( "Username", it->getUserName() );
				if ( it->getSaveUserPassword() )
				{
					this->config->writeEntry( "User password", it->getUserPassword() );
				}
				if ( it->getSavePsk() )
				{
					this->config->writeEntry( "Pre shared key", it->getPreSharedKey() );
				}
				this->config->writeEntry( "Save user password", it->getSaveUserPassword() );
				this->config->writeEntry( "Save PSK", it->getSavePsk() );

				this->config->writeEntry( "Use advanced settings", it->getUseAdvancedSettings() );
				this->config->writeEntry( "Application version", it->getApplicationVersion() );
				this->config->writeEntry( "Local port", it->getLocalPort() );
				this->config->writeEntry( "NT domain name", it->getNtDomainName() );
				this->config->writeEntry( "Perfect forward secrety", it->getPerfectForwardSecrety() );
				this->config->writeEntry( "IKE group", it->getIkeGroup() );
				this->config->writeEntry( "Use application version", it->getUseApplicationVersion() );
				this->config->writeEntry( "Use global IPSec secret", it->getUseGlobalIpsecSecret() );
				this->config->writeEntry( "Use IKE group", it->getUseIkeGroup() );
				this->config->writeEntry( "Use local port", it->getUseLocalPort() );
				this->config->writeEntry( "Use NT domain name", it->getUseNtDomainName() );
				this->config->writeEntry( "Use single DES", it->getUseSingleDes() );
				this->config->writeEntry( "Use perfect Forward Secrety", it->getUsePerfectForwardSecrety() );
				QString RemoteNetAddr = it->getRemoteNet().section( '/', 0, 0 );
				QString RemoteNetMask = it->getRemoteNet().section( '/', 1, 1 );
				this->config->writeEntry( "Remote net address", RemoteNetAddr );
				this->config->writeEntry( "Remote net mask", RemoteNetMask );
				this->config->writeEntry( "Pre shared key", it->getPreSharedKey() );
				this->config->writeEntry( "x509 certificate", it->getX509Certificate() );
				this->config->writeEntry( "Ca certificate", it->getCaCertificate() );
				this->config->writeEntry( "Network device", it->getNetworkDevice() );
				this->config->writeEntry( "Cert path", it->getCertPath() );
				if ( it->getAuthType() == VpnAccountData::cert )
					this->config->writeEntry( "Auth type", "cert" );
				else
					this->config->writeEntry( "Auth type", "psk" );
				this->config->writeEntry( "do ping ip", it->getDoPingIP() );
				this->config->writeEntry( "Ping host IP", it->getPingHostIP() );
				this->config->writeEntry( "PrivateKey password", it->getPrivateKeyPass() );
				this->config->writeEntry( "Private key", it->getPrivateKey() );
				this->config->writeEntry( "Use special remote ID", it->getUseSpecialRemoteID() );
				this->config->writeEntry( "Special server certificate", it->getSpecialServerCertificate() );
				this->config->writeEntry( "Use special server certificate", it->getUseSpecialServerCertificate() );
				this->config->writeEntry( "Special remote ID", it->getSpecialRemoteID() );
				this->config->writeEntry( "Use UDP-protocol", it->getUseUdp() );
				this->config->writeEntry( "Use UDP-port", it->getUseUdpPort() );
				this->config->writeEntry( "local UDP port", it->getUdpPort() );
				this->config->writeEntry( "use DNS-server", it->getUseDnsServer() );
				this->config->writeEntry( "DNS server", it->getDnsServer() );
				this->config->writeEntry( "get dns server from peer", it->getUseGetDnsFromPeer() );
				this->config->writeEntry( "refuse 40 bit encryption", it->getRefuse40BitEncryption() );
				this->config->writeEntry( "refuse 128 bit encryption", it->getRefuse128BitEncryption() );
				this->config->writeEntry( "require stateless encryption", it->getAllowStatefulMode() );
				this->config->writeEntry( "require mppe", it->getRequireMppe() );
				this->config->writeEntry( "refuse EAP", it->getRefuseEap() );
				this->config->writeEntry( "Local virtual IP", it->getLocalVirtualIP() );
				this->config->writeEntry( "Remote virtual IP", it->getRemoteVirtualIP() );
				this->config->writeEntry( "Pre shared key file", it->getPreSharedKeyFile() );
				this->config->writeEntry( "Execute command after connect", it->getExecuteCmdAfterConnect() );
				this->config->writeEntry( "Execute command after disconnect", it->getExecuteCmdAfterDisconnect() );
				this->config->writeEntry( "Command after connect", it->getCommandAfterConnect() );
				this->config->writeEntry( "Command after disconnect", it->getCommandAfterDisconnect() );
				this->config->writeEntry( "Use no BSD compression", it->getUseNoBsdComp() );
				this->config->writeEntry( "Use no deflate", it->getUseNoDeflate() );
				this->config->writeEntry( "Set default route", it->getSetDefaultRoute() );
				this->config->writeEntry( "Replace default route", it->getReplaceDefaultRoute() );
				this->config->writeEntry( "Use own MTU", it->getUseMtu() );
				this->config->writeEntry( "MTU for pppd", it->getMtu() );
				this->config->writeEntry( "Use Virtual IP", it->getUseVirtualIP() );
				this->config->writeEntry( "Virtual IP", it->getVirtualIP() );
				this->config->writeEntry( "PSK is in file", it->getPskIsInFile() );
				this->config->writeEntry( "Use additional network routes", it->getUseAdditionalNetworkRoutes() );

				QString AdditionalNetworkRoutesString;
				QStringList AdditionalNetworkRoutes = it->getAdditionalNetworkRoutes();

				// Networks delimited by a ','
				for ( QStringList::Iterator it2 = AdditionalNetworkRoutes.begin(); it2 != 	AdditionalNetworkRoutes.end(); ++it2 )
				{
					AdditionalNetworkRoutesString+=*it2;
					AdditionalNetworkRoutesString+=",";
				}
				AdditionalNetworkRoutesString=AdditionalNetworkRoutesString.left(AdditionalNetworkRoutesString.length()-1);
				this->config->writeEntry( "Additional network routes", AdditionalNetworkRoutesString );


				/* give info */
				slotStatusMsg ( i18n( "Profile \"%1\" saved." ).arg( ProfileName ), ID_FLASH_MSG );
				appendLogEntry ( i18n( "Profile \"%1\" saved." ).arg( ProfileName ) , info);
			}
		}
	}

	this->config->sync();
}

void KVpnc::loadOptions()
{
	toolBar( "mainToolBar" ) ->applySettings( this->config, "ToolBar Settings" );
	showToolbarAction->setChecked( !toolBar( "mainToolBar" ) ->isHidden() );

	this->config->setGroup( "General Options" );
	GlobalConfig->minimizeAfterConnect = this->config->readBoolEntry( "Minimize after connect", true );
	GlobalConfig->useSilentQuit = this->config->readBoolEntry( "Use silent quit", false );
	GlobalConfig->showDebugConsole = this->config->readBoolEntry( "Show debugconsole", false );
	GlobalConfig->VpncDebugLevel = this->config->readNumEntry( "Vpnc debuglevel", 0 );
	Vpnc_pid_file = this->config->readEntry( "Pid file for vpnc ", "/var/run/vpnc/pid" );
	GlobalConfig->RacoonDebugLevel = this->config->readNumEntry( "Racoon debuglevel", 0 );
	GlobalConfig->KvpncDebugLevel = this->config->readNumEntry( "Kvpnc debuglevel", 0 );
	GlobalConfig->PppdDebugLevel = this->config->readNumEntry( "Pppd debuglevel", 0 );
	GlobalConfig->OpenvpnDebugLevel = this->config->readNumEntry( "Openvpn debuglevel", 0 );
	GlobalConfig->pathToVpnc = this->config->readEntry( "Path to vpnc-bin", "/usr/sbin/vpnc" );
	GlobalConfig->pathToSetkey = this->config->readEntry( "Path to setkey", "/usr/sbin/setkey" );
	GlobalConfig->pathToRacoon = this->config->readEntry( "Path to racoon", "/usr/sbin/racoon" );
	GlobalConfig->pathToIptables = this->config->readEntry( "Path to iptables", "/sbin/iptables" );
	GlobalConfig->pathToOpenssl = this->config->readEntry( "Path to openssl", "/usr/bin/openssl" );
	GlobalConfig->pathToFreeswan = this->config->readEntry( "Path to freeswan", "/usr/sbin/ipsec" );
	GlobalConfig->pathToPppd = this->config->readEntry ("Path to pppd","/usr/sbin/pppd");
	GlobalConfig->pathToKill = this->config->readEntry ("Path to kill","/bin/kill");
	GlobalConfig->pathToKillall = this->config->readEntry ("Path to kill-all","/usr/bin/killall");
	GlobalConfig->pathToOpenvpn = this->config->readEntry ("Path to openvpn","/usr/sbin/openvpn");

	tryConnectTimeout = QString( this->config->readEntry( "Try connect timeout", "10" ) ).toInt();
	lastProfile = this->config->readEntry( "Last Profile", "" );
	GlobalConfig->writeLogFile = this->config->readBoolEntry( "Write log file", true );
	GlobalConfig->holdGeneratedScripts = this->config->readBoolEntry( "Hold generated scripts", true );
	bool bViewStatusbar = this->config->readBoolEntry( "Show Statusbar", true );
	GlobalConfig->programsInPath = this->config->readBoolEntry( "Programs in path", true );

	if (GlobalConfig->programsInPath)
		GlobalConfig->useDefaultPaths=true;

	showStatusbarAction->setChecked( bViewStatusbar );
	slotShowStatusbar();

	if ( GlobalConfig->showDebugConsole )
	{
		mw->DebugOutput->show();
		//resize( minimumSizeHint() );
	}
	else
	{
		mw->DebugOutput->hide();
		setMinimumSize(0,0);
		this->resize( minimumSizeHint() );
	}



	WindowSize = QSize( this->config->readNumEntry( "Mainwindow width", 50 ), this->config->readNumEntry( "Mainwindow height", 50 ) );
	WindowPos = QPoint ( this->config->readNumEntry( "Mainwindow position x", 10 ), this->config->readNumEntry( "Mainwindow position y", 10 ) );

	if ( WindowSize.width() < 50 || WindowSize.height() < 50 )
	{
		WindowSize = QSize( minimumWidth(), minimumHeight() );
	}

	if ( WindowPos.x() < 0 || WindowPos.y() < 0 )
		WindowPos = QPoint ( 0, 0 );

	resize( WindowSize );
	move ( WindowPos );

	InfoLogColor = this->config->readColorEntry ( "Info logwindow color", new QColor(Qt::black) );
	RemoteLogColor = this->config->readColorEntry ( "Remote logwindow color", new QColor(Qt::black) );
	ErrorLogColor = this->config->readColorEntry( "Error logwindow color", new QColor(Qt::red ));
	SuccessLogColor = this->config->readColorEntry( "Success  logwindow color", new QColor(Qt::darkGreen) );
	DebugLogColor = this->config->readColorEntry( "Debug logwindow color", new QColor(255,175,0 ));
	GlobalConfig->useColorizedLogOutput = this->config->readBoolEntry("Use colorized log output", true);
	GlobalConfig->showInterfaceIPinTooltip = this->config->readBoolEntry( "Show interface IP in tooltip", true );


	if ( GlobalConfig->writeLogFile )
		if ( !logfile.isOpen() )
		{
			KStandardDirs *dirs = KGlobal::dirs();

			QDir AppFolder (QString(dirs->saveLocation( "data" )+"/kvpnc"));
			if (!AppFolder.exists())
			{
				// 		KMessageBox::error( this, i18n( "creating dir: %1" ).arg(dirs->saveLocation( "data" )+"/kvpnc" ));
				QDir AppFolder2(dirs->saveLocation( "data" ));
				if(!AppFolder2.mkdir("kvpnc",true))
					KMessageBox::error( this, i18n( "The appdir for kvpnc could not be created. Be sure that you have write permission of \"%1\"." ).arg(dirs->saveLocation( "data" )) );
			}

			logfileName = dirs->saveLocation( "data" );
			logfileName += "/kvpnc/kvpnc.log";
			//KMessageBox::information( this, logfileName, "info" );
			logfile.setName( logfileName );
			if ( !logfile.open( IO_WriteOnly | IO_Append ) )
				KMessageBox::error( this, i18n( "Log file cant be opened!" ) );
			else
			{
				QTextStream s( &logfile );
				s << i18n( "Log session started at: " ) << QDateTime::currentDateTime().toString( Qt::TextDate ) << endl;
			}
		}

	if ( GlobalConfig->KvpncDebugLevel > 0 )
		appendLogEntry( i18n( "Global configuration loaded." ) ,info);

	//Failsafe
	if ( this->isHidden() && this->kickerMenu->isHidden() )
	{
		this->show();
	}

	/* = user data = */
	/* old style: one profile: will be removed in future */
	/*
	this->config->setGroup( "Client Options" );
	this->mw->gatewayLineEdit->setText( this->config->readEntry( "VPN Gateway", "" ) );
	this->mw->IDLineEdit->setText( this->config->readEntry( "VPN ID", "" ) );
	this->mw->PSKLineEdit->setText( this->config->readEntry( "Group password", "" ) );
	this->mw->UsernameLineEdit->setText( this->config->readEntry( "Username", "" ) );
	this->mw->PasswordEdit->setText( this->config->readEntry( "Password", "" ) );

	this->mw->CheckUserPass->setChecked( this->config->readBoolEntry( "Save user password", false ) );
	this->mw->CheckGroupPass->setChecked( this->config->readBoolEntry( "Save group password", false ) );
	*/

	/* profiles are stored in a group which is named to the following scheme:
	 Profile_XXXXX
	 XXXXX is the account name (important: unique!)
	*/
	QStringList configGroups = this->config->groupList();
	bool oldConfigurationFound = false;

	bool sessionFound = false;

	for ( QStringList::Iterator it = configGroups.begin(); it != configGroups.end(); ++it )
	{
		QString name = *it;
		if ( name.left( 7 ) == "Profile" && name.length() > 8 )
		{
			// Profile found

			sessionFound = true;

			if ( GlobalConfig->KvpncDebugLevel > 0 )
			{
				QString tmp = i18n( i18n( "Profile found: " ) );
				tmp += name.right( ( name.length() ) - 8 );
				appendLogEntry ( tmp, debug );
			}
			this->config->setGroup( name );
			VpnAccountData::ConnectionType ConnType = VpnAccountData::ConnectionType( VpnAccountData::cisco );
			QString type_string = this->config->readEntry( "Connection type", "cisco" );
			if ( type_string == "cisco" )
				ConnType = VpnAccountData::cisco;
			else if ( type_string == "racoon" )
				ConnType = VpnAccountData::racoon;
			else if ( type_string == "freeswan" )
				ConnType = VpnAccountData::freeswan;
			else if ( type_string == "pptp" )
				ConnType = VpnAccountData::pptp;
			else if ( type_string == "openvpn" )
				ConnType = VpnAccountData::openvpn;
			else
				ConnType = VpnAccountData::other;
			QString Name = name.right( name.length() - 8 );
			QString Gateway = this->config->readEntry( "VPN Gateway", "" );
			QString ID = this->config->readEntry( "VPN ID", "" ) ;
			QString Username = this->config->readEntry( "Username", "" );
			QString UserPassword = this->config->readEntry( "User password", "" );
			bool saveUserPassword = this->config->readBoolEntry( "Save user password", false );
			bool savePsk = this->config->readBoolEntry( "Save PSK", false );
			bool useAdvancedSettings = this->config->readBoolEntry( "Use advanced settings", false );
			QString ApplicationVersion = this->config->readEntry( "Application version", "Cisco Systems VPN Client 0.2-rm+zomb.1:Linux" );
			int Port = this->config->readNumEntry( "Local port", 500 );
			QString NtDomainName = this->config->readEntry( "NT domain name", "" );
			QString PerfectForwardSecrecy = this->config->readEntry( "Perfect forward secrety", "server" );
			QString IkeGroup = this->config->readEntry( "IKE group", "dh1" );
			bool useApplicationVersion = this->config->readBoolEntry( "Use application version", false );
			bool useGlobalIpsecSecret = this->config->readBoolEntry( "Use global IPSec secret", false );
			bool useIkeGroup = this->config->readBoolEntry( "Use IKE group", false );
			bool useLocalPort = this->config->readBoolEntry( "Use local port", false );
			bool useNtDomainName = this->config->readBoolEntry( "Use NT domain name", false );
			bool useSingleDes = this->config->readBoolEntry( "Use single DES", false );
			bool usePerfectSecrecy = this->config->readBoolEntry( "Use perfect Forward Secrety", false );
			QString RemoteNetAddr = this->config->readEntry( "Remote net address", "" );
			int RemoteNetMask = this->config->readNumEntry( "Remote net mask", 24 );
			QString PreSharedKey = this->config->readEntry( "Pre shared key", "" );
			QString X509Certificate = this->config->readEntry( "x509 certificate", "" );
			QString CaCertificate = this->config->readEntry( "Ca certificate", "" );
			QString NetworkDevice = this->config->readEntry( "Network device", "default" );
			QString CertPath = this->config->readEntry( "Cert path", "/etc/racoon/certs" );
			QString PrivateKeyPass = this->config->readEntry( "PrivateKey password", "" );
			QString PrivateKey = this->config->readEntry( "Private key", "" );
			VpnAccountData::AuthenticationType authtype;
			if ( this->config->readEntry( "Auth type", "cert" ) == "cert" )
				authtype = VpnAccountData::cert;
			else
				authtype = VpnAccountData::psk;

			bool doPingIP = this->config->readBoolEntry( "do ping ip", false );
			QString pingHostIP = this->config->readEntry( "Ping host IP", "" );
			bool useSpecialRemoteID = this->config->readBoolEntry( "Use special remote ID", false );
			QString SpecialRemoteID = this->config->readEntry( "Special remote ID", "" );
			bool useSpecialServerCertificate = this->config->readBoolEntry( "Use special server certificate", false );
			QString SpecialServerCertificate = this->config->readEntry( "Special server certificate", "" );
			bool useUdpProtocol = this->config->readBoolEntry( "Use UDP-protocol", false );
			bool useUdpPort = this->config->readBoolEntry( "Use UDP-port", false );
			int UdpPort = this->config->readNumEntry( "local UDP port", 4500 );
			bool useDnsServer = this->config->readBoolEntry( "use DNS-server", false);
			QString DnsServer = this->config->readEntry( "DNS server", "" );
			bool getDnsFromPeer = this->config->readBoolEntry( "get dns server from peer", true );
			bool refuse40BitEncryption = this->config->readBoolEntry( "refuse 40 bit encryption", false );
			bool refuse128BitEncryption = this->config->readBoolEntry( "refuse 128 bit encryption", false );
			bool requireStatelessEncryption = this->config->readBoolEntry( "require stateless encryption",false );
			bool requireMppe = this->config->readBoolEntry( "require mppe", true );
			bool refuseEap = this->config->readBoolEntry( "refuse EAP", false );
			bool useNoBsdComp = this->config->readBoolEntry( "Use no BSD compression", true );
			bool useNoDeflate = this->config->readBoolEntry( "Use no deflate", true );
			QString LocalVirtualIP = this->config->readEntry( "Local virtual IP", "" );
			QString RemoteVirtualIP = this->config->readEntry( "Remote virtual IP", "" );
			QString PreSharedKeyFile = this->config->readEntry( "Pre shared key file", "" );
			bool executeCmdAfterConnect = this->config->readBoolEntry( "Execute command after connect",false);
			bool executeCmdAfterDisconnect = this->config->readBoolEntry( "Execute command after disconnect", false);
			QString CommandAfterConnect = this->config->readEntry( "Command after connect", "" );
			QString CommandAfterDisconnect = this->config->readEntry( "Command after disconnect", "" );
			bool setDefaultRoute = this->config->readBoolEntry( "Set default route", true );
			bool replaceDefaultRoute = this->config->readBoolEntry( "Replace default route", false );
			bool useMtu = this->config->readBoolEntry( "Use own MTU", false );
			int Mtu = this->config->readNumEntry( "MTU for pppd", 1492 );
			bool useVirtualIP = this->config->readBoolEntry( "Use Virtual IP", false );
			QString VirtualIP = this->config->readEntry( "Virtual IP", "" );
			bool PskIsInFile = this->config->readBoolEntry( "PSK is in file", false );
			bool useAdditionalNetworkRoutes =	this->config->readBoolEntry( "Use additional network routes", false );

			QString AdditionalNetworkRoutesString = this->config->readEntry( "Additional network routes", "" );
			// Networks delimited by a ','
			QStringList AdditionalNetworkRoutes(QStringList::split( ",", AdditionalNetworkRoutesString ));

			VpnAccountData *data = new VpnAccountData( ConnType, Name );

			data->setGateway( Gateway );
			data->setID( ID );
			data->setUserName( Username );
			data->setUserPassword( UserPassword );
			data->setSaveUserPassword( saveUserPassword );
			data->setSavePsk( savePsk );
			data->setApplicationVersion( ApplicationVersion );
			data->setLocalPort( Port );
			data->setNtDomainName( NtDomainName );
			data->setPerfectForwardSecrety( PerfectForwardSecrecy );
			data->setIkeGroup( IkeGroup );
			data->setUseApplicationVersion( useApplicationVersion );
			data->setUseGlobalIpsecSecret( useGlobalIpsecSecret );
			data->setUseIkeGroup( useIkeGroup );
			data->setUseLocalPort( useLocalPort );
			data->setUseNtDomainName( useNtDomainName );
			data->setUseSingleDes( useSingleDes );
			data->setUseAdvancedSettings( useAdvancedSettings );
			data->setUsePerfectForwardSecrety( usePerfectSecrecy );
			data->setRemoteNet( RemoteNetAddr + "/" + QString().setNum( RemoteNetMask ) );
			data->setPreSharedKey( PreSharedKey );
			data->setX509Certificate( X509Certificate );
			data->setCaCertificate( CaCertificate );
			data->setCertPath( CertPath );
			data->setAuthType( authtype );
			data->setNetworkDevice( NetworkDevice );
			data->setDoPingIP( doPingIP );
			data->setPingHostIP( pingHostIP );
			data->setPrivateKey( PrivateKey );
			data->setPrivateKeyPass( PrivateKeyPass );
			data->setUseSpecialRemoteID( useSpecialRemoteID );
			data->setSpecialRemoteID( SpecialRemoteID );
			data->setUseUdp( useUdpProtocol );
			data->setUseUdpPort( useUdpPort );
			data->setUdpPort( UdpPort );
			data->setUseDnsServer( useDnsServer );
			data->setDnsServer( DnsServer );
			data->setUseGetDnsFromPeer( getDnsFromPeer );
			data->setRefuse40BitEncryption( refuse40BitEncryption );
			data->setRefuse128BitEncryption( refuse128BitEncryption );
			data->setAllowStatefulMode(requireStatelessEncryption );
			data->setRequireMppe( requireMppe );
			data->setRefuseEap( refuseEap );
			data->setLocalVirtualIP( LocalVirtualIP );
			data->setRemoteVirtualIP( RemoteVirtualIP );
			data->setPreSharedKeyFile( PreSharedKeyFile);
			data->setExecuteCmdAfterConnect( executeCmdAfterConnect );
			data->setExecuteCmdAfterDisconnect( executeCmdAfterDisconnect );
			data->setCommandAfterConnect( CommandAfterConnect );
			data->setCommandAfterDisconnect( CommandAfterDisconnect );
			data->setUseNoBsdComp( useNoBsdComp );
			data->setUseNoDeflate( useNoDeflate );
			data->setSetDefaultRoute( setDefaultRoute );
			data->setReplaceDefaultRoute( replaceDefaultRoute );
			data->setUseMtu(useMtu);
			data->setMtu(Mtu);
			data->setUseVirtualIP(useVirtualIP);
			data->setVirtualIP(VirtualIP);
			data->setPskIsInFile(PskIsInFile);
			data->setUseAdditionalNetworkRoutes(useAdditionalNetworkRoutes);
			data->setAdditionalNetworkRoutes( AdditionalNetworkRoutes );
			data->setUseSpecialServerCertificate( useSpecialServerCertificate );
			data->setSpecialServerCertificate( SpecialServerCertificate );

			GlobalConfig->AccountList->append( data );

			//ConnType, Name, Gateway, ID, GroupPassword, Username, UserPassword, saveUserPassword, saveGroupPassword, ApplicationVersion, Port, NtDomainName, PerfectForwardSecrecy, IkeGroup, useApplicationVersion, useGlobalIpsecSecret, useIkeGroup, useLocalPort, useNtDomainName, useSingleDes, useAdvancedSettings ) );

			if ( !Name.isEmpty() )
				mw->SessionCombo->insertItem( Name );
		}

		if ( sessionFound )
		{
			mw->SessionCombo->setEnabled( TRUE );
			mw->buttonConnect->setEnabled( TRUE );
		}

		if ( name == "Client Options" )
		{
			// old style configuration found
			this->config->setGroup( "Client Options" );
			QString Name = "Default";
			VpnAccountData::ConnectionType ConnType = VpnAccountData::cisco;
			QString Gateway = this->config->readEntry( "VPN Gateway", "" );
			QString ID = this->config->readEntry( "VPN ID", "" );
			QString Psk = this->config->readEntry( "Group password", "" );
			QString Username = this->config->readEntry( "Username", "" );
			QString UserPassword = this->config->readEntry( "Password", "" );
			bool saveUserPassword = this->config->readBoolEntry( "Save user password", false );
			bool savePsk = this->config->readBoolEntry( "Save group password", false );

			VpnAccountData *AccData = new VpnAccountData( ConnType, Name);
			AccData->setGateway ( Gateway );
			AccData->setID( ID );
			AccData->setPreSharedKey( Psk);
			AccData->setUserName( Username );
			AccData->setUserPassword( UserPassword );
			AccData->setSaveUserPassword( saveUserPassword );
			AccData->setSavePsk( savePsk );
			AccData->setLocalPort( 500 );
			AccData->setPerfectForwardSecrety ( "server" );
			AccData->setIkeGroup( "dh1" );
			GlobalConfig->AccountList->append( AccData );

			mw->SessionCombo->insertItem( Name );
			mw->SessionCombo->setCurrentItem( Name );

			lastProfile = "Default";

			sessionToggled( "Defaults" );

			// now mark for deleting old configuration
			oldConfigurationFound = true;
			if ( GlobalConfig->KvpncDebugLevel > 0 )
				appendLogEntry ( i18n( "Old configuration found, converted." ),info );

			if ( GlobalConfig->KvpncDebugLevel > 0 )
				appendLogEntry( i18n( "Configuration for profile \"%1\" loaded." ).arg( Name ),info );
		}


	}
	if ( oldConfigurationFound )
	{
		// delete configuration file
		KStandardDirs * dirs = KGlobal::dirs();
		QString filePath = dirs->findResource ( "config", "kvpncrc" );
		QFile::remove
			( filePath );
		if ( GlobalConfig->KvpncDebugLevel > 0 )
			appendLogEntry ( i18n( "Old configuration deleted." ),debug );
	}

	if ( !lastProfile.isEmpty() )
	{
		VpnAccountData * it;
		for ( it = GlobalConfig->AccountList->first(); it; it = GlobalConfig->AccountList->next() )
		{
			if (it->getName() == lastProfile)
			{
				currentProfile = it;
				mw->SessionCombo->setCurrentText( lastProfile );
				break;
			}
		}

		sessionToggled( lastProfile );
	}
	else
	{
		if ( !GlobalConfig->AccountList->isEmpty() )
		{
			this->connectAction->setEnabled( true );
			mw->SessionCombo->setCurrentItem( 0 );
			currentProfile = GlobalConfig->AccountList->first();
		}
	}

	//mw->minimumSizeHint();

}

bool KVpnc::queryExit()
{
	lastProfile = mw->SessionCombo->currentText();
	this->saveOptions();
	return true;
}

void KVpnc::slotSettings()
{
	this->slotStatusMsg( i18n( "Setup KVpnc..." ), ID_FLASH_MSG );
	this->slotStatusMsg( i18n( "Ready." ), ID_STATUS_MSG );

	PreferencesDialog prefDlg( this, i18n("Preferences..."),parent, mw->DebugOutput, GlobalConfig, currentProfile);
	prefDlg.main->VpncPathInput->setURL( GlobalConfig->pathToVpnc );
	prefDlg.main->RacoonPathInput->setURL( GlobalConfig->pathToRacoon );
	prefDlg.main->SetkeyPathInput->setURL( GlobalConfig->pathToSetkey );
	prefDlg.main->IptablesPathInput->setURL( GlobalConfig->pathToIptables );
	prefDlg.main->FreeswanPathInput->setURL( GlobalConfig->pathToFreeswan );
	prefDlg.main->OpenSSLPathInput->setURL( GlobalConfig->pathToOpenssl );
	prefDlg.main->PppdPathInput->setURL( GlobalConfig->pathToPppd );
	prefDlg.main->KillPathInput->setURL( GlobalConfig->pathToKill );
	prefDlg.main->KillallPathInput->setURL( GlobalConfig->pathToKillall );
	prefDlg.main->OpenvpnPathInput->setURL( GlobalConfig->pathToOpenvpn );
	prefDlg.main->checkMinimizeAfterConnectCheck->setChecked( GlobalConfig->minimizeAfterConnect );
	prefDlg.main->showDebugCheck->setChecked( GlobalConfig->showDebugConsole );
	prefDlg.main->WriteLogCheck->setChecked( GlobalConfig->writeLogFile );
	prefDlg.main->spinKvpncDebugLevel->setValue ( GlobalConfig->KvpncDebugLevel );
	prefDlg.main->spinRacoonDebugLevel->setValue ( GlobalConfig->RacoonDebugLevel );
	prefDlg.main->spinVpncDebugLevel->setValue ( GlobalConfig->VpncDebugLevel );
	prefDlg.main->spinOpenvpnDebugLevel->setValue ( GlobalConfig->OpenvpnDebugLevel );
	prefDlg.main->spinTryConnectTimeout->setValue( tryConnectTimeout );
	prefDlg.main->SilentQuitcheckBox->setChecked( GlobalConfig->useSilentQuit );
	prefDlg.main->ProgramsInPathcheckBox->setChecked(GlobalConfig->programsInPath);
	prefDlg.main->DebugPppdcheckBox->setChecked(GlobalConfig->enableDebugPppd);
	prefDlg.main->KeepConnectionFilesCheckBox->setChecked(GlobalConfig->holdGeneratedScripts);
	prefDlg.main->InfoLogLabel->setPaletteBackgroundColor (InfoLogColor);
	prefDlg.main->RemoteLogLabel->setPaletteBackgroundColor (RemoteLogColor);
	prefDlg.main->ErrorLogLabel->setPaletteBackgroundColor (ErrorLogColor);
	prefDlg.main->SuccessLogLabel->setPaletteBackgroundColor (SuccessLogColor);
	prefDlg.main->DebugLogLabel->setPaletteBackgroundColor (DebugLogColor);
	prefDlg.main->UseColorizedLogOutputcheckBox->setChecked(GlobalConfig->useColorizedLogOutput);
	prefDlg.main->ShowInterfaceIPinTooltipCheckBox->setChecked(GlobalConfig->showInterfaceIPinTooltip);

	prefDlg.exec();

	GlobalConfig->pathToVpnc = prefDlg.main->VpncPathInput->url();
	GlobalConfig->pathToSetkey = prefDlg.main->SetkeyPathInput->url();
	GlobalConfig->pathToRacoon = prefDlg.main->RacoonPathInput->url();
	GlobalConfig->pathToIptables = prefDlg.main->IptablesPathInput->url();
	GlobalConfig->pathToOpenssl = prefDlg.main->OpenSSLPathInput->url();
	GlobalConfig->pathToPppd = prefDlg.main->PppdPathInput->url();
	GlobalConfig->pathToKill = prefDlg.main->KillPathInput->url();
	GlobalConfig->pathToKillall = prefDlg.main->KillallPathInput->url();
	GlobalConfig->pathToOpenvpn = prefDlg.main->OpenvpnPathInput->url();
	GlobalConfig->pathToFreeswan = prefDlg.main->FreeswanPathInput->url();
	GlobalConfig->minimizeAfterConnect = prefDlg.main->checkMinimizeAfterConnectCheck->isChecked();
	GlobalConfig->showDebugConsole = prefDlg.main->showDebugCheck->isChecked();
	GlobalConfig->useSilentQuit = prefDlg.main->SilentQuitcheckBox->isChecked();

	if (prefDlg.MyprofileWidget->profileAddedOrDeleted)
	{
		saveOptions();
		if (GlobalConfig->AccountList->isEmpty())
		{
			currentProfile=0L;
			mw->SessionCombo->setEnabled ( FALSE );
			connectAction->setEnabled(false);
			mw->buttonConnect->setEnabled ( FALSE );
		}
		else
		{
			sessionToggled(currentProfile->getName());
		}

		profileAddedOrDeleted();
		if (!prefDlg.MyprofileWidget->NewProfileName.isEmpty())
			sessionToggled(prefDlg.MyprofileWidget->NewProfileName);
	}

	if ( GlobalConfig->showDebugConsole )
	{
		mw->DebugOutput->show();
		//resize( minimumSizeHint() );
	}
	else
	{
		mw->DebugOutput->hide();
		setMinimumSize(0,0);
		this->resize( minimumSizeHint() );
	}
	GlobalConfig->KvpncDebugLevel = prefDlg.main->spinKvpncDebugLevel->value();
	GlobalConfig->VpncDebugLevel = prefDlg.main->spinVpncDebugLevel->value();
	GlobalConfig->OpenvpnDebugLevel = prefDlg.main->spinOpenvpnDebugLevel->value();
	tryConnectTimeout = prefDlg.main->spinTryConnectTimeout->value();
	GlobalConfig->programsInPath = prefDlg.main->ProgramsInPathcheckBox->isChecked();
	GlobalConfig->holdGeneratedScripts = prefDlg.main->KeepConnectionFilesCheckBox->isChecked();

	if (prefDlg.main->DebugPppdcheckBox->isChecked())
		GlobalConfig->enableDebugPppd = true;
	else
		GlobalConfig->enableDebugPppd = false;

	InfoLogColor = prefDlg.main->InfoLogLabel->paletteBackgroundColor();
	RemoteLogColor = prefDlg.main->RemoteLogLabel->paletteBackgroundColor();
	ErrorLogColor = prefDlg.main->ErrorLogLabel->paletteBackgroundColor();
	SuccessLogColor = prefDlg.main->SuccessLogLabel->paletteBackgroundColor();
	DebugLogColor = prefDlg.main->DebugLogLabel->paletteBackgroundColor();
	GlobalConfig->useColorizedLogOutput = prefDlg.main->UseColorizedLogOutputcheckBox->isChecked();
	GlobalConfig->showInterfaceIPinTooltip = prefDlg.main->ShowInterfaceIPinTooltipCheckBox->isChecked();

	if ( GlobalConfig->writeLogFile )
	{
		if ( !logfile.isOpen() )
		{
			KStandardDirs * dirs = KGlobal::dirs();
			logfileName = dirs->saveLocation( "data" );
			logfileName += "/kvpnc/kvpnc.log";
			//KMessageBox::information( this, "info", logfileName );
			logfile.setName( logfileName );
			if ( !logfile.open( IO_WriteOnly | IO_Append ) )
				KMessageBox::error( this, i18n( "Log file cant be opened!" ) );
			else
			{
				s << i18n( "Log session started at: " ) << QDateTime::currentDateTime().toString( Qt::TextDate ) << endl;
			}
		}
	}
	else
	{
		if ( logfile.isOpen() )
		{
			QTextStream s( &logfile );
			s << i18n( "Log session ended at: " ) << QDateTime::currentDateTime().toString( Qt::TextDate ) << endl;
			logfile.close();
		}
	}
	saveOptions();
}

void KVpnc::slotShowToolbar()
{
	if ( !showToolbarAction->isChecked() )

		toolBar( "mainToolBar" ) ->hide();
	else
		toolBar( "mainToolBar" ) ->show();
}

void KVpnc::slotShowStatusbar()
{

	if ( !showStatusbarAction->isChecked() )
		statusBar() ->hide();
	else
		statusBar() ->show();
}

void KVpnc::slotKeyBindings()
{
	// KKeyDialog::configureKeys( actionCollection(), xmlFile() );
}

void KVpnc::slotConfigToolbars()
{
	KEditToolbar dlg( actionCollection() );

	if ( dlg.exec() )
	{
		createGUI();
	}
}

void KVpnc::slotConfigNotifications()
{
	KNotifyDialog::configure( this, i18n("Configure notifications") );
}

void KVpnc::slotStatusMsg( const QString &text, int id )
{
	if ( id != ID_FLASH_MSG )
	{
		statusBar()->clear();
		statusBar()->changeItem( text, id );
	}
	else
	{
		statusBar()->message( text, 2000 );
	}

}

void KVpnc:: quitCalled()
{
	bool reallyQuit = false;

	WindowSize = QSize( QWidget::width(), QWidget::height() );
	WindowPos = QPoint( this->x(), this->y() );

	if ( status == connected )
	{
		int res;
		if ( !GlobalConfig->useSilentQuit )
		{
			res = KMessageBox::questionYesNo ( this, i18n( "You are still conneced. Really quit?" ), i18n( "Quit?" ) );
			if ( res != 4 )
			{
				disconnectClicked();
				reallyQuit = true;
				if ( GlobalConfig->writeLogFile )
				{
					if ( logfile.isOpen() )
					{
						QTextStream s( &logfile );
						s << i18n( "Log session ended at: " ) << QDateTime::currentDateTime().toString( Qt::TextDate ) << endl;
						logfile.flush();
						logfile.close();
					}
				}
				kapp->quit();
			}
		}
		else
		{
			kapp->quit();
		}
	}
	else
	{
		reallyQuit = true;
		if ( reallyQuit )
		{
			if ( GlobalConfig->writeLogFile )
			{
				if ( logfile.isOpen() )
				{
					QTextStream s( &logfile );
					s << i18n( "Log session ended at: " ) << QDateTime::currentDateTime().toString( Qt::TextDate ) << endl;
					logfile.flush();
					logfile.close();
				}
			}
			kapp->quit();
		}
	}
}

void KVpnc::closeEvent( QCloseEvent* e )
{
	bool reallyQuit = false;
	if ( status == connected )
	{
		int res;
		if ( !GlobalConfig->useSilentQuit )
		{
			res = KMessageBox::questionYesNo ( this, i18n( "You are still conneced. Really quit?" ), i18n( "Quit?" ) );
			//KMessageBox::error( this,QString().setNum(res) );
			if ( res != 4 )
			{
				reallyQuit = true;
				disconnectClicked();

				if ( reallyQuit )
				{
					saveOptions();

					if ( GlobalConfig->writeLogFile )
					{
						if ( logfile.isOpen() )
						{
							QTextStream s( &logfile );
							s << i18n( "Log session ended at: " ) << QDateTime::currentDateTime().toString( Qt::TextDate ) << endl;
							logfile.close();
						}
					}
					KMainWindow::closeEvent( e );
				}
				else
				{
					e = 0L;
				}
			}
			else
			{
				e = 0L;
			}
		}
	}
	else
	{
		// not connected
		reallyQuit = true;
	}

	if ( reallyQuit )
	{
		saveOptions();
		if ( GlobalConfig->writeLogFile )
		{
			if ( logfile.isOpen() )
			{
				s << i18n( "Log session ended at: " ) << QDateTime::currentDateTime().toString( Qt::TextDate ) << endl;
				logfile.flush();
				logfile.close();
			}
		}
		//KMainWindow::closeEvent( e );
		kapp->quit();
	}
}

void KVpnc::connectClicked()
{
	bool ok = true; // if ok we will run process

	// type selection
	/* ============ vpnc ================ */
	if ( currentProfile != 0 && currentProfile->getConnectionType() == VpnAccountData::cisco )
	{

		//setGuiStatus(connecting);

		// looking for vpnc
		if ( GlobalConfig->useDefaultPaths || GlobalConfig->pathToVpnc.isEmpty() )
		{
			QFile vpnc( "/usr/sbin/vpnc" );
			if ( vpnc.exists() )
				GlobalConfig->pathToVpnc = "/usr/sbin/vpnc";
			else
			{
				vpnc.setName( "/usr/local/sbin/vpnc" );
				if ( vpnc.exists() )
					GlobalConfig->pathToVpnc = "/usr/local/sbin/vpnc";
				else
				{
					vpnc.setName( "/usr/bin/vpnc" );
					if ( vpnc.exists() )
						GlobalConfig->pathToVpnc = "/usr/bin/vpnc";
					else
					{
						KMessageBox::error( this, i18n( "Unable to find \"%1\"!" ).arg("vpnc") );
						ok = false;
					}
				}
			}
		}

		if ( GlobalConfig->KvpncDebugLevel > 0 )
			appendLogEntry( i18n( "vpnc: %1" ) .arg( GlobalConfig->pathToVpnc ),debug );


		if ( !Utils().tunDevExists() )
			if ( !Utils().createTunDev() )
			{
				KMessageBox::error( this, i18n( "Unable to create tunnel device file \"%1\"!" ).arg("/dev/net/tun") );
				ok=false;
			}
			else
				ok=true;

		TmpGatewayIP = Utils().resolveName(currentProfile->getGateway(), parent);
		if (TmpGatewayIP=="0.0.0.0")
		{
			KMessageBox::error( this, i18n( "Host \"%1\" could not be resolved!" ).arg(TmpGatewayIP) );
			ok =false;
		}
		else
		{
			if (GlobalConfig->KvpncDebugLevel > 0 )
				appendLogEntry( i18n( "Gateway hostname (%1) resolved to \"%2\"." ).arg(currentProfile->getGateway()).arg(TmpGatewayIP),debug );
		}

		if (ok)
		{
			if ( !currentProfile->getSaveUserPassword() || !currentProfile->getSavePsk() || AuthRetry)
			{
				bool requestUsername = currentProfile->getUserPassword().isEmpty();
				EnterUsernamePasswordDialog dlg(this, i18n("Enter account data..."), true, requestUsername);

				if (!requestUsername)
					dlg.main->UsernameEdit->setText(currentProfile->getUserName());
				
				dlg.main->LabelGroupPassword->setText(i18n("Enter group password:"));

				if( currentProfile->getSaveUserPassword() )
					dlg.main->PasswordEdit->setText(currentProfile->getUserPassword());

				if (currentProfile->getPrivateKey() ){
					dlg.main->PSKLineEdit->setText(currentProfile->getPreSharedKey());
				}

				if (dlg.exec())
				{

					if ( !currentProfile->getSaveUserPassword() || AuthRetry)
						TmpPassword= dlg.Password;

					if ( !currentProfile->getSavePsk() || AuthRetry)
						TmpGroupPassword= dlg.GroupPassword;

					currentProfile->setUserName(dlg.Username);

					if ( GlobalConfig->KvpncDebugLevel > 0 )
						appendLogEntry( i18n( "Some account data which are need got from password enter dialog." ),debug );

					if (currentProfile->getSavePsk() )
						currentProfile->setPreSharedKey(TmpGroupPassword);
					if (currentProfile->getSaveUserPassword())
						currentProfile->setUserPassword(TmpPassword);

					saveSessionClicked();
				}
				else
				{
					ok=false;
					AuthRetry=false;
					if ( GlobalConfig->KvpncDebugLevel > 0 )
						appendLogEntry( i18n( "Connect canceled because account data dialog aborted." ),debug );
				}
			}
		}


		// here we go
		if ( ok )
		{

			tmpPath = locateLocal ( "data", "kvpnc/" );

			QString tmpInterface;
			QString LocalIP;
			NetworkInterface iface(parent);
			if ( currentProfile->getNetworkDevice() == "default" )
			{
				tmpInterface = iface.getDefaultInterface();
				LocalIP = iface.getInterfaceIP( iface.getDefaultInterface() );
			}
			else
			{
				tmpInterface = currentProfile->getNetworkDevice();
				LocalIP = iface.getInterfaceIP( currentProfile->getNetworkDevice() );
			}
			if ( tmpInterface.isEmpty() )
			{
				tmpInterface = iface.getDefaultInterface();
				LocalIP = iface.getInterfaceIP( iface.getDefaultInterface() );
				if ( tmpInterface.isEmpty() ) {
					if ( GlobalConfig->KvpncDebugLevel > 0 ){
						appendLogEntry( i18n( "No default interface found, using \"lo\"." ),debug );
						tmpInterface = "lo"; // no default interface found :(
					}
				}
				else {
					appendLogEntry( i18n( "No default interface given, tried default interface, got success, using \"%1\"." ).arg(tmpInterface),debug );
				}
			}

			if ( LocalIP.isEmpty() )
			{
				if ( GlobalConfig->KvpncDebugLevel > 0 )
					appendLogEntry( i18n( "No IP for default interface found, using \"127.0.0.1\"." ),debug );
				LocalIP = "127.0.0.1"; // no IP for default interface found :(
			}

			//   QString tunneldevice = "tun0";
			int mtusize = 1412;
			OldDefaultroute = tmpPath + "default-route." + currentProfile->getName(); // default-route.<id>
			VpncGateway = tmpPath + "gateway." + currentProfile->getName(); // gateway.<id>
			VpncConnectScript = tmpPath + "vpnc-connect." + currentProfile->getName() + ".sh"; // vpnc-connect.<id>.sh
			Vpnc_pid_file = tmpPath + "vpnc-pid." + currentProfile->getName() + ".pid"; // vpnc-pid.<id>.pid

            // define 0.2.1
            int vpnc_version_major=0;
            int vpnc_version_minor=2;
            int vpnc_version_subminor=1;

            ToolInfo *Tool;
            for (Tool=GlobalConfig->ToolList->first();Tool;GlobalConfig->ToolList->next())
            {
                if (Tool->Name == "vpnc")
                {
                    if (!Tool->Version.isEmpty())
                    {
                        vpnc_version_major=(Tool->Version.section('.',0,0)).toInt();
                        vpnc_version_minor=(Tool->Version.section('.',1,1)).toInt();
                        vpnc_version_subminor=(Tool->Version.section('.',2,2)).toInt();
                        if (GlobalConfig->KvpncDebugLevel > 0){
                            appendLogEntry ( i18n( "vpnc version (major): \"%1\"" ).arg( vpnc_version_major ) , info);
                            appendLogEntry ( i18n( "vpnc version (minor): \"%1\"" ).arg( vpnc_version_minor ) , info);
                            appendLogEntry ( i18n( "vpnc version (subminor): \"%1\"" ).arg( vpnc_version_subminor ) , info);
                        }
                    }
                    break;
                }
            }

			// write vpnc connect script
			VpncScript = tmpPath + "vpnc-script." + currentProfile->getName(); // vpnc-script.<name>
			mw->DebugOutput->append( "VpncScript: " + VpncScript + "\n" );
			QFile file ( VpncScript );
			QTextStream stream( &file );
			if ( file.open( IO_WriteOnly ) )
			{
				stream << "#!/bin/bash" << "\n";
				stream << "# generated by kvpnc. Do not edit it." << "\n";
				stream << "# profile: " + currentProfile->getID() << "\n";
				stream << "" << "\n";
				stream << "#* VPNGATEWAY             -- vpn gateway address (always present)\n";
				stream << "#* TUNDEV                 -- tunnel device (always present)\n";
				stream << "#* INTERNAL_IP4_ADDRESS   -- address (always present)\n";
				stream << "#* INTERNAL_IP4_NETMASK   -- netmask (often unset)\n";
				stream << "#* INTERNAL_IP4_DNS       -- list of dns serverss\n";
				stream << "#* INTERNAL_IP4_NBNS      -- list of wins servers\n";
				stream << "#* CISCO_DEF_DOMAIN       -- default domain name\n";
				stream << "#* CISCO_BANNER           -- banner from server\n";
                stream << "\n";


                
                // min 0.3.3
                if (vpnc_version_minor >= 3 && vpnc_version_subminor >= 3   ) {
                    stream << "do_connect() {\n";
                }
				stream << "test -e /proc/sys/kernel/modprobe && `cat /proc/sys/kernel/modprobe` tun 2>/dev/null\n";
				stream << "\n";
				stream << "if [ $? != 0 ]; then\n";
   			stream << "# all this checks fail on a self-compiled kernel 2.4, though\n";
        stream << "( grep -q \" tun_init\" /proc/k*syms || test -e /sys/class/misc/tun || grep -q CONFIG_TUN=y /boot/config -`uname -r` ) 2>/dev/null || echo -e \"WARNING: failed to load the tun module\\n\"\n";
				stream << "fi\n";
	
				stream << "fix_ip_get_output () {\n";
				stream << "head -n 1 | sed 's/cache//;s/metric[0-9]\\+ [0-9]\\+//g' | xargs echo\n";
				stream << "}\n";
				stream << "\n";
				stream << "ifconfig $TUNDEV inet $INTERNAL_IP4_ADDRESS pointopoint $INTERNAL_IP4_ADDRESS netmask 255.255.255.255 mtu " << mtusize << " up\n";
				stream << "ip route add $(ip route get $VPNGATEWAY | fix_ip_get_output)\n";
				stream << "ip route | grep '^default' | fix_ip_get_output > \"" + OldDefaultroute + "\"\n";
				stream << "echo \"$VPNGATEWAY\" > \"" + VpncGateway + "\"\n";
				stream << "ip route del default\n";
				stream << "ip route add default dev $TUNDEV\n";

				stream << "if [ \"$INTERNAL_IP4_DNS\" ] ; then\n";
				stream << "	if test -x /sbin/resolvconf && [ \"$(getvar \"${config}\" \"DNSUpdate\")\" != \"no\" ] ; then\n";
				stream << "		(\n";
				stream << "			if [ \"$CISCO_DEF_DOMAIN\" ] ; then\n";
				stream << "				echo domain \"$CISCO_DEF_DOMAIN\"\n";
				stream << "				echo search \"$CISCO_DEF_DOMAIN\"\n";
				stream << "			fi\n";
				stream << "			for ip in \"$INTERNAL_IP4_DNS\" ; do\n";
				stream << "				echo nameserver $ip\n";
				stream << "			done\n";
				stream << "			) | /sbin/resolvconf -a $TUNDEV\n";
				stream << "			echo $TUNDEV > $dnsdev\n";
				stream << "	else\n";
				stream << "	# in case we have no /sbin/resolvconf handle /etc/resolv.conf manually\n";
				stream << "	mv /etc/resolv.conf /etc/resolv.conf.before.kvpnc."+currentProfile->getName()+"\n";
				stream << "	for i in $INTERNAL_IP4_DNS; do\n";
				stream << "		echo nameserver $i >> /etc/resolv.conf\n";
				stream << "	done\n";
				stream << "	fi\n";
				stream << "	for ip in $INTERNAL_IP4_DNS ; do\n";
				stream << "		ip route get $ip | grep -q \"dev $TUNDEV\" || ip route add $ip dev $TUNDEV\n";
				stream << "	done\n";
				stream << "fi\n";
				// end patch

				stream << "ip route flush cache\n";
                if (vpnc_version_minor >= 3 && vpnc_version_subminor >= 3   ) {
                    stream << "}\n";
                    stream << "\n";
                    stream << "case \"$reason\" in\n";
                    stream << " pre-init)\n";
                    stream << " #    do_pre_init\n";
                    stream << "     ;;\n";
                    stream << " connect)\n";
                    stream << "     do_connect\n";
                    stream << "     ;;\n";
                    stream << " disconnect)\n";
                    stream << "#        do_disconnect\n";
                    stream << "     ;;\n";
                    stream << " *)\n";
                    stream << "     echo \"unknown reason '$reason'. Maybe vpnc-script is out of date\" 1>&2\n";
                    stream << "     exit 1\n";
                    stream << "     ;;\n";
                    stream << "esac\n";
                    stream << "\n";
                }
			}

			file.close();

                // write vpnc.<profile>.conf # dummy!
		if (vpnc_version_minor > 2 ) {
                file.setName( tmpPath + "vpnc." + currentProfile->getName() + ".conf" );
                stream.setDevice( &file );
                if ( file.open( IO_WriteOnly ))
                {
                    stream << "# generated by kvpnc. Do not edit it." << "\n";
                    stream << "# profile: " + currentProfile->getName() << "\n";
                    stream << "" << "\n";
                    stream << "IPSec gateway " << TmpGatewayIP <<"\n";
		    stream << "IPSec ID " << currentProfile->getID() <<"\n";
		    QString GrpPassStr="";
		    if (!currentProfile->getSavePsk() || AuthRetry)
			GrpPassStr = TmpGroupPassword;
		    else
			GrpPassStr = currentProfile->getPreSharedKey();
		    
		    stream << "IPSec secret " << GrpPassStr << "\n";
		    stream << "Xauth username "<< currentProfile->getUserName() << "\n";
                }
                file.close();
            	stream.unsetDevice();
        	}

			ConnectProcess = new QProcess ( this );
			ConnectProcess->addArgument( "/bin/chmod" );
			ConnectProcess->addArgument( "+x" );
			ConnectProcess->addArgument( VpncScript );

			if ( ConnectProcess->start() )
			{

				sleep ( 1 );

				if (!Utils().isValidNetworkDevice("tun0"))
				{
					QFile tundevfile("/dev/net/tun");
					if (!tundevfile.exists())
						if (!Utils().createTunDev())
						{
							KMessageBox::error( this, i18n( "Creating of \"%1\" has been failed!" ).arg("/dev/net/tun"));
							appendLogEntry(i18n("Tunnel device is missing, creating has been failed: stop."),error);
							disconnectClicked();
							setGuiStatus(disconnected);
							status=disconnected;
							return;
						}
						else
						{
							appendLogEntry(i18n("Tunnel device is missing, creating has been succeded."),info);
						}

					// load the module 'tun'

					if (!Utils().loadKernelModule("tun",parent))
					{
						KMessageBox::information( this, i18n( "Loading of module \"%1\" failed!" ).arg("tun"));
						appendLogEntry(i18n("Tunnel device is missing, loading module has failed: stop."),error);
						disconnectClicked();
						setGuiStatus(disconnected);
						status=disconnected;
						return;
					}
				}
				ConnectProcess = new QProcess ( this );

				ConnectProcess->addArgument( GlobalConfig->pathToVpnc );
				// ConnectProcess->addArgument("--no-detach");
				ConnectProcess->addArgument( "--script" );
				ConnectProcess->addArgument( VpncScript );
				if ( GlobalConfig->VpncDebugLevel > 0 )
				{

					ConnectProcess->addArgument( "--debug" );
					ConnectProcess->addArgument( QString().setNum( GlobalConfig->VpncDebugLevel ) );
				}
				ConnectProcess->addArgument( "--pid-file" );
				ConnectProcess->addArgument( Vpnc_pid_file );
				//    ConnectProcess->addArgument( "--gateway" );
				//    ConnectProcess->addArgument( currentProfile->getGateway() );
				//    ConnectProcess->addArgument( "--id" );
				//    ConnectProcess->addArgument( currentProfile->getID() );
				//    ConnectProcess->addArgument( "--username" );
				//    ConnectProcess->addArgument( currentProfile->getUserName() );


				if ( currentProfile->getUseAdvancedSettings() )
				{

					if ( currentProfile->getUseApplicationVersion() )
					{
						ConnectProcess->addArgument( "--application-version" );
						ConnectProcess->addArgument( currentProfile->getApplicationVersion() );
						appendLogEntry ( i18n( "Using userdefinied application version \"%1\"." ).arg( currentProfile->getApplicationVersion() ) ,debug);
					}

					if ( currentProfile->getUseLocalPort() )
					{
						ConnectProcess->addArgument( "--local-port" );
						ConnectProcess->addArgument( QString().setNum( currentProfile->getLocalPort() ) );
						appendLogEntry ( i18n( "Using userdefinied local port \"%1\"." ).arg( QString().setNum( currentProfile->getLocalPort() ) ),info );
					}

					if ( currentProfile->getUseNtDomainName() )
					{
						ConnectProcess->addArgument( "--domain" );
						ConnectProcess->addArgument( currentProfile->getNtDomainName() );
						appendLogEntry ( i18n( "Using (NT) domain name \"%1\"." ).arg( currentProfile->getNtDomainName() ),info );
					}

					if ( currentProfile->getUsePerfectForwardSecrety() )
					{
						ConnectProcess->addArgument( "--pfs" );
						ConnectProcess->addArgument( currentProfile->getPerfectForwardSecrety() );
						appendLogEntry ( i18n( "Using userdefinied PFS \"%1\"." ).arg( currentProfile->getPerfectForwardSecrety() ),info );
					}

					if ( currentProfile->getUseIkeGroup() )
					{
						ConnectProcess->addArgument( "--dh" );
						ConnectProcess->addArgument( currentProfile->getIkeGroup() );
						appendLogEntry ( i18n( "Using userdefinied IKE group \"%1\"." ).arg( currentProfile->getIkeGroup() ),info );
					}

					if ( currentProfile->getUseSingleDes() )
					{
						ConnectProcess->addArgument( "--enable-1des" );
						appendLogEntry ( i18n( "Using single DES." ),info );
					}

					if ( currentProfile->getUseUdp() )
					{
						ConnectProcess->addArgument( "--udp" );
						appendLogEntry ( i18n( "Using UDP." ),info );
					}

					if ( currentProfile->getUseUdpPort() )
					{
						ConnectProcess->addArgument( "--udp-port" );
						ConnectProcess->addArgument( QString().setNum(currentProfile->getUdpPort()) );
						appendLogEntry ( i18n( "Using userdefinied UDP port \"%1.\"" ).arg(currentProfile->getUdpPort()),info );
					}

				}

                // vpnc >= 0.3.x needs a dummy config file as last argument
                if (vpnc_version_minor > 2 ) {
                    ConnectProcess->addArgument(tmpPath + "vpnc." + currentProfile->getName() + ".conf");
                }
			}
			connect( ConnectProcess, SIGNAL( readyReadStdout() ), this, SLOT( readFromStdout_vpnc() ) );
			connect( ConnectProcess, SIGNAL( readyReadStderr() ), this, SLOT( readFromStderr_vpnc() ) );



			appendLogEntry ( i18n( "Trying to connect to server \"%1\" (%2) with user \"%3\" and IPSec ID \"%4\"...\n" ).arg( currentProfile->getGateway() ).arg( TmpGatewayIP ).arg( currentProfile->getUserName() ).arg( currentProfile->getID() ),info );
			slotStatusMsg( i18n( "Connecting..." ), ID_CONNECTION_MSG );

			if ( !ConnectProcess->start() )
			{
				KMessageBox::sorry( this, i18n( "\"%1\" start failed!" ).arg( "vpnc" ) );
				disconnectClicked();
				setGuiStatus(disconnected);
			}
			else
			{

				appendLogEntry ( i18n( "\"%1\" started." ).arg("vpnc"),info );
				ConnectingProcess = VpnAccountData::cisco;
				status = connecting;
				mw->buttonDisconnect->setEnabled( true );
				this->disconnectAction->setEnabled( true );

				connect ( ConnectProcess, SIGNAL( processExited () ), this, SLOT( vpncStarted() ) );
				connect( &timer, SIGNAL( timeout() ), this, SLOT( slotConnectTimedOut() ) );
				timer.start( tryConnectTimeout * 1000, FALSE );

			}
		}
		else
		{
			setGuiStatus(disconnected);
		}
	} // vpnc

	/* ============ racoon ================ */
	else if ( currentProfile != 0 && currentProfile->getConnectionType() == VpnAccountData::racoon )
	{
		//setGuiStatus(connecting);
		// looking for racoon
		if ( GlobalConfig->useDefaultPaths || GlobalConfig->pathToRacoon.isEmpty() )
		{
			QFile vpnc( "/usr/sbin/racoon" );
			if ( vpnc.exists() )
				GlobalConfig->pathToRacoon = "/usr/sbin/racoon";
			else
			{
				vpnc.setName( "/usr/local/sbin/racoon" );
				if ( vpnc.exists() )
					GlobalConfig->pathToRacoon = "/usr/local/sbin/racoon";
				else
				{
					vpnc.setName( "/usr/bin/racoon" );
					if ( vpnc.exists() )
						GlobalConfig->pathToRacoon = "/usr/bin/racoon";
					else
					{
						KMessageBox::error( this, i18n( "Unable to find \"%1\"!" ).arg("racoon") );
						ok = false;
					}
				}
			}
		}

		// looking for setkey
		if ( GlobalConfig->useDefaultPaths || GlobalConfig->pathToSetkey.isEmpty() )
		{
			QFile vpnc( "/usr/sbin/setkey" );
			if ( vpnc.exists() )
				GlobalConfig->pathToSetkey = "/usr/sbin/setkey";
			else
			{
				vpnc.setName( "/usr/local/sbin/setkey" );
				if ( vpnc.exists() )
					GlobalConfig->pathToSetkey = "/usr/local/sbin/setkey";
				else
				{
					vpnc.setName( "/usr/bin/setkey" );
					if ( vpnc.exists() )
						GlobalConfig->pathToSetkey = "/usr/bin/setkey";
					else
					{
						KMessageBox::error( this, i18n( "Unable to find \"%1\"!" ).arg("setkey") );
						ok = false;
					}
				}
			}
		}

		// here we go
		if ( ok )
		{
			QString tmpInterface;
			QString LocalIP;
			NetworkInterface iface(parent);
			if ( currentProfile->getNetworkDevice() == "default" )
			{
				tmpInterface = iface.getDefaultInterface();
				LocalIP = iface.getInterfaceIP( iface.getDefaultInterface() );
			}
			else
			{
				tmpInterface = currentProfile->getNetworkDevice();
				LocalIP = iface.getInterfaceIP( currentProfile->getNetworkDevice() );
			}
			if ( tmpInterface.isEmpty() )
				tmpInterface = "lo"; // no default interface found :(

			if ( LocalIP.isEmpty() )
				LocalIP = "127.0.0.1"; // no IP for default interface found :(

			mw->DebugOutput->append( i18n( "default interface: %1" ).arg( tmpInterface ) );
			mw->DebugOutput->append( i18n( "Local IP address: %1" ).arg( LocalIP ) );

			QString Gateway = currentProfile->getGateway();
			QString RemoteNet = currentProfile->getRemoteNet();

			tmpPath = locateLocal ( "data", "kvpnc/" ); // should be
			// KTempDir tmpdir;
			// tmpPath = tmpdir.name();
			mw->DebugOutput->append( "tmppath: " + tmpPath );
			QString ProfileName = currentProfile->getName();
			QString CertPath = currentProfile->getCertPath();

			// write setkey.conf
			QFile file( tmpPath + "setkey." + ProfileName + ".conf" );
			QTextStream stream( &file );
			if ( file.open( IO_WriteOnly ) )
			{
				stream << "spdadd " + LocalIP + " " + Gateway + " any -P out ipsec esp/tunnel/" + LocalIP + "-" + Gateway + "/require;" << "\n";
				stream << "spdadd " + Gateway + " " + LocalIP + " any -P in ipsec esp/tunnel/" + Gateway + "-" + LocalIP + "/require;" << "\n";
				stream << "spdadd " + LocalIP + " " + RemoteNet + " any -P out ipsec esp/tunnel/" + LocalIP + "-" + Gateway + "/require;" << "\n";
				stream << "spdadd " + RemoteNet + " " + LocalIP + " any -P in ipsec esp/tunnel/" + Gateway + "-" + LocalIP + "/require;" << "\n";
			}
			file.close();

			// write racoon.<profile>.conf
			file.setName( tmpPath + "racoon." + ProfileName + ".conf" );
			if ( file.open( IO_WriteOnly ) )
			{
				stream.setDevice( &file );
				//     stream2 << "# generated by kvpnc. Do not edit it." << "\n";
				//     stream2 << "# profile: " + ProfileName << "\n";

				if ( currentProfile->getAuthType() == VpnAccountData::cert )
					stream << "path certificate \"" + currentProfile->getCertPath() + "\";" << "\n";

				if ( currentProfile->getAuthType() == VpnAccountData::psk )
					stream << "path pre_shared_key \"" + tmpPath + "psk." + ProfileName + ".key" + "\";" << "\n";

				stream << "" << "\n";
				stream << "remote " + Gateway + " {" << "\n";
				stream << "  exchange_mode main;" << "\n";

				if ( currentProfile->getAuthType() == VpnAccountData::cert )
				{
					stream << "  certificate_type x509 \"" << currentProfile->getX509Certificate() << "\" \"" << currentProfile->getX509Certificate() + "\";" << "\n";
					stream << "  verify_cert on;" << "\n";
				}
				if ( currentProfile->getAuthType() == VpnAccountData::cert )
				{
					stream << "  my_identifier asn1dn;" << "\n";
					stream << "  peers_identifier asn1dn;" << "\n";
				}
				if ( currentProfile->getUseUdp() )
				{

					stream << "   nat_traversal on;" << "\n";
					appendLogEntry ( i18n( "Using UDP." ),info );
				}
				stream << "  proposal {" << "\n";
				stream << "    encryption_algorithm 3des;" << "\n";
				stream << "    hash_algorithm md5;" << "\n";

				// x509
				if ( currentProfile->getAuthType() == VpnAccountData::cert )
					stream << "    authentication_method rsasig;" << "\n";
				if ( currentProfile->getAuthType() == VpnAccountData::psk )
					stream << "    authentication_method pre_shared_key;" << "\n";
				if ( currentProfile->getAuthType() == VpnAccountData::cert )
					if (!currentProfile->getUseIkeGroup())
					{
						stream << "    dh_group modp1024;" << "\n";
					}
					else
					{
						stream << "    dh_group "+currentProfile->getIkeGroup()+";" << "\n";
					}

				else
				{
					if (currentProfile->getUseAdvancedSettings() && currentProfile->getUseIkeGroup())
						stream << "    dh_group " << currentProfile->getIkeGroup() << ";\n";
					else
						stream << "    dh_group 2;" << "\n";
				}
				stream << "  }" << "\n";
				stream << "}" << "\n";
				stream << "" << "\n";
				stream << "sainfo address " << LocalIP << " any address " << currentProfile->getRemoteNet() << " any {" << "\n";

				if (currentProfile->getUseAdvancedSettings())
				{
					if (currentProfile->getUsePerfectForwardSecrety())
					{
						QString pfs_group="";
						if (currentProfile->getPerfectForwardSecrety() == "server")
							pfs_group = "";
						else if (currentProfile->getPerfectForwardSecrety() == "nopfs")
							pfs_group="";
						else if (currentProfile->getPerfectForwardSecrety() == "dh1")
							pfs_group="1";
						else if (currentProfile->getPerfectForwardSecrety() == "dh2")
							pfs_group="2";
						else if (currentProfile->getPerfectForwardSecrety() == "dh5")
							pfs_group="5";
						stream << "    dh group " << pfs_group << ";\n";
					}
					else
					{
						stream << "  pfs_group modp768;" << "\n";
					}
				}

				stream << "  encryption_algorithm 3des;" << "\n";
				stream << "  authentication_algorithm hmac_md5;" << "\n";
				stream << "  compression_algorithm deflate;" << "\n";
				stream << "}" << "\n";

				if ( currentProfile->getUseUdp() )
				{
					stream << "listen {" << "\n";
					//stream << "isakmp 192.168.1.37[500];" << "\n";
					if ( currentProfile->getUseUdpPort() )
						stream << "isakmp_natt " << LocalIP << "["<< currentProfile->getUdpPort() << "];" << "\n";
					else
						stream << "isakmp_natt " << LocalIP << "[4500]" << "\n";
					stream << "}" << "\n";
				}
			}
			file.close();
			stream.unsetDevice();

			// write setkey.<profile>.sh
			file.setName( tmpPath + "setkey." + ProfileName + ".sh" );
			stream.setDevice( &file );
			if ( file.open( IO_WriteOnly )
			   )
			{
				stream << "#!/bin/sh" << "\n";
				stream << "# generated by kvpnc. Do not edit it." << "\n";
				stream << "# profile: " + ProfileName << "\n";
				stream << "" << "\n";
				stream << "# Flush rules" << "\n";
				//     stream << "#  AH" << "\n";
				stream << GlobalConfig->pathToSetkey << " -F" << "\n";
				//     stream << "#  ESP" << "\n";
				stream << GlobalConfig->pathToSetkey << " -PF" << "\n";
				stream << "" << "\n";
				stream << "# run setkey" << "\n";
				stream << GlobalConfig->pathToSetkey << " -f " << tmpPath + "setkey." + currentProfile->getName() + ".conf" << "\n";
			}
			file.close();
			stream.unsetDevice();

			// iptables need the right interface (eth0:1 => eth0)
			QString tmpInterface2 = "";
			int pointPos = tmpInterface.find( ':', 0, FALSE );
			if ( pointPos > 1 )
			{
				tmpInterface2 = tmpInterface.left( pointPos );
			}
			else
				tmpInterface2 = tmpInterface;

			// write iptables.<profile>.sh
			file.setName( tmpPath + "iptables." + ProfileName + ".add_ipsec_racoon.sh" );
			stream.setDevice( &file );
			if ( file.open( IO_WriteOnly )
			   )
			{
				stream << "#!/bin/sh" << "\n";
				stream << "# generated by kvpnc. Do not edit it." << "\n";
				stream << "# profile: " + ProfileName << "\n";
				stream << "" << "\n";
				stream << "#  AH" << "\n";
				stream << GlobalConfig->pathToIptables << " -I INPUT -p 50 -i " << tmpInterface2 << " -s " << currentProfile->getGateway() << " -j ACCEPT" << "\n";
				stream << GlobalConfig->pathToIptables << " -I OUTPUT -p 50 -d " << currentProfile->getGateway() << " -j ACCEPT" << "\n";
				stream << "" << "\n";
				stream << "#  ESP" << "\n";
				stream << GlobalConfig->pathToIptables << " -I INPUT -p 51 -i " << tmpInterface2 << " -s " << currentProfile->getGateway() << " -j ACCEPT" << "\n";
				stream << GlobalConfig->pathToIptables << " -I OUTPUT -p 51 -d " << currentProfile->getGateway() << " -j  ACCEPT" << "\n";
				stream << "" << "\n";
				stream << "#  IKE" << "\n";
				stream << GlobalConfig->pathToIptables << " -I INPUT -p udp --dport 500 -i " << tmpInterface2 << " -s " << currentProfile->getGateway() << " -j ACCEPT" << "\n";
				stream << "" << "\n";
				stream << "#  NAT-T" << "\n";
				stream << GlobalConfig->pathToIptables << " -I INPUT -p udp --dport 4500 -i " << tmpInterface2 << " -s " << currentProfile->getGateway() << " -j ACCEPT" << "\n";

			}
			file.close();
			stream.unsetDevice();

			if ( currentProfile->getAuthType() == VpnAccountData::psk )
			{
				// write iptables.<profile>.sh
				file.setName( tmpPath + "psk." + ProfileName + ".key" );
				stream.setDevice( &file );
				if ( file.open( IO_WriteOnly ) )
				{
					stream << "# generated by kvpnc. Do not edit it." << "\n";
					stream << "# profile: " + ProfileName << "\n";
					stream << currentProfile->getGateway() + "\t" + currentProfile->getPreSharedKey() << "\n";
					file.close();
					stream.unsetDevice();
				}
			}

			// 			if (currentProfile->useVirtualIP()){
			// 				if (!setVirtualIP()){
			// 					setGuiStatus(disconnected);
			// 					return;
			// 				}
			//
			// 			}

			ConnectProcess = new QProcess ( this );
			//
			ConnectProcess->addArgument( "/bin/sh" );
			ConnectProcess->addArgument( tmpPath + "/setkey." + currentProfile->getName() + ".sh" );

			connect( ConnectProcess, SIGNAL( readyReadStdout() ), this, SLOT( readFromStdout_racoon() ) );
			connect( ConnectProcess, SIGNAL( readyReadStderr() ), this, SLOT( readFromStderr_racoon() ) );
			connect( ConnectProcess, SIGNAL( processExited () ), this, SLOT( doRouteRacoon() ) );

			if ( !ConnectProcess->start() )
			{
				KMessageBox::error( this, i18n( "\"%1\" start failed!" ).arg( "setkey."+ProfileName+".sh" ) );
				disconnectClicked();
				//setGuiStatus(disconnected);
			}
			else
			{
				ConnectingProcess = VpnAccountData::racoon;
				status = connecting;
				mw->buttonDisconnect->setEnabled( true );
				mw->buttonConnect->setEnabled(false);
			}
		}
		else
		{
			setGuiStatus(disconnected);
		}
	} // racoon

	/* ============ freeswan ================ */
	else if ( currentProfile != 0 && currentProfile->getConnectionType() == VpnAccountData::freeswan )
	{

		//setGuiStatus(connecting);

		// looking for ipsec
		if ( GlobalConfig->useDefaultPaths || GlobalConfig->pathToFreeswan.isEmpty() )
		{
			QFile vpnc( "/usr/sbin/ipsec" );
			if ( vpnc.exists() )
				GlobalConfig->pathToRacoon = "/usr/sbin/ipsec";
			else
			{
				vpnc.setName( "/usr/local/sbin/ipsec" );
				if ( vpnc.exists() )
					GlobalConfig->pathToRacoon = "/usr/local/sbin/ipsec";
				else
				{
					vpnc.setName( "/usr/bin/ipsec" );
					if ( vpnc.exists() )
						GlobalConfig->pathToRacoon = "/usr/bin/ipsec";
					else
					{
						KMessageBox::error( this, i18n( "Unable to find \"%1\"!" ).arg("freeswan (ipsec)") );
						ok = false;
					}
				}
			}
		}

		QString leftid = "";
		QString rightid = "";

		// x509
		if ( currentProfile->getAuthType() == VpnAccountData::cert )
		{
			leftid = getX509CertificateID( currentProfile->getX509Certificate(), true ); // local
			if ( currentProfile->getUseSpecialRemoteID() )
				rightid = currentProfile->getSpecialRemoteID();
			else
				rightid = getX509CertificateID( currentProfile->getX509Certificate(), false ); // remote

			if ( leftid.isEmpty() || rightid.isEmpty() )
			{
				KMessageBox::error( this, i18n( "Left or right ID is empty!" ) );
				ok = false;
			}
		}

		TmpGatewayIP = Utils().resolveName(currentProfile->getGateway(), parent);
		if (TmpGatewayIP=="0.0.0.0")
		{
			KMessageBox::error( this, i18n( "Host \"%1\" could not be resolved!" ).arg(TmpGatewayIP) );
			ok =false;
		}


		//   // here we go
		if ( ok )
		{
			QString tmpInterface;
			QString LocalIP;
			NetworkInterface iface(parent);
			if ( currentProfile->getNetworkDevice() == "default" )
			{
				tmpInterface = iface.getDefaultInterface();
				LocalIP = iface.getInterfaceIP( iface.getDefaultInterface() );
			}
			else
			{
				tmpInterface = currentProfile->getNetworkDevice();
				LocalIP = iface.getInterfaceIP( currentProfile->getNetworkDevice() );
			}
			if ( tmpInterface.isEmpty() )
				tmpInterface = "lo"; // no default interface found :(

			if ( LocalIP.isEmpty() )
				LocalIP = "127.0.0.1"; // no IP for default interface found :(

			mw->DebugOutput->append( "default interface: " + tmpInterface );
			mw->DebugOutput->append( "LocalIP: " + LocalIP );

			QString Gateway = currentProfile->getGateway();
			QString RemoteNet = currentProfile->getRemoteNet();

			tmpPath = locateLocal ( "data", "kvpnc/" ); // should be
			// KTempDir tmpdir;
			// tmpPath = tmpdir.name();
			mw->DebugOutput->append( "tmppath: " + tmpPath );
			QString ProfileName = currentProfile->getName();
			QString CertPath = currentProfile->getCertPath();
			QString PrivateKeyPass = currentProfile->getPrivateKeyPass();
			QString PrivateKey = currentProfile->getPrivateKey();

			// write /etc/ipsec.secrets
			//QFile file( tmpPath + "ipsec." + ProfileName + ".secrets" );
			QFile file( "/etc/ipsec.secrets" ); // warning: static
			QTextStream stream( &file );
			if ( file.open( IO_WriteOnly ) )
			{
				// cert
				if ( currentProfile->getAuthType() == VpnAccountData::cert )
				{
					stream << ": RSA " + PrivateKey + " \"" + PrivateKeyPass + "\"\n";
					//  mw->DebugOutput->append( "cert: private key: : RSA " + PrivateKey + " \"" + PrivateKeyPass );
				}
				// PSK
				else
				{
					stream << ": RSA " + PrivateKey + "\n";
					//  mw->DebugOutput->append( "cert: private key: "+PrivateKey);
				}

			}
			file.close();

			// write /etc/ipsec.conf
			//file.setName( tmpPath + "freeswan." + ProfileName + ".conf" );
			file.setName( "/etc/ipsec.conf" ); // warning: static
			if ( file.open( IO_WriteOnly ) )
			{
				int freeswan_version=1;
				ToolInfo *Tool;
				for (Tool=GlobalConfig->ToolList->first();Tool;GlobalConfig->ToolList->next()) {
					if (Tool->Name == "freeswan") {
						if (!Tool->Version.isEmpty()){
							if (GlobalConfig->KvpncDebugLevel > 0)
							freeswan_version=(Tool->Version.left(1)).toInt();
							appendLogEntry ( i18n( "FreeSWAN version: \"%n\"" ).arg( freeswan_version ) , info);
						}
						break;
					}
				}
					
				
				stream.setDevice( &file );
				stream << "# generated by kvpnc. Do not edit it." << "\n";
				stream << "# profile: " + ProfileName << "\n";
				stream << "" << "\n";

				if (freeswan_version == 2)
					stream << "version 2" << "\n";

				stream << "config setup" << "\n";
				stream << "    interfaces=%defaultroute" << "\n";
				stream << "    klipsdebug=none" << "\n";
				stream << "    plutodebug=none" << "\n";

				if (freeswan_version == 1){
					stream << "    plutoload=%search" << "\n";
					stream << "    plutostart=%search" << "\n";
				}
				stream << "" << "\n";
				stream << "conn %default" << "\n";
				stream << "    authby=rsasig" << "\n";

				// x509
				if ( currentProfile->getAuthType() == VpnAccountData::cert )
				{
					stream << "    leftrsasigkey=%cert" << "\n";
					stream << "    rightrsasigkey=%cert" << "\n";
				}
				if ( currentProfile->getAuthType() == VpnAccountData::psk )
				{
					stream << "    leftrsasigkey=" << currentProfile->getPrivateKey() << "\n";
					stream << " rightrsasigkey = " << currentProfile->getPreSharedKey() << "\n";
				}
				stream << "    left=%defaultroute" << "\n";
				stream << "    keyingtries=1" << "\n";
				stream << "    #keylife=1200s" << "\n";
				stream << "    #ikelifetime=1200s" << "\n";
				stream << "\n";
				stream << "conn roadwarrior-to-gw" << "\n";
				stream << "    auto=start" << "\n";
				stream << "    leftsubnet=" << "\n";
				stream << "    rightnexthop=" << "\n";
				//    stream << "    rightnexthop=" << "192.168.1.1"  << "\n";
				if ( currentProfile->getAuthType() == VpnAccountData::cert )
					stream << "    leftcert=" + currentProfile->getX509Certificate() << "\n";
				else
					stream << "    leftid = \"" + leftid + "\"" << "\n";
				//stream << "    right=" + currentProfile->getGateway() << "\n";
				stream << "    right=" + TmpGatewayIP << "\n";

				if (currentProfile->getUseSpecialServerCertificate())
					stream << "    rightcert="+currentProfile->getSpecialServerCertificate() << "\n";
				else
					stream << "    rightid=\"" + rightid + "\"" << "\n";
				
				//stream << "    rightid=\"/C=DE/ST=Sachsen-Anhalt/O=Powersoft/OU=IT/CN=212.100.100.212\"" << "\n";

			}

			stream << "  rightsubnet=" << currentProfile->getRemoteNet() << "\n";

			file.close();
			stream.unsetDevice();

			// iptables need the right interface (eth0:1 => eth0)
			QString tmpInterface2 = "";
			int pointPos = tmpInterface.find( ':', 0, FALSE );
			if ( pointPos > 1 )
			{
				tmpInterface2 = tmpInterface.left( pointPos );
			}
			else
				tmpInterface2 = tmpInterface;

			// write iptables.<profile>_add_ipsec_freeswan.sh
			file.setName( tmpPath + "iptables." + ProfileName + ".add_ipsec_freeswan.sh" );
			stream.setDevice( &file );
			if ( file.open( IO_WriteOnly ) )
			{
				stream << "#!/bin/sh" << "\n";
				stream << "# generated by kvpnc. Do not edit it." << "\n";
				stream << "# profile: " + ProfileName << "\n";
				stream << "" << "\n";
				stream << "#  AH" << "\n";
				stream << GlobalConfig->pathToIptables << " -I INPUT -p 50 -i " << tmpInterface2 << " -s " << currentProfile->getGateway() << " -j ACCEPT" << "\n";
				stream << GlobalConfig->pathToIptables << " -I OUTPUT -p 50 -d " << currentProfile->getGateway() << " -j ACCEPT" << "\n";
				stream << "" << "\n";
				stream << "#  ESP" << "\n";
				stream << GlobalConfig->pathToIptables << " -I INPUT -p 51 -i " << tmpInterface2 << " -s " << currentProfile->getGateway() << " -j ACCEPT" << "\n";
				stream << GlobalConfig->pathToIptables << " -I OUTPUT -p 51 -d " << currentProfile->getGateway() << " -j  ACCEPT" << "\n";
				stream << "" << "\n";
				stream << "#  IKE" << "\n";
				stream << GlobalConfig->pathToIptables << " -I INPUT -p udp --dport 500 -i " << tmpInterface2 << " -s " << currentProfile->getGateway() << " -j ACCEPT" << "\n";
				stream << "" << "\n";
				stream << "#  NAT-T" << "\n";
				stream << GlobalConfig->pathToIptables << " -I INPUT -p udp --dport 4500 -i " << tmpInterface2 << " -s " << currentProfile->getGateway() << " -j ACCEPT" << "\n";

			}
			file.close();
			stream.unsetDevice();

			if ( currentProfile->getAuthType() == VpnAccountData::psk )
			{
				// write iptables.<profile>.sh
				file.setName( tmpPath + "psk." + ProfileName + ".key" );
				stream.setDevice( &file );
				if ( file.open( IO_WriteOnly ) )
				{
					stream << currentProfile->getPreSharedKey() << "\n";
				}
				file.close();
				stream.unsetDevice();
			}


			// 			if (currentProfile->useVirtualIP()){
			// 				if (!setVirtualIP()){
			// 					setGuiStatus(disconnected);
			// 					return;
			// 				}
			// 			}

			ConnectProcess = new QProcess ( this );
			//
			ConnectProcess->addArgument( GlobalConfig->pathToFreeswan );
			ConnectProcess->addArgument( "setup" );
			ConnectProcess->addArgument( "start" );

			connect( ConnectProcess, SIGNAL( readyReadStdout() ), this, SLOT( readFromStdout_freeswan() ) );
			connect( ConnectProcess, SIGNAL( readyReadStderr() ), this, SLOT( readFromStderr_freeswan() ) );
			connect( ConnectProcess, SIGNAL( processExited () ), this, SLOT( doRouteFreeswan() ) );

			if ( !ConnectProcess->start() )
			{
				KMessageBox::error( this, i18n( "\"%1\" start failed!" ).arg( "freeswan (ipsec)" ) );
				disconnectClicked();
				setGuiStatus(disconnected);
			}
			else
			{
				freeswanStarted();

				connectionTime.start();
				connect( &durationTimer, SIGNAL( timeout() ), this, SLOT( slotDurationEvent() ) );
				ConnectingProcess = VpnAccountData::freeswan;
				status = connecting;
				mw->buttonDisconnect->setEnabled( true );
				durationTimer.start( 1000, FALSE );
				setGuiStatus(connected);
				status = connected;

				if ( currentProfile->getExecuteCmdAfterConnect())
					executeCommandAfterConnect();

				if (currentProfile->getUseAdditionalNetworkRoutes() && !currentProfile->getAdditionalNetworkRoutes().isEmpty())
					addAdditionalNetworkRoutes();

				if ( currentProfile->getDoPingIP() )
					sleep( 3 );
				doPingTest();
			}
		}
		else
		{
			setGuiStatus(disconnected);
		}
	} // freeswan

	/* ============ pptp ================ */
	else if ( currentProfile != 0 && currentProfile->getConnectionType() == VpnAccountData::pptp )
	{
		//setGuiStatus(connecting);

		// looking for pppd
		if ( GlobalConfig->useDefaultPaths || GlobalConfig->pathToPppd.isEmpty() )
		{
			QFile pppd( "/usr/sbin/pppd" );
			if ( pppd.exists() )
				GlobalConfig->pathToPppd = "/usr/sbin/pppd";
			else
			{
				pppd.setName( "/usr/local/sbin/pppd" );
				if ( pppd.exists() )
					GlobalConfig->pathToPppd = "/usr/local/sbin/pppd";
				else
				{
					pppd.setName( "/usr/bin/pppd" );
					if ( pppd.exists() )
						GlobalConfig->pathToPppd = "/usr/bin/pppd";
					else
					{
						KMessageBox::error( this, i18n( "Unable to find \"%1\"!" ).arg("pppd") );
						ok = false;
					}
				}
			}
		}

		if ( GlobalConfig->enableDebugPppd )
			appendLogEntry( i18n( "pppd: %1" ) .arg( GlobalConfig->pathToPppd ),debug );

		if ( !currentProfile->getSaveUserPassword() || currentProfile->getUserPassword().isEmpty())
		{
			bool requestUsername = currentProfile->getUserPassword().isEmpty();
			EnterUsernamePasswordDialog dlg(this, i18n("Enter account data..."),false,requestUsername); // only password

			if( currentProfile->getSaveUserPassword() )
				dlg.main->PasswordEdit->setText(currentProfile->getUserPassword());

			if (!currentProfile->getUserPassword().isEmpty())
				dlg.main->UsernameEdit->setText(currentProfile->getUserName());

			if (dlg.exec())
			{

				if ( !currentProfile->getSaveUserPassword())
					TmpPassword= dlg.Password;

				if ( GlobalConfig->PppdDebugLevel > 0 )
					appendLogEntry( i18n( "Some passwords which are need got from password enter dialog." ),debug );
			}
			else
			{
				ok=false;
				if ( GlobalConfig->PppdDebugLevel > 0 )
					appendLogEntry( i18n( "Connect canceled because password enter dialog aborted." ),debug );
			}
		}

		// here we go
		if ( ok )
		{

			tmpPath = locateLocal ( "data", "kvpnc/" );

			QString tmpInterface;
			QString LocalIP;
			NetworkInterface iface(parent);
			if ( currentProfile->getNetworkDevice() == "default" )
			{
				tmpInterface = iface.getDefaultInterface();
				LocalIP = iface.getInterfaceIP( iface.getDefaultInterface() );
			}
			else
			{
				tmpInterface = currentProfile->getNetworkDevice();
				LocalIP = iface.getInterfaceIP( currentProfile->getNetworkDevice() );
			}
			if ( tmpInterface.isEmpty() )
			{
				tmpInterface = iface.getDefaultInterface();
				LocalIP = iface.getInterfaceIP( iface.getDefaultInterface() );
				if ( tmpInterface.isEmpty() ) {
					if ( GlobalConfig->KvpncDebugLevel > 0 ){
						appendLogEntry( i18n( "No default interface found, using \"lo\"." ),debug );
						tmpInterface = "lo"; // no default interface found :(
					}
				}
				else {
					appendLogEntry( i18n( "No default interface given, tried default interface, got success, using \"%1\"." ).arg(tmpInterface),debug );
				}
			}

			if ( LocalIP.isEmpty() )
			{
				if ( GlobalConfig->PppdDebugLevel > 0 )
					appendLogEntry( i18n( "No IP for default interface found, using \"127.0.0.1\"." ),debug );
				LocalIP = "127.0.0.1"; // no IP for default interface found :(
			}

			OldDefaultroute = tmpPath + "default-route." + currentProfile->getName(); // default-route.<id>
			PptpGateway = tmpPath + "gateway." + currentProfile->getName(); // gateway.<id>
			Pppd_pid_file = "/var/run/ppp-"+currentProfile->getName()+".pid"; // /var/run/ppp-<id>.pid

			// write pppd peer script
			QString PppdPeerScript = "/etc/ppp/peers/"+currentProfile->getName(); // /etc/ppp/peers/<id>
			mw->DebugOutput->append( "pppd peer script: " + PppdPeerScript + "\n" );
			QFile file ( PppdPeerScript );
			QTextStream stream( &file );
			if ( file.open( IO_WriteOnly ) )
			{
				stream << "# generated by kvpnc. Do not edit it." << "\n";
				stream << "# profile: " + currentProfile->getName() << "\n";
				stream << "" << "\n";
				stream << "\n";
				stream << "# name of tunnel, used to select lines in secrets files\n";
				//stream << "remotename "+currentProfile->getName()+"\n";
				stream << "remotename pptp-server\n";
				stream << "\n";
				stream << "# name of tunnel, used to name /var/run pid file\n";
				stream << "linkname "+currentProfile->getName()+"\n";
				stream << "\n";
				stream << "# name of tunnel, passed to ip-up scripts\n";
				stream << "ipparam "+currentProfile->getName()+"\n";
				stream << "\n";
				stream << "# data stream for pppd to use\n";
				stream << "pty \"pptp "+currentProfile->getGateway()+" --nolaunchpppd\"\n";
				stream << "\n";
				stream << "# domain and username, used to select lines in secrets files\n";
				if (currentProfile->getUseAdvancedSettings() && currentProfile->getUseNtDomainName() && !currentProfile->getNtDomainName().isEmpty())
				{
					stream << "name "+currentProfile->getNtDomainName()+"\\\\"+currentProfile->getUserName()+"\n";
					appendLogEntry ( i18n( "Using (NT) domain name \"%1\"." ).arg( currentProfile->getNtDomainName() ),debug );
				}
				else
					stream << "name "+currentProfile->getUserName()+"\n";
				stream << "\n";

				if (currentProfile->getUseDnsServer() && !currentProfile->getDnsServer().isEmpty())
				{
					stream << "# set own dns server\n";
					stream << "ms-dns "+currentProfile->getDnsServer()+"\n";
					stream << "\n";
				}
				else
				{
					stream << "# retrieve DNS from peer\n";
					stream << "usepeerdns\n";
					stream << "\n";
				}

				if ( currentProfile->getRequireMppe())
				{
					stream << "# use MPPE compression\n";
					if (currentProfile->getRefuse128BitEncryption() || currentProfile->getRefuse40BitEncryption())
					{
						if (currentProfile->getRefuse128BitEncryption())
							stream << "nomppe-128\n";
						if (currentProfile->getRefuse40BitEncryption())
							stream << "nomppe-40\n";

						//stream << "require-mppe-40\n";
						stream << "require-mppe\n";
					}
					else
					{
						stream << "require-mppe-128\n";
					}
					stream << "\n";
				}
				else
				{
					stream  << "# disable MPPE compression\n";
					stream << "nomppe\n" ;
					stream << "\n";
				}

				if (currentProfile->getAllowStatefulMode())
				{
					stream << "# Allow MPPE stateful mode\n";
					stream << "mppe-stateful\n";
					stream << "\n";
				}

				if (GlobalConfig->enableDebugPppd)
				{
					stream << "# enable debug\n";
					stream << "debug\n";
					stream << "kdebug 1\n";
					stream << "\n";
				}
				stream << "# do not require the server to authenticate to our client\n";
				stream << "noauth\n";
				stream << "\n";

				stream << "# we want to see what happen\n";
				stream << "nodetach\n";
				stream << "\n";

				stream << "# lock the device\n";
				stream << "lock\n";
				stream << "\n";

				if (currentProfile->getUseNoBsdComp())
				{
					stream << "# Dont use BSD compression\n";
					stream << "nobsdcomp\n";
					stream << "\n";
				}
				else
				{
					stream << "# Use BSD compression\n";
					stream << "bsdcomp\n";
					stream << "\n";
				}

				if (currentProfile->getUseNoDeflate())
				{
					stream << "# Dont use deflate method\n";
					stream << "nodeflate\n";
					stream << "\n";
				}
				else
				{
					stream << "# Use deflate method\n";
					// FIXME make values dynamiclly
					stream << "deflate 9,9\n";
					stream << "\n";
				}


				if (currentProfile->getSetDefaultRoute())
				{
					stream << "defaultroute\n";
					// 					if (currentProfile->getReplaceDefaultRoute())
					// 					{
					// 						stream << "replacedefaultroute\n";
					// 					}
					stream <<"\n";
				}

				if (currentProfile->getUseMtu())
				{
					stream << "# userdefinied MTU\n";
					stream << "mtu "+QString().setNum(currentProfile->getMtu())+"\n";
					stream << "\n";
				}

				file.close();

				// write pppd /etc/ppp/chap-secrets
				// TODO save origal file
				QString ChapSecrets = "/etc/ppp/chap-secrets"; // /etc/ppp/chap-secrets
				mw->DebugOutput->append( QString("pppd chat script: /etc/ppp/chap-secrets\n") );
				file.setName ( ChapSecrets  );
				stream.setDevice( &file );
				if ( file.open( IO_WriteOnly ) )
				{
					stream << "# generated by kvpnc. Do not edit it." << "\n";
					stream << "# profile: " + currentProfile->getName() << "\n";
					stream << "# client server secret IP addresses" << "\n";
					if (currentProfile->getUseAdvancedSettings() && currentProfile->getUseNtDomainName() && !currentProfile->getNtDomainName().isEmpty())
					{
						stream << "PPTP " << currentProfile->getNtDomainName() << "\\\\" << currentProfile->getUserName()<<" *\n";
					}
					else
					{
						//   stream << currentProfile->getUserName() << " " << currentProfile->getName() << " " <<  currentProfile->getUserPassword() << " *\n";
						if (currentProfile->getSaveUserPassword())
						{
							stream << currentProfile->getUserName() << " " << "pptp-server" << " " <<  currentProfile->getUserPassword() << " *\n";
							stream << "pptp-server "  << currentProfile->getUserName() << " " <<  currentProfile->getUserPassword() << " *\n";
						}
						else
						{
							stream << currentProfile->getUserName() << " " << "pptp-server" << " " <<  TmpPassword << " *\n";
							stream << "pptp-server" << currentProfile->getUserName() << " " << " " <<  TmpPassword << " *\n";
						}

					}
                    file.close();
				}

				// load the module 'ppp_mppe'
				if (!Utils().loadKernelModule("ppp-compress-18",parent))
				{
					KMessageBox::information( this, i18n( "Loading of module \"%1\" failed!" ).arg("\"ppp-compress-18\" (ppp_mppe)"));
					appendLogEntry(i18n("Tunnel device is missing, loading module has failed: stop."),error);
					disconnectClicked();
				}
				else
				{

					sleep(2);
					mw->DebugOutput->append( "pppd: " + GlobalConfig->pathToPppd + "\n" );
					PptpProcess = new QProcess(this);

					PptpProcess->addArgument( GlobalConfig->pathToPppd );
					PptpProcess->addArgument( "call" );
					PptpProcess->addArgument( currentProfile->getName() );

					//      PptpProcess->addArgument( "/bin/sh" );
					//      PptpProcess->addArgument( "/home/crissi/pppd.sh" );

					connect( PptpProcess, SIGNAL( readyReadStdout() ), this, SLOT( readFromStdout_pptp() ) );
					connect( PptpProcess, SIGNAL( readyReadStderr() ), this, SLOT( readFromStderr_pptp() ) );


					appendLogEntry ( i18n( "Trying to connect to server \"%1\" with user \"%2\"...\n" ).arg( currentProfile->getGateway() ).arg( currentProfile->getUserName() ),debug );
					slotStatusMsg( i18n( "Connecting..." ), ID_CONNECTION_MSG );

					if ( !PptpProcess->start() )
					{
						KMessageBox::sorry( this, i18n( "\"%1\" start failed!" ).arg( "pppd" ) );
						disconnectClicked();
						setGuiStatus(disconnected);
					}
					else
					{
						appendLogEntry ( i18n( "\"%1\" started." ).arg("pppd"),info );
						ConnectingProcess = VpnAccountData::pptp;
						status = connecting;
						connect ( PptpProcess, SIGNAL( processExited () ), this, SLOT( pppdStarted() ) );

						//     connect( &timer, SIGNAL( timeout() ), this, SLOT( slotConnectTimedOut() ) );
						//     timer.start( tryConnectTimeout * 1000, FALSE );

						mw->buttonConnect->setEnabled( false );
						mw->buttonDisconnect->setEnabled( true );
                    }
				}
			}
		}
		else
		{
			setGuiStatus(disconnected);
		}

	} // pptp

	/* ============ openvpn ================ */
	else if ( currentProfile != 0 && currentProfile->getConnectionType() == VpnAccountData::openvpn )
	{
		//setGuiStatus(connecting);

		QString OpenvpnSecrectFile="";
		// looking for openvpn
		if ( GlobalConfig->useDefaultPaths || GlobalConfig->pathToOpenvpn.isEmpty() )
		{
			QFile openvpn( "/usr/sbin/openvpn" );
			if ( openvpn.exists() )
				GlobalConfig->pathToOpenvpn = "/usr/sbin/openvpn";
			else
			{
				openvpn.setName( "/usr/local/sbin/openvpn" );
				if ( openvpn.exists() )
					GlobalConfig->pathToOpenvpn = "/usr/local/sbin/openvpn";
				else
				{
					openvpn.setName( "/usr/bin/openvpn" );
					if ( openvpn.exists() )
						GlobalConfig->pathToOpenvpn = "/usr/bin/openvpn";
					else
					{
						KMessageBox::error( this, i18n( "Unable to find \"%1\"!" ).arg("openvpn") );
						ok = false;
					}
				}
			}
		}

		if ( GlobalConfig->OpenvpnDebugLevel > 0 )
			appendLogEntry( i18n( "openvpn: %1" ) .arg( GlobalConfig->pathToOpenvpn ) ,info);

		{

			// 			if ( !currentProfile->getSaveUserPassword() || !currentProfile->getSavePsk())
			// 			{
			// 				EnterPasswordDialog dlg(this, i18n("Enter password"), true,false);
			// 				dlg.main->LabelPassword->hide();
			// 				dlg.main->PasswordEdit->hide();
			// 				dlg.main->LabelGroupPassword->setText(i18n("Private key password:"));
			// 				if (dlg.exec())
			// 				{
			// 					if ( !currentProfile->getSavePsk())
			// 						TmpPrivKeyPassStr = dlg.GroupPassword;
			//
			// 					if ( GlobalConfig->OpenvpnDebugLevel > 0 )
			// 						appendLogEntry( i18n( "Private key password which is need got from password enter dialog." ),debug );
			// 				}
			// 				else
			// 				{
			// 					ok=false;
			// 					if ( GlobalConfig->KvpncDebugLevel > 0 )
			// 						appendLogEntry( i18n( "Connect canceled because password enter dialog aborted." ),debug );
			// 				}
			// 			}

			// here we go
			if ( ok )
			{
				mw->buttonConnect->setEnabled( false );
				tmpPath = locateLocal ( "data", "kvpnc/" );

				QString tmpInterface;
				QString LocalIP;
				NetworkInterface iface(parent);
				if ( currentProfile->getNetworkDevice() == "default" )
				{
					tmpInterface = iface.getDefaultInterface();
					LocalIP = iface.getInterfaceIP( iface.getDefaultInterface() );
				}
				else
				{
					tmpInterface = currentProfile->getNetworkDevice();
					LocalIP = iface.getInterfaceIP( currentProfile->getNetworkDevice() );
				}
				if ( tmpInterface.isEmpty() )
				{
					if ( GlobalConfig->KvpncDebugLevel > 0 )
						appendLogEntry( i18n( "No default interface found, using \"lo\"." ) ,info);
					tmpInterface = "lo"; // no default interface found :(
				}

				if ( LocalIP.isEmpty() )
				{
					if ( GlobalConfig->KvpncDebugLevel > 0 )
						appendLogEntry( i18n( "No IP for default interface found, using \"127.0.0.1\"." ),info );
					LocalIP = "127.0.0.1"; // no IP for default interface found :(
				}

				// 				appendLogEntry( "debug: local virtual ip: "+currentProfile->getLocalVirtualIP(),info );
				// 				appendLogEntry( "debug: remote virtual ip: "+currentProfile->getRemoteVirtualIP(),info );

				// write openvpn.<profile>.conf
				QFile file( tmpPath + "openvpn." + currentProfile->getName() + ".conf" );
				QTextStream stream( &file );
				if ( file.open( IO_WriteOnly ) )
				{
					stream << "# generated by kvpnc. Do not edit it." << "\n";
					stream << "# profile: " + currentProfile->getName() << "\n";
					stream << "" << "\n";
					stream << "# verbosity" << "\n";
					stream << "verb " << GlobalConfig->OpenvpnDebugLevel << "\n";
					stream << "" << "\n";
					stream << "# client side" << "\n";
					stream << "client" << "\n";
					stream << "" << "\n";
					stream << "# use a tun device" << "\n";
					stream << "dev tun" << "\n";
					stream << "" << "\n";
					stream << "# dont listen" << "\n";
					stream << "nobind" << "\n";
					stream << "" << "\n";
					stream << "# keep the connection alive" << "\n";
					stream << "keepalive 10 60" << "\n";
					stream << "" << "\n";
					stream << "# compression" << "\n";
					stream << "comp-lzo" << "\n";
					stream << "" << "\n";
					stream << "# try to resolve names infinite" << "\n";
					stream << "resolv-retry infinite" << "\n";
					stream << "" << "\n";
					stream << "# diffie hellman parameters" << "\n";
					stream << "dh dh1024.pem" << "\n";
					// 					stream << "" << "\n";
					// 					stream << "# ca cert" << "\n";
					// 					stream << "/etc/openvpn/" << "\n";
					stream << "" << "\n";
					if (currentProfile->getAuthType() == VpnAccountData::AuthenticationType(VpnAccountData::cert ))
					{
						stream << "# client cert" << "\n";
						stream << "cert " << currentProfile->getX509Certificate() << "\n";
						stream << "" << "\n";
						stream << "# client private key" << "\n";
						stream << "key " << currentProfile->getPrivateKey() << "\n";
						stream << "" << "\n";
						stream << "# ca" << "\n";
						stream << "ca " << currentProfile->getCaCertificate() << "\n";
					}
					else if (currentProfile->getAuthType() == VpnAccountData::AuthenticationType(VpnAccountData::psk ))
					{
						stream << "# client psk" << "\n";
						stream << "secret " << currentProfile->getPreSharedKeyFile() << "\n";
					}

					if ( !currentProfile->getUseUdp() )
					{
						stream << "" << "\n";
						stream << "# use tcp" << "\n";
						stream << "proto tcp" << "\n";
					}
					stream << "" << "\n";
					stream << "# remote side" << "\n";
					stream << "remote " << currentProfile->getGateway() << "\n";

					stream << "" << "\n";
					stream << "# tunnel device" << "\n";
					stream << "dev tun0 " << "\n";

					stream << "" << "\n";
					stream << "# tunnel device" << "\n";
					stream << "dev tun0 " << "\n";

					// 				stream << "" << "\n";
					// 				stream << "# manual ip config" << "\n";
					// 				stream << "ifconfig " << currentProfile->getLocalVirtualIP() << currentProfile->getRemoteVirtualIP() "\n";
					QString RemoteNetAddr = currentProfile->getRemoteNet().section( '/', 0, 0 );
					int RemoteNetMaskNum = currentProfile->getRemoteNet().section( '/', 1, 1 ).toInt();
					QString RemoteNetMask="";
					if (RemoteNetMaskNum == 24 )
						RemoteNetMask+="255.255.255.0";
					else if (RemoteNetMaskNum == 25 )
						RemoteNetMask+="255.255.255.128";
					else if (RemoteNetMaskNum == 26 )
						RemoteNetMask+="255.255.255.192";
					else if (RemoteNetMaskNum == 27 )
						RemoteNetMask+="255.255.255.224";
					else if (RemoteNetMaskNum == 28 )
						RemoteNetMask+="255.255.255.240";
					else if (RemoteNetMaskNum == 29 )
						RemoteNetMask+="255.255.255.248";
					else if (RemoteNetMaskNum == 30 )
						RemoteNetMask+="255.255.255.252";
					else if (RemoteNetMaskNum == 31 )
						RemoteNetMask+="255.255.255.254";
					else if (RemoteNetMaskNum == 32 )
						RemoteNetMask+="255.255.255.255";

					stream << "" << "\n";
					stream << "# route to remote network" << "\n";
					stream << "route " << RemoteNetAddr << " " << RemoteNetMask << "\n";

					stream << "" << "\n";
					stream << "# permissions" << "\n";
					stream << "user nobody" << "\n";
					stream << "group nogroup" << "\n";

					stream << "" << "\n";
					stream << "# port" << "\n";
					stream << "port 1194" << "\n";

					stream << "" << "\n";
					stream << "# don't re-read key files across SIGUSR1 or --ping-restart" << "\n";
					stream << "persist-key" << "\n";

					stream << "" << "\n";
					stream << "# don't close and reopen TUN/TAP device across SIGUSR1 or --ping-restart restarts" << "\n";
					stream << "persist-tun" << "\n";
				}
				file.close();


				OpenvpnProcess = new QProcess ( this );
				OpenvpnProcess->addArgument( GlobalConfig->pathToOpenvpn );

				OpenvpnProcess->addArgument( "--config" );
				OpenvpnProcess->addArgument( tmpPath + "openvpn." + currentProfile->getName() + ".conf" ); // default

				connect( OpenvpnProcess, SIGNAL( readyReadStdout() ), this, SLOT( readFromStdout_openvpn() ) );
				connect( OpenvpnProcess, SIGNAL( readyReadStderr() ), this, SLOT( readFromStderr_openvpn() ) );

				//appendLogEntry ( i18n( "Trying to connect to server \"%1\" with user \"%2\" and IPSec ID \"%3\"...\n" ).arg( currentProfile->getGateway() ). arg( currentProfile->getUserName() ).arg( currentProfile->getID() ) ,info);
				slotStatusMsg( i18n( "Connecting..." ), ID_CONNECTION_MSG );

				if ( !OpenvpnProcess->start() )
				{
					KMessageBox::sorry( this, i18n( "\"%1\" start failed!" ).arg( "openvpn" ) );
					disconnectClicked();
				}
				else
				{
					ConnectingProcess = VpnAccountData::openvpn;
					status = connecting;
					mw->buttonDisconnect->setEnabled( true );
					this->disconnectAction->setEnabled( true );

					appendLogEntry ( i18n( "\"%1\" started." ).arg("openvpn"),info );


					//connect ( OpenvpnProcess, SIGNAL( processExited () ), this, SLOT( openvpnStarted() ) );
					/*
										connect( &timer, SIGNAL( timeout() ), this, SLOT( slotConnectTimedOut() ) );
										timer.start( tryConnectTimeout * 1000, FALSE );*/
					// 					if ( currentProfile->getDoPingIP() )
					// 					{
					// 						sleep (3);
					// 						doPingTest();
					// 					}

				}
			} // openvpn
			// unsupported
			else
			{
				appendLogEntry ( i18n( "Unsupported connection type.\n" ),info );
			}
		}
	}
}

void KVpnc::disconnectClicked()
{
	if ( status == connected )
	{
		QString type_string = "";
		bool ok = true; // if ok we will run process

		// here we go
		if ( ok )
		{
			if ( currentProfile->getConnectionType() == VpnAccountData::cisco )
			{

				tmpPath = locateLocal ( "data", "kvpnc/" );
				//Vpnc_pid_file = tmpPath + "vpnc-pid." + currentProfile->getID() + ".pid"; // vpnc-pid.<id>.pid
				VpncDisconnectScript = tmpPath + "vpnc-disconnect." + currentProfile->getID() + ".sh";
				// write vpnc connect script
				QFile pidFile( Vpnc_pid_file );
				QTextStream pidFileStream( &pidFile );
				QString Vpnc_pid = "";
				if ( pidFile.open( IO_ReadOnly ) )
				{
					Vpnc_pid = pidFileStream.readLine();
				}

				QFile file( VpncDisconnectScript );
				QTextStream stream( &file );
				if ( file.open( IO_WriteOnly ) )
				{
					//     stream << "#!/bin/bash\n";
					stream << "# generated by kvpnc. Do not edit it." << "\n";
					stream << "# profile: " + currentProfile->getName() << "\n";
					stream << "\n";

					if ( !Vpnc_pid.isEmpty() )
					{
						if ( GlobalConfig->VpncDebugLevel > 0 )
							appendLogEntry( i18n( "Vpnc pid file found, killing process %1" ).arg( Vpnc_pid ) ,debug);
						stream << GlobalConfig->pathToKill << " -3 " << Vpnc_pid << "\n";
					}
					else
					{
						if ( GlobalConfig->KvpncDebugLevel > 0 )
							appendLogEntry( i18n( "No vpnc pid file found, using \"killall\" for killing vpnc." ),debug );
						stream << GlobalConfig->pathToKillall << " -3 kvpnc" << "\n";
					}

					stream << "ip route del default > /dev/null 2>&1\n";
					stream << "ip route add $(cat \"" + OldDefaultroute + "\")\n";
					stream << "ip route del $(cat \"" + VpncGateway + "\")\n";

					stream << "if [ -s \"$dnsdev\" ] && [ -e /sbin/resolvconf ] ; then\n";
					stream << "sbin/resolvconf -d `cat $dnsdev`\n";
					stream << "else\n";
					stream << "	mv /etc/resolv.conf.before.kvpnc."+currentProfile->getName()+" /etc/resolv.conf\n";
					stream << "fi\n";

					stream << "ip route flush cache\n";

				}
				file.close();

				if ( currentProfile->getExecuteCmdAfterDisconnect() )
					executeCommandAfterDisconnect();

				if (currentProfile->getUseAdditionalNetworkRoutes())
					removeAdditionalNetworkRoutes();

				DisconnectProcess = new QProcess ();
				DisconnectProcess->addArgument( "/bin/sh" );
				DisconnectProcess->addArgument( VpncDisconnectScript );

				if ( GlobalConfig->KvpncDebugLevel > 0 )
				{
					appendLogEntry ( i18n( "Setting %1 debug level %2." ).arg("vpnc").arg( QString().setNum( GlobalConfig->VpncDebugLevel ) ),info );
					DisconnectProcess->addArgument( "--debug" );
					DisconnectProcess->addArgument( QString().setNum( GlobalConfig->VpncDebugLevel ) );
				}

				connect( DisconnectProcess, SIGNAL( readyReadStdout() ), this, SLOT( readFromStdout_disconnect() ) );
				connect( DisconnectProcess, SIGNAL( readyReadStderr() ), this, SLOT( readFromStderr_disconnect() ) );

				if ( !DisconnectProcess->start() )
				{
					KMessageBox::sorry( this, i18n( "\"%1\" start failed!" ).arg( "vpnc_disconnect" ) );
				}
				else
				{
					setGuiStatus(disconnected);
					connectionEnded();

				}
			}

			// racoon
			else if ( currentProfile->getConnectionType() == VpnAccountData::racoon )
			{

				//     // looking for setkey
				//     if ( GlobalConfig->pathToSetkey.isEmpty() ) {
				//      QFile vpnc( "/usr/sbin/setkey" );
				//      if ( vpnc.exists() )
				//       GlobalConfig->pathToSetkey = "/usr/sbin/setkey";
				//      else {
				//       vpnc.setName( "/usr/local/sbin/setkey" );
				//       if ( vpnc.exists() )
				//        GlobalConfig->pathToSetkey = "/usr/local/sbin/setkey";
				//       else {
				//        vpnc.setName( "/usr/bin/setkey" );
				//        if ( vpnc.exists() )
				//         GlobalConfig->pathToSetkey = "/usr/bin/setkey";
				//        else {
				//         KMessageBox::error( this, i18n( "Unable to find \"%1\"!" ).arg("setkey") );
				//         ok = false;
				//        }
				//       }
				//      }
				//     }

				// iptables need the right interface (eth0:1 => eth0)
				QString tmpInterface2 = "";
				int pointPos = tmpInterface.find( ':', 0, FALSE );
				if ( pointPos > 1 )
				{
					tmpInterface2 = tmpInterface.left( pointPos );
				}
				else
					tmpInterface2 = tmpInterface;

				QFile file1( tmpPath + "iptables." + currentProfile->getName() + ".remove_ipsec_racoon.sh" );
				QTextStream stream1( &file1 );
				if ( file1.open( IO_WriteOnly ) )
				{
					stream1 << "#!/bin/sh" << "\n";
					stream1 << "# generated by kvpnc. Do not edit it." << "\n";
					stream1 << "# profile: " + currentProfile->getName() << "\n";
					stream1 << "" << "\n";
					stream1 << "#  AH" << "\n";
					stream1 << GlobalConfig->pathToIptables << " -D INPUT -p 50 -i " << tmpInterface2 << " -s " << currentProfile->getGateway() << " -j ACCEPT" << "\n";
					stream1 << GlobalConfig->pathToIptables << " -D OUTPUT -p 50 -d " << currentProfile->getGateway() << " -j ACCEPT" << "\n";
					stream1 << "" << "\n";
					stream1 << "#  ESP" << "\n";
					stream1 << GlobalConfig->pathToIptables << " -D INPUT -p 51 -i " << tmpInterface2 << " -s " << currentProfile->getGateway() << " -j ACCEPT" << "\n";
					stream1 << GlobalConfig->pathToIptables << " -D OUTPUT -p 51 -d " << currentProfile->getGateway() << " -j  ACCEPT" << "\n";
					stream1 << "" << "\n";
					stream1 << "#  IKE" << "\n";
					stream1 << GlobalConfig->pathToIptables << " -D INPUT -p udp --dport 500 -i " << tmpInterface2 << " -s " << currentProfile->getGateway() << " -j ACCEPT" << "\n";
					stream1 << "" << "\n";
					stream1 << "#  NAT-T" << "\n";
					stream1 << GlobalConfig->pathToIptables << " -D INPUT -p udp --dport 4500 -i " << tmpInterface2 << " -s " << currentProfile->getGateway() << " -j ACCEPT" << "\n";
				}
				file1.close();

				QFile file2( tmpPath + "setkey." + currentProfile->getName() + ".remove_setkey.sh" );
				QTextStream stream2( &file2 );
				if ( file2.open( IO_WriteOnly ) )
				{
					stream2 << "#!/bin/sh" << "\n";
					stream2 << "# generated by kvpnc. Do not edit it." << "\n";
					stream2 << "# profile: " + currentProfile->getName() << "\n";
					stream2 << GlobalConfig->pathToSetkey << " -PF" << "\n";
					stream2 << GlobalConfig->pathToSetkey << " -F" << "\n";
				}
				file2.close();

				if ( currentProfile->getExecuteCmdAfterDisconnect() )
					executeCommandAfterDisconnect();

				if (currentProfile->getUseAdditionalNetworkRoutes())
					removeAdditionalNetworkRoutes();

				if ( RacoonProcess != 0 )
				{
					terminateConnection( RacoonProcess );

				}

				removeSetkey();

				//     DisconnectProcess->clearArguments();
				//     DisconnectProcess->addArgument( GlobalConfig->pathToKillall );
				//     DisconnectProcess->addArgument( "-3" );
				//     DisconnectProcess->addArgument( "racoon" );
				//
				//     connect( DisconnectProcess, SIGNAL( readyReadStdout() ), this, SLOT( readFromStdout_racoon() ) );
				//     connect( DisconnectProcess, SIGNAL( readyReadStderr() ), this, SLOT( readFromStderr_racoon() ) );
				//     connect( DisconnectProcess, SIGNAL( processExited () ), this, SLOT( removeSetkey() ) );
				//
				//     if ( !DisconnectProcess->start() )
				//      KMessageBox::error( this, i18n( "\"%1\" start failed!" ).arg("killall -3 racoon") );
				//     else {
				//      TailProcess->tryTerminate();
				//      if ( GlobalConfig->RacoonDebugLevel > 0 )
				//       appendLogEntry( i18n( "\"%1\" was successful." ).arg("killall -3 racoon"),debug );
				//     }

			}

			// freeswan
			else if ( currentProfile->getConnectionType() == VpnAccountData::freeswan )
			{
				type_string = "freeswan";

				//     // looking for ipsec
				//     if ( GlobalConfig->pathToFreeswan.isEmpty() ) {
				//      QFile vpnc( "/usr/sbin/ipsec" );
				//      if ( vpnc.exists() )
				//       GlobalConfig->pathToRacoon = "/usr/sbin/ipsec";
				//      else {
				//       vpnc.setName( "/usr/local/sbin/ipsec" );
				//       if ( vpnc.exists() )
				//        GlobalConfig->pathToRacoon = "/usr/local/sbin/ipsec";
				//       else {
				//        vpnc.setName( "/usr/bin/ipsec" );
				//        if ( vpnc.exists() )
				//         GlobalConfig->pathToRacoon = "/usr/bin/ipsec";
				//        else {
				//         KMessageBox::error( this, i18n( "Unable to find \"%1\"!" ).arg("freeswan (ipsec)") );
				//         ok = false;
				//        }
				//       }
				//      }
				//     }

				// iptables need the right interface (eth0:1 => eth0)
				QString tmpInterface2 = "";
				int pointPos = tmpInterface.find( ':', 0, FALSE );
				if ( pointPos > 1 )
				{
					tmpInterface2 = tmpInterface.left( pointPos );
				}
				else
					tmpInterface2 = tmpInterface;

				QFile file1( tmpPath + "iptables." + currentProfile->getName() + ".remove_ipsec_freeswan.sh" );
				QTextStream stream1( &file1 );
				if ( file1.open( IO_WriteOnly ) )
				{
					stream1 << "#!/bin/sh" << "\n";
					stream1 << "# generated by kvpnc. Do not edit it." << "\n";
					stream1 << "# profile: " + currentProfile->getName() << "\n";
					stream1 << "" << "\n";
					stream1 << "#  AH" << "\n";
					stream1 << GlobalConfig->pathToIptables << " -D INPUT -p 50 -i " << tmpInterface2 << " -s " << currentProfile->getGateway() << " -j ACCEPT" << "\n";
					stream1 << GlobalConfig->pathToIptables << " -D OUTPUT -p 50 -d " << currentProfile->getGateway() << " -j ACCEPT" << "\n";
					stream1 << "" << "\n";
					stream1 << "#  ESP" << "\n";
					stream1 << GlobalConfig->pathToIptables << " -D INPUT -p 51 -i " << tmpInterface2 << " -s " << currentProfile->getGateway() << " -j ACCEPT" << "\n";
					stream1 << GlobalConfig->pathToIptables << " -D OUTPUT -p 51 -d " << currentProfile->getGateway() << " -j  ACCEPT" << "\n";
					stream1 << "" << "\n";
					stream1 << "#  IKE" << "\n";
					stream1 << GlobalConfig->pathToIptables << " -D INPUT -p udp --dport 500 -i " << tmpInterface2 << " -s " << currentProfile->getGateway() << " -j ACCEPT" << "\n";
					stream1 << "" << "\n";
					stream1 << "#  NAT-T" << "\n";
					stream1 << GlobalConfig->pathToIptables << " -D INPUT -p udp --dport 4500 -i " << tmpInterface2 << " -s " << currentProfile->getGateway() << " -j ACCEPT" << "\n";
				}
				file1.close();

				if ( currentProfile->getExecuteCmdAfterDisconnect() )
					executeCommandAfterDisconnect();

				if (currentProfile->getUseAdditionalNetworkRoutes())
					removeAdditionalNetworkRoutes();

				DisconnectProcess = new QProcess ( this );
				DisconnectProcess->addArgument( GlobalConfig->pathToFreeswan );
				DisconnectProcess->addArgument( "setup" );
				DisconnectProcess->addArgument( "stop" );

				connect( DisconnectProcess, SIGNAL( readyReadStdout() ), this, SLOT( readFromStdout_racoon() ) );
				connect( DisconnectProcess, SIGNAL( readyReadStderr() ), this, SLOT( readFromStderr_racoon() ) );
				connect( DisconnectProcess, SIGNAL( processExited () ), this, SLOT( removeRouteFreeswan() ) );

				if ( !DisconnectProcess->start() )
					KMessageBox::error( this, i18n( "\"%1\" start failed!" ).arg("ipsec setup stop") );

				else
				{
					if ( GlobalConfig->KvpncDebugLevel > 0 )
						appendLogEntry( i18n( "starting of \"%1\" was successful." ).arg("ipsec setup stop"),debug );

					setGuiStatus( disconnected );
				}
			}
			// pptp
			else if ( currentProfile->getConnectionType() == VpnAccountData::pptp )
			{
				disconnect( PptpProcess, SIGNAL( readyReadStdout() ), this, SLOT( readFromStdout_pptp() ) );
				disconnect( PptpProcess, SIGNAL( readyReadStderr() ), this, SLOT( readFromStderr_pptp() ) );
				appendLogEntry( i18n( "killing \"%1\"..." ).arg("pptp") ,debug);
				DisconnectProcess = new QProcess(this);
				DisconnectProcess->addArgument( GlobalConfig->pathToKillall );
				DisconnectProcess->addArgument( "-3" );
				DisconnectProcess->addArgument( "pptp" );

				connect( DisconnectProcess, SIGNAL( readyReadStdout() ), this, SLOT( readFromStdout_pptp() ) );
				connect( DisconnectProcess, SIGNAL( readyReadStderr() ), this, SLOT( readFromStderr_pptp() ) );

				if ( currentProfile->getExecuteCmdAfterDisconnect() )
					executeCommandAfterDisconnect();

				if (currentProfile->getUseAdditionalNetworkRoutes())
					removeAdditionalNetworkRoutes();

				if ( !DisconnectProcess->start() )
					KMessageBox::error( this, i18n( "\"%1\" start failed!" ).arg("killall -3 pptp") );
				else
				{
					if ( GlobalConfig->KvpncDebugLevel > 0 )
						appendLogEntry( i18n( "\"%1\" was successful." ).arg("killall -3 pptp"),debug );
				}

				timer.stop();
				durationTimer.stop();
				status = disconnected;
				connectCounter = 0;
				connectionEnded();
				setGuiStatus ( disconnected );

			}
			// openvpn
			else if ( currentProfile->getConnectionType() == VpnAccountData::openvpn )
			{
				disconnect( OpenvpnProcess, SIGNAL( readyReadStdout() ), this, SLOT( readFromStdout_openvpn() ) );
				disconnect( OpenvpnProcess, SIGNAL( readyReadStderr() ), this, SLOT( readFromStderr_openvpn() ) );
				appendLogEntry( i18n( "killing \"%1\"..." ).arg("openvpn"),debug );
				//     DisconnectProcess = new QProcess(this);
				//     DisconnectProcess->addArgument( GlobalConfig->pathToKill );
				//     DisconnectProcess->addArgument( "-3" );
				//     DisconnectProcess->addArgument( "openvpn" );
				//
				//     connect( DisconnectProcess, SIGNAL( readyReadStdout() ), this, SLOT( readFromStdout_pptp() ) );
				//     connect( DisconnectProcess, SIGNAL( readyReadStderr() ), this, SLOT( readFromStderr_pptp() ) );
				//
				//     if ( !DisconnectProcess->start() )
				//      KMessageBox::error( this, i18n( "\"%1\" start failed!" ).arg("killall -3 openvpn") );
				//     else
				//     {
				//      if ( GlobalConfig->KvpncDebugLevel > 0 )
				//       appendLogEntry( i18n( "\"killall -3 %1\" successful." ).arg("openvpn"),debug );
				//     }

				if ( currentProfile->getExecuteCmdAfterDisconnect() )
					executeCommandAfterDisconnect();

				if (currentProfile->getUseAdditionalNetworkRoutes())
					removeAdditionalNetworkRoutes();
				terminateConnection(OpenvpnProcess);
				connectionEnded();
				setGuiStatus(disconnected);
			}
		}
		ConnectingProcess = 0L;

	}
	else if (status == connecting)
	{
		if (GlobalConfig->KvpncDebugLevel > 0)
			appendLogEntry ( i18n( "Killing process while connecting.\n" ),debug );
		if (ConnectingProcess!= 0)
		{
			if (ConnectingProcess == VpnAccountData::cisco)
				ConnectProcess->kill();
			else if (ConnectingProcess == VpnAccountData::freeswan)
				FreeswanProcess->kill();
			else if (ConnectingProcess == VpnAccountData::racoon)
				RacoonProcess->kill();
			else if (ConnectingProcess == VpnAccountData::pptp)
				PptpProcess->kill();
			else if (ConnectingProcess == VpnAccountData::openvpn)
				OpenvpnProcess->kill();

		}
		//connectionEnded();
		setGuiStatus(disconnected);
		ConnectingProcess = -1;
	}
	else
		appendLogEntry ( i18n( "Not connected.\n" ),info );

}

void KVpnc::helpClicked()
{

	/*
	mw->DebugOutput->append( "getting userID:\n" );
	QString leftid = getX509CertificateID( "/etc/racoon/certs/mykeys_thielecke.pem", true ); // we
	mw->DebugOutput->append( "getting issuerID:\n" );
	QString rightid = getX509CertificateID( "/etc/racoon/certs/mykeys_thielecke.pem", false ); // remote
	*/
	//doPingTest();

// 	QString buggystring= "aaaaa::::bbbb    cccc$$$***//////";
// 	mw->DebugOutput->append(QString("Buggy string: "+buggystring));
// 	buggystring.replace(QRegExp("[*]+"),"_");
// 	buggystring.replace(QRegExp("[+] +"),"_");
// 	buggystring.replace(QRegExp("[$]+"),"_");
// 	buggystring.replace(QRegExp(":+"),"_");
// 	buggystring.replace(QRegExp("�"),"_");
// 	buggystring.replace(QRegExp("�"),"_");
// 	buggystring.replace(QRegExp("+"),"_");
// 	buggystring.replace(QRegExp("�"),"_");
// 	buggystring.replace(QRegExp("�"),"_");
// 	buggystring.replace(QRegExp("�"),"_");
// 	buggystring.replace(QRegExp("�"),"_");
// 	buggystring.replace(QRegExp("\\+"),"_");
// 	buggystring.replace(QRegExp("/+"),"_");
// 	buggystring.replace(QRegExp(";+"),"_");
// 	buggystring.replace(QRegExp(" "),"_");
// 	buggystring.replace(QRegExp("_+"),"_");
// 
// 	mw->DebugOutput->append(QString("Buggy string fixed: "+buggystring));

		HelpDialog d1;
		d1.exec();

	slotStatusMsg ( i18n( "Help dialog called." ), ID_FLASH_MSG );

	// 	QString host = "www.fdkldghgfdsfjhsjfgdjhsgfds.de";
	// 	QString ip = Utils().resolveName(host, parent);
	// 	if (ip=="0.0.0.0")
	// 		KMessageBox::error( this, i18n( "Host \"%1\" could not be resolved!" ).arg(host) );
	// 	else
	// 		mw->DebugOutput->append( QString("first ip of "+host+": "+ip ));
	//
	// 	QString host2 = "www.hs-harz.de";
	// 	QString ip2 = Utils().resolveName(host2, parent);
	// 	if (ip2=="0.0.0.0")
	// 		KMessageBox::error( this, i18n( "Host \"%1\" could not be resolved!" ).arg(host2) );
	// 	else
	// 		mw->DebugOutput->append( QString("first ip of "+host2+": "+ip2 ));

	// 	if (Utils().loadKernelModule("parport_pc",parent))
	// 		mw->DebugOutput->append( "Loading module ok" );
	// 	else
	// 		mw->DebugOutput->append( "Loading module failed." );

	//appHelpActivated ();

	/* network tests */
	/*	
	NetworkInterface iface(parent);
	bool success = iface.interfaceExist( "eth0" );
	QString succString;
	if ( success == true )
		mw->DebugOutput->append( "Testing interface eth0: success" );
	else
		mw->DebugOutput->append( "Testing interface eth0: failed" );

	QStringList list = iface.getAllNetworkInterfaces();

	for ( QStringList::Iterator it = list.begin(); it != list.end(); ++it ) {
		QString aa = QString( "Interface: " + *it );
		mw->DebugOutput->append( aa );
	}

	QString defaultinterface = "Defaultinterface: " + iface.getDefaultInterface();
	if ( defaultinterface.isEmpty() )
		defaultinterface += "none";

	mw->DebugOutput->append( defaultinterface );


	QString interfaceaddress = "Interface address: " + iface.getInterfaceAddress( "127.0.0.1" );
	if ( interfaceaddress.isEmpty() )
	interfaceaddress += "none";
	mw->DebugOutput->append( interfaceaddress );

	QString interfaceip = "Interface IP: " + iface.getInterfaceIP( "eth0" );
	if ( interfaceip.isEmpty() )
	interfaceip += "none";
	mw->DebugOutput->append( interfaceip );
	*/	

	/*
	QString ipstring = "aaa\ngot address 194.95.192.28\nbbbb\ncccc";
	mw->DebugOutput->append(QString("Ipstring: \n"+ipstring));
	QString ip = ipstring.section('\n',1,1);
	ip = ip.right(ip.length()-12);
	mw->DebugOutput->append(QString("Ip: "+ip));
	*/
}

void KVpnc::toolsInfoClicked()
{
	ToolsInfoDialog infodlg(GlobalConfig->ToolList);
	infodlg.exec();
}

void KVpnc::donateClicked()
{
	KRun *run=new KRun(KURL("http://home.gna.org/kvpnc/paypal.html"));
	if (run) ;
}

void KVpnc::readFromStderr_vpnc()
{
	//mw->DebugOutput->append( "readFromStderr_vpnc!\n" );
	// while ( ConnectProcess->canReadLineStderr() ) {
	//  ProcessMsg_connect = ConnectProcess->readLineStderr();
	ProcessMsg_connect = QString( ConnectProcess->readStderr() );

	/* TMP !!!! */
	// requesting group password
	if ( ProcessMsg_connect.find( "Enter IPSec secret", 0, FALSE ) > -1 )
	{
		if ( GlobalConfig->VpncDebugLevel > 0 )
			appendLogEntry ( i18n( "Group password requested, send it...\n" ),debug );



		QString GrpPassStr="";
		if (!currentProfile->getSavePsk() || AuthRetry)
			GrpPassStr = TmpGroupPassword;
		else
			GrpPassStr = currentProfile->getPreSharedKey();

		ConnectProcess->writeToStdin( GrpPassStr );
		ConnectProcess->writeToStdin( "\n" );

	}

	// requesting password
	if ( ProcessMsg_connect.find( "Enter password", 0, FALSE ) > -1 )
	{
		if ( GlobalConfig->VpncDebugLevel > 0 )
			appendLogEntry ( i18n( "User password requested, send it...\n" ),debug );

		QString UserPassStr="";
		if (!currentProfile->getSaveUserPassword() || AuthRetry)
			UserPassStr = TmpPassword;
		else
			UserPassStr = currentProfile->getUserPassword();


		ConnectProcess->writeToStdin( UserPassStr );
		ConnectProcess->writeToStdin( "\n" );
	}

	// requesting gatway
	if ( ProcessMsg_connect.find( "Enter IPSec gateway address", 0, FALSE ) > -1 )
	{
		if ( GlobalConfig->KvpncDebugLevel > 0 )
			appendLogEntry ( i18n( "IPSec gateway address requested, send it...\n" ),debug );
		//QString GatewayStr = currentProfile->getGateway();
		QString GatewayStr = TmpGatewayIP;
		ConnectProcess->writeToStdin( GatewayStr );
		ConnectProcess->writeToStdin( "\n" );
	}

	// requesting IPSec ID
	if ( ProcessMsg_connect.find( "Enter IPSec ID", 0, FALSE ) > -1 )
	{
		if ( GlobalConfig->VpncDebugLevel > 0 )
			appendLogEntry ( i18n( "IPSec ID requested, send it...\n" ),debug );
		QString IpsecIDStr = currentProfile->getID();
		ConnectProcess->writeToStdin( IpsecIDStr );
		ConnectProcess->writeToStdin( "\n" );
	}

	// requesting username
	if ( ProcessMsg_connect.find( "Enter username", 0, FALSE ) > -1 )
	{
		if ( GlobalConfig->VpncDebugLevel > 0 )
			appendLogEntry ( i18n( "Username requested, send it...\n" ),debug );
		QString UsernameStr = currentProfile->getUserName();
		ConnectProcess->writeToStdin( UsernameStr );
		ConnectProcess->writeToStdin( "\n" );
	}

// 	//got address 194.95.192.28
// 		if ( ProcessMsg_connect.find( "got address", 0, FALSE ) > -1 )
// 	{
// 		// "got address 194.95.192.28" => "194.95.192.28"
// 		TunnelInterfaceIP = ProcessMsg_connect.section('\n',1,1);
// 		TunnelInterfaceIP = TunnelInterfaceIP.right(TunnelInterfaceIP.length()-12);
// 
// 		if ( GlobalConfig->VpncDebugLevel > 0 )
// 			appendLogEntry ( i18n( "Tunnel IP: %1\n" ).arg(TunnelInterfaceIP),debug );
// 	}

	if ( !ProcessMsg_connect.isEmpty() )
		//    appendLogEntry ( ProcessMsg_connect + "\n" ,info);
		appendLogEntry ( ProcessMsg_connect ,debug);

	bool connectSuccess = false;
	if ( ProcessMsg_connect.find( "started in background", 0, FALSE ) > -1 )
		connectSuccess = true;

	if ( ProcessMsg_connect.find( "started in foreground", 0, FALSE ) > -1 )
		connectSuccess = true;

	if ( connectSuccess )
	{
		if ( GlobalConfig->KvpncDebugLevel > 0 )
			appendLogEntry( i18n( "Connection established." ) ,info);

		//connectionTime.restart();
		//uint duration = connectionTime.elapsed();

		sec = hour = min = 0;

		status = connected;

		timer.stop();


		connectionTime.start();
		connect( &durationTimer, SIGNAL( timeout() ), this, SLOT( slotDurationEvent() ) );
		durationTimer.start( 1000, FALSE );
		setGuiStatus(connected);
		connectSuccess = false;

		NetworkInterface iface(parent);
		TunnelInterfaceIP = iface.getInterfaceIP( "tun0" );
		if (GlobalConfig->VpncDebugLevel > 0 )
			appendLogEntry( i18n( "Tunnel IP:" ).arg(TunnelInterfaceIP) ,debug);

		if ( currentProfile->getExecuteCmdAfterConnect())
			executeCommandAfterConnect();

		if (currentProfile->getUseAdditionalNetworkRoutes() && !currentProfile->getAdditionalNetworkRoutes().isEmpty()){
			addAdditionalNetworkRoutes();
			//sleep (2);
		}
	}

	bool abort = false;
	bool doContinue=true;
	//ProcessMsg_connect = QString(ConnectProcess->readStdout());
	if ( this->GlobalConfig->VpncDebugLevel > 0 )
		this->appendLogEntry ( QString ( "raw stderr: " + ProcessMsg_connect ) ,error);


	if ( ProcessMsg_connect.find( "AUTHENTICATION_FAILED", 0, FALSE ) > -1 )
	{
		KMessageBox::error( this, i18n( "Authentication failed (%1)!" ).arg(i18n("group password") ));
		//abort = true;
		AuthRetry=true;
		disconnect( ConnectProcess, SIGNAL( readyReadStdout() ), this, SLOT( readFromStdout_vpnc() ) );
		disconnect( ConnectProcess, SIGNAL( readyReadStderr() ), this, SLOT( readFromStderr_vpnc() ) );
		connectClicked();
	}
	if ( ProcessMsg_connect.find( "authentication unsuccessful", 0, FALSE ) > -1 )
	{
		KMessageBox::error( this, i18n( "Authentication failed (%1)!" ).arg(i18n("user password")) );
		//abort = true;
		AuthRetry=true;
		disconnect( ConnectProcess, SIGNAL( readyReadStdout() ), this, SLOT( readFromStdout_vpnc() ) );
		disconnect( ConnectProcess, SIGNAL( readyReadStderr() ), this, SLOT( readFromStderr_vpnc() ) );
		connectClicked();
	}

	//FIXME made it obsolete!
	// no ip for gateway
	if ( ProcessMsg_connect.find( "vpnc: unknown host `<Replace with your gateway IP address>' ", 0, FALSE ) > -1 )
	{
		KMessageBox::error( this, i18n( "You have to enter a IP address for the remote gateway!" ) );
		abort = true;
	}

	// no answer
	if ( ProcessMsg_connect.find( "no response from target", 0, FALSE ) > -1 )
	{
		KMessageBox::error( this, i18n( "No Response from VPN server" ) );
		abort = true;
	}

	// initalizing tunnel interface failed
	if ( ProcessMsg_connect.find( "can't initialise tunnel interface", 0, FALSE ) > -1 )
	{
		KMessageBox::error( this, i18n( "Tunnel interface can't be initalized" ) );
		abort = true;
		doContinue=false;
	}


	// /dev/net/tun doesnt exist
	if ( ProcessMsg_connect.find( "can't open /dev/net/tun", 0, FALSE ) > -1 )
		if (doContinue)
		{
			KMessageBox::error( this, i18n( "Device file \"%1\" can't be opened" ).arg("/dev/net/tun") );
			abort = true;
		}

	// host unknown
	if ( ProcessMsg_connect.find( "unknown host" , 0, FALSE ) > -1 )
	{
		KMessageBox::error( this, i18n( "Host unknown" ) );
		abort = true;
	}

	//socket creation failed
	if ( ProcessMsg_connect.find( "making socket", 0, FALSE ) > -1 )
	{
		KMessageBox::error( this, i18n( "Socket creation failed" ) );
		abort = true;
	}


	// binding to port failed
	if ( ProcessMsg_connect.find( "binding to port", 0 , FALSE ) > -1 )
	{
		KMessageBox::error( this, i18n( "Port binding failed" ) );
		abort = true;
	}


	if ( ProcessMsg_connect.find( "Network is unreachable", 0 , FALSE ) > -1 )
	{
		KMessageBox::error( this, i18n( "No network reachable" ) );
		terminateConnection ( ConnectProcess );
	}

	if ( ProcessMsg_connect.find( "vpnc found running", 0 , FALSE ) > -1 )
	{
		//KMessageBox::sorry( this, i18n( "vpnc is already running, kill it manually" ) );
		//terminateConnection ( ConnectProcess );
		if ( GlobalConfig->KvpncDebugLevel > 0 )
		{
			appendLogEntry ( i18n( "Vpnc found running, killing it...\n" ) ,debug);
		}
		disconnectClicked();
		if ( GlobalConfig->KvpncDebugLevel > 0 )
			appendLogEntry ( i18n( "Trying again...\n" ),debug );
		connectClicked();
	}

	if ( abort )
	{
		if ( GlobalConfig->KvpncDebugLevel > 0 )
			appendLogEntry( i18n( "There is a reason for stop connecting, terminating \"%1\" process." ).arg("vpnc") ,debug);
		terminateConnection ( ConnectProcess );
	}

	// }
}

void KVpnc::readFromStdout_vpnc()
{
	//mw->DebugOutput->append( "readFromStdout_vpnc!\n" );

	//while ( ConnectProcess->canReadLineStdout() ) {
	// ProcessMsg_connect = ConnectProcess->readLineStdout();
	ProcessMsg_connect = QString( ConnectProcess->readStdout() );
	if ( !ProcessMsg_connect.isEmpty() )
		//    appendLogEntry ( ProcessMsg_connect + "\n",debug );
		if (GlobalConfig->VpncDebugLevel > 0)
			appendLogEntry ( ProcessMsg_connect ,debug);


	if ( ProcessMsg_connect.find( "got address", 0, FALSE ) > -1 )
	{
		// "got address 194.95.192.28" => "194.95.192.28"
		QString ipline="";
		for (int i=0;i<5;i++){
			ipline = ProcessMsg_connect.section('\n',i,i);
			if ( ipline.find( "got address", 0, FALSE ) > -1 ){
				//mw->DebugOutput->append(QString("line: "+ipline));
				TunnelInterfaceIP = ipline.section(' ',2,2);
				//mw->DebugOutput->append(QString("tunnel address: "+TunnelInterfaceIP));
				break;
			}
		}
		
		if ( GlobalConfig->VpncDebugLevel > 0 )
			appendLogEntry ( i18n( "Tunnel IP: %1\n" ).arg(TunnelInterfaceIP),debug );
	}

	bool connectSuccess = false;
	if ( ProcessMsg_connect.find( "started in background", 0, FALSE ) > -1 )
		connectSuccess = true;

	if ( ProcessMsg_connect.find( "started in foreground", 0, FALSE ) > -1 )
		connectSuccess = true;

	// success
	// background -> normal but no check if running :(
	// foreground -> stays in foreground and we can test if running :)
	if ( connectSuccess )
	{

		if ( GlobalConfig->KvpncDebugLevel > 0 )
			appendLogEntry( i18n( "Connection established." ),debug );

		//connectionTime.restart();
		//uint duration = connectionTime.elapsed();

		sec = hour = min = 0;

		status = connected;
		timer.stop();

		connectionTime.start();
		connect( &durationTimer, SIGNAL( timeout() ), this, SLOT( slotDurationEvent() ) );
		durationTimer.start( 1000, FALSE );
		setGuiStatus(connected);
		connectSuccess = false;

		if ( currentProfile->getExecuteCmdAfterConnect())
			executeCommandAfterConnect();

		if (currentProfile->getUseAdditionalNetworkRoutes()  && !currentProfile->getAdditionalNetworkRoutes().isEmpty())
			addAdditionalNetworkRoutes();
	}

	//ProcessMsg_connect = QString(ConnectProcess->readStdout());

	if ( this->GlobalConfig->KvpncDebugLevel > 0 )
		this->appendLogEntry ( QString ( "raw stdout: " + ProcessMsg_connect ),debug );

	if ( deviceaddrstr2addr )
	{
		deviceaddrstr = ProcessMsg_connect;
		//    this->appendLogEntry ( QString ( "deviceaddrstr: " + deviceaddrstr ),info );
		deviceaddr = "addr: " + QString( deviceaddrstr.simplifyWhiteSpace ().section( ' ', 1, 1 ) ).section( ':', 1, 1 );
		//    this->appendLogEntry ( deviceaddr,info ); //
		deviceaddrstr2addr = false;
		//disconnect( ConnectProcess, SIGNAL( readyReadStdout() ), this, SLOT( readFromStdout_vpnc() ) );
		//disconnect( ConnectProcess, SIGNAL( readyReadStderr() ), this, SLOT( readFromStderr_vpnc() ) );
	}

	// requesting group password
	if ( ProcessMsg_connect.find( "Enter IPSec secret", 0, FALSE ) > -1 )
	{
		if ( GlobalConfig->VpncDebugLevel > 0 )
			appendLogEntry ( i18n( "Group password requested, send it...\n" ),debug );



		QString GrpPassStr="";
		if (!currentProfile->getSavePsk() || AuthRetry)
			GrpPassStr = TmpGroupPassword;
		else
			GrpPassStr = currentProfile->getPreSharedKey();


		ConnectProcess->writeToStdin( GrpPassStr );
		ConnectProcess->writeToStdin( "\n" );
		
	}

	// requesting password
	if ( ProcessMsg_connect.find( "Enter password", 0, FALSE ) > -1 )
	{
		if ( GlobalConfig->VpncDebugLevel > 0 )
			appendLogEntry ( i18n( "User password requested, send it...\n" ),debug );

		QString UserPassStr="";
		if (!currentProfile->getSaveUserPassword() || AuthRetry)
			UserPassStr = TmpPassword;
	
		else
			UserPassStr = currentProfile->getUserPassword();


		ConnectProcess->writeToStdin( UserPassStr );
		ConnectProcess->writeToStdin( "\n" );
	}

	// requesting gatway
	if ( ProcessMsg_connect.find( "Enter IPSec gateway address", 0, FALSE ) > -1 )
	{
		if ( GlobalConfig->VpncDebugLevel > 0 )
			appendLogEntry ( i18n( "IPSec gateway address requested, send it...\n" ),debug );
		//QString GatewayStr = currentProfile->getGateway();
		QString GatewayStr = TmpGatewayIP;
		ConnectProcess->writeToStdin( GatewayStr );
		ConnectProcess->writeToStdin( "\n" );
	}

	// requesting IPSec ID
	if ( ProcessMsg_connect.find( "Enter IPSec ID", 0, FALSE ) > -1 )
	{
		if ( GlobalConfig->VpncDebugLevel > 0 )
			appendLogEntry ( i18n( "IPSec ID requested, send it...\n" ),debug );
		QString IpsecIDStr = currentProfile->getID();
		ConnectProcess->writeToStdin( IpsecIDStr );
		ConnectProcess->writeToStdin( "\n" );
	}

	// requesting username
	if ( ProcessMsg_connect.find( "Enter username", 0, FALSE ) > -1 )
	{
		if ( GlobalConfig->VpncDebugLevel > 0 )
			appendLogEntry ( i18n( "Username requested, send it...\n" ),debug );
		QString UsernameStr = currentProfile->getUserName();
		ConnectProcess->writeToStdin( UsernameStr );
		ConnectProcess->writeToStdin( "\n" );
	}

	if ( ProcessMsg_connect.find( "vpnc found running", 0 , FALSE ) > -1 )
	{
		//KMessageBox::sorry( this, i18n( "vpnc is already running, kill it manually" ) );
		//terminateConnection ( ConnectProcess );
		if ( GlobalConfig->VpncDebugLevel > 0 )
		{
			appendLogEntry ( i18n( "Vpnc found running, killing it...\n" ),debug );
		}
		disconnectClicked();
		if ( GlobalConfig->KvpncDebugLevel > 0 )
			appendLogEntry ( i18n( "Trying again...\n" ),debug );
		connectClicked();
	}

	slotStatusMsg ( i18n( "Ready." ), ID_STATUS_MSG );
	// }
}

void KVpnc::readFromStderr_disconnect()
{

	ProcessMsg_disconnect = DisconnectProcess->readLineStderr();
	if ( !ProcessMsg_disconnect.isEmpty() )
		appendLogEntry ( ProcessMsg_disconnect + "\n",info );
}

void KVpnc::readFromStdout_disconnect()
{
	ProcessMsg_disconnect = DisconnectProcess->readLineStdout();

	if ( !ProcessMsg_disconnect.isEmpty() )
		appendLogEntry ( ProcessMsg_disconnect + "\n",info );
}

void KVpnc::readFromStdout_racoon()
{
	//while ( ConnectProcess->canReadLineStdout() ) {
	//  ProcessMsg_connect = ConnectProcess->readLineStdout();
	QString ProcessMsg_connect = QString( ConnectProcess->readStdout() );

	if ( pingtest )
	{
		//   if ( GlobalConfig->KvpncDebugLevel > 0 )
		//    mw->DebugOutput->append( "raw out (pingtest): " + ProcessMsg_connect );

		if ( ProcessMsg_connect.find( "time=", 0, FALSE ) > -1 )
		{
			mw->DebugOutput->append( i18n( "out: success: <font color=\"green\">" ) + ProcessMsg_connect + "</font>" );
		}
		else if ( ProcessMsg_connect.find( "100 % packet loss", 0, FALSE ) > -1 )
		{
			mw->DebugOutput->append( i18n( "out: failed: <font color=\"red\">" ) + ProcessMsg_connect + "</font>" );
		}
		else
		{
			if ( ProcessMsg_connect.find( "failed to bind", 0, FALSE ) > -1 )
			{
				KMessageBox::error( this, i18n( "Binding interface failed!" ) );
			}
			if ( ProcessMsg_connect.find( "Destination Host Unreachable", 0, FALSE ) > -1 )
			{
				KMessageBox::error( this, i18n( "Destination host is unreachable!" ) );
			}


			mw->DebugOutput->append( "out: " + ProcessMsg_connect );
		}
	}

	// normal
	else
	{
		mw->DebugOutput->append( "out: " + ProcessMsg_connect );
	}
	//}
}

void KVpnc::readFromStderr_racoon()
{
	// while ( ConnectProcess->canReadLineStderr() ) {
	//  ProcessMsg_connect = ConnectProcess->readLineStderr();
	QString ProcessMsg_connect = QString( ConnectProcess->readStderr() );
	mw->DebugOutput->append( "err: " + ProcessMsg_connect );

	if ( ProcessMsg_connect.find( "failed to bind", 0, FALSE ) > -1 )
	{
		KMessageBox::error( this, i18n( "Binding interface failed!" ) );
	}

	if ( ProcessMsg_connect.find( "no default route, %defaultroute cannot cope!!!", 0, FALSE ) > -1 )
	{
		KMessageBox::error( this, i18n( "No default route found, nesessary for freeswan!" ) );
		disconnectClicked();
	}

	if ( ProcessMsg_connect.find( "SIOCADDRT: File exists", 0, FALSE ) > -1 )
	{
		mw->DebugOutput->append( i18n( "err: setting route failed: route still exists" ) );
	}

	if ( ProcessMsg_connect.find( "SIOCDELRT: File exists", 0, FALSE ) > -1 )
	{
		mw->DebugOutput->append( i18n( "err: deleting route failed: route dont exists" ) );
	}

	if ( pingtest )
	{
		// mw->DebugOutput->append( "raw err (pingtest): " + ProcessMsg_connect );
		if ( ProcessMsg_connect.find( "time=", 0, FALSE ) > -1 )
		{
			mw->DebugOutput->append( i18n( "out: success: <font color=\"green\">" ) + ProcessMsg_connect + "</font>" );
		}
		else if ( ProcessMsg_connect.find( "100 % packet loss", 0, FALSE ) > -1 )
		{
			mw->DebugOutput->append( i18n( "out: failed: <font color=\"red\">" ) + ProcessMsg_connect + "</font>" );
		}
		else
		{
			if ( ProcessMsg_connect.find( "failed to bind", 0, FALSE ) > -1 )
			{
				KMessageBox::error( this, i18n( "Binding interface failed!" ) );
			}
			if ( ProcessMsg_connect.find( "Destination Host Unreachable", 0, FALSE ) > -1 )
			{
				KMessageBox::error( this, i18n( "Destination host is unreachable!" ) );
			}
		}
	}
}

void KVpnc::readFromStdout_freeswan()
{
	//while ( ConnectProcess->canReadLineStdout() ) {
	//  ProcessMsg_connect = ConnectProcess->readLineStdout();
	QString ProcessMsg_connect = QString( ConnectProcess->readStdout() );

	if ( pingtest )
	{
		//   if ( GlobalConfig->KvpncDebugLevel > 0 )
		//    mw->DebugOutput->append( "raw out (pingtest): " + ProcessMsg_connect );

		if ( ProcessMsg_connect.find( "time=", 0, FALSE ) > -1 )
		{
			mw->DebugOutput->append( i18n( "out: success: <font color=\"green\">" ) + ProcessMsg_connect + "</font>" );
		}
		else if ( ProcessMsg_connect.find( "100 % packet loss", 0, FALSE ) > -1 )
		{
			mw->DebugOutput->append( i18n( "out: failed: <font color=\"red\">" ) + ProcessMsg_connect + "</font>" );
		}
		else
		{
			if ( ProcessMsg_connect.find( "failed to bind", 0, FALSE ) > -1 )
			{
				KMessageBox::error( this, i18n( "Binding interface failed!" ) );
			}
			if ( ProcessMsg_connect.find( "Destination Host Unreachable", 0, FALSE ) > -1 )
			{
				KMessageBox::error( this, i18n( "Destination host is unreachable!" ) );
			}


			mw->DebugOutput->append( "out: " + ProcessMsg_connect );
		}
	}

	// normal
	else
	{
		mw->DebugOutput->append( "out: " + ProcessMsg_connect );
	}
	//}
}

void KVpnc::readFromStderr_freeswan()
{
	// while ( ConnectProcess->canReadLineStderr() ) {
	//  ProcessMsg_connect = ConnectProcess->readLineStderr();
	QString ProcessMsg_connect = QString( ConnectProcess->readStderr() );
	mw->DebugOutput->append( "err: " + ProcessMsg_connect );

	if ( ProcessMsg_connect.find( "failed to bind", 0, FALSE ) > -1 )
	{
		KMessageBox::error( this, i18n( "Binding interface failed!" ) );
	}

	if ( ProcessMsg_connect.find( "no default route, %defaultroute cannot cope!!!", 0, FALSE ) > -1 )
	{
		KMessageBox::error( this, i18n( "No default route found, nesessary for freeswan!" ) );
		disconnectClicked();
	}

	if ( ProcessMsg_connect.find( "SIOCADDRT: File exists", 0, FALSE ) > -1 )
	{
		mw->DebugOutput->append( i18n( "err: setting route failed: route still exists" ) );
	}

	if ( ProcessMsg_connect.find( "SIOCDELRT: File exists", 0, FALSE ) > -1 )
	{
		mw->DebugOutput->append( i18n( "err: deleting route failed: route dont exists" ) );
	}

	if ( ProcessMsg_connect.find( "we only support version 2", 0, FALSE ) > -1 )
	{
		mw->DebugOutput->append( i18n( "err: only freeswan version 1.x is supported " ) );
		disconnectClicked();
	}

	if ( pingtest )
	{
		// mw->DebugOutput->append( "raw err (pingtest): " + ProcessMsg_connect );
		if ( ProcessMsg_connect.find( "time=", 0, FALSE ) > -1 )
		{
			mw->DebugOutput->append( i18n( "out: success: <font color=\"green\">" ) + ProcessMsg_connect + "</font>" );
		}
		else if ( ProcessMsg_connect.find( "100 % packet loss", 0, FALSE ) > -1 )
		{
			mw->DebugOutput->append( i18n( "out: failed: <font color=\"red\">" ) + ProcessMsg_connect + "</font>" );
		}
		else
		{
			if ( ProcessMsg_connect.find( "failed to bind", 0, FALSE ) > -1 )
			{
				KMessageBox::error( this, i18n( "Binding interface failed!" ) );
			}
			if ( ProcessMsg_connect.find( "Destination Host Unreachable", 0, FALSE ) > -1 )
			{
				KMessageBox::error( this, i18n( "Destination host is unreachable!" ) );
			}
		}
	}
}


void KVpnc::readFromStdout_tail_racoon()
{
	//QString ProcessMsg_connect = QString( TailProcess->readStdout() );
	while ( TailProcess->canReadLineStdout() )
	{
		QString ProcessMsg_connect = TailProcess->readLineStdout();
		if ( GlobalConfig->RacoonDebugLevel > 0 )
		{
			mw->DebugOutput->append( "debug racoon: <font color=\"orange\">" + ProcessMsg_connect+"</font>" );
		}
	}
}

void KVpnc::readFromStderr_tail_racoon()
{
	//QString ProcessMsg_connect = QString( TailProcess->readStderr() );
	while ( TailProcess->canReadLineStdout() )
	{
		QString ProcessMsg_connect = TailProcess->readLineStdout();
		if ( GlobalConfig->RacoonDebugLevel > 0 )
		{
			mw->DebugOutput->append( "debug racoon (err): <font color=\"red\">"  + ProcessMsg_connect+"</font>" );
		}
	}
}

void KVpnc::readFromStdout_pptp()
{
	ProcessMsg_connect = QString( PptpProcess->readStdout() );
	//if ( !ProcessMsg_connect.isEmpty() )
	appendLogEntry ( ProcessMsg_connect,info );

	bool abort = false;


	if ( ProcessMsg_connect.find( "could not find module", 0 , FALSE ) > -1 )
	{
		KMessageBox::error( this, i18n( "Module not found." ) );
		abort = true;
	}

	if ( ProcessMsg_connect.find( "Connection terminated.", 0 , FALSE ) > -1 )
	{
		//KMessageBox::error( this, i18n( "Connection has been terminated." ) );
		abort = true;
	}

	if ( ProcessMsg_connect.find( "No route to host", 0 , FALSE ) > -1 )
	{
		KMessageBox::error( this, i18n( "Host not found." ) );
		abort = true;
	}

	if ( ProcessMsg_connect.find( "authentication failed", 0 , FALSE ) > -1 )
	{
		KMessageBox::error( this, i18n( "Authentication has been failed." ) );
		abort = true;
	}

	if ( ProcessMsg_connect.find( "Call manager exited with error", 0 , FALSE ) > -1 )
	{
		//KMessageBox::error( this, i18n( "Call manager exited with a error." ) );
		abort = true;
	}

	if ( ProcessMsg_connect.find( "Using interface", 0 , FALSE ) > -1 )
	{
		QString dev = ProcessMsg_connect.right(ProcessMsg_connect.length()-16);
		if ( GlobalConfig->PppdDebugLevel > 0 )
			appendLogEntry( i18n( "device found: %1" ).arg(dev),debug );
		currentProfile->setNetworkDevice(dev);
		abort = false;
	}

	if ( ProcessMsg_connect.find( "tcflush failed: Bad file descriptor", 0 , FALSE ) > -1 )
	{
		KMessageBox::error( this, i18n( "Sending data has failed. Connection was terminated." ) );
		abort = true;
	}

	if ( abort )
	{
		if ( GlobalConfig->PppdDebugLevel > 0 )
			appendLogEntry( i18n( "There is a reason for stop connecting, terminating \"%1\" process." ).arg("pppd") ,debug);
		//terminateConnection ( PptpProcess );
		//disconnectClicked();
		status = disconnected;
		setGuiStatus(disconnected);
	}


	bool connectSuccess = false;
	if ( ProcessMsg_connect.find( "remote IP address", 0, FALSE ) > -1 )
		connectSuccess = true;

	// success
	if ( connectSuccess )
	{

		if ( GlobalConfig->PppdDebugLevel > 0 )
			appendLogEntry( i18n( "Connection established." ),debug );

		//connectionTime.restart();
		//uint duration = connectionTime.elapsed();

		sec = hour = min = 0;

		status = connected;

		connectionTime.start();
		connect( &durationTimer, SIGNAL( timeout() ), this, SLOT( slotDurationEvent() ) );
		durationTimer.start( 1000, FALSE );
		setGuiStatus(connected);
		connectSuccess = false;

		if (!currentProfile->getRemoteNet().isEmpty())
			doRoutePptp();
		sleep(1);
		if ( currentProfile->getExecuteCmdAfterConnect())
			executeCommandAfterConnect();

		if (currentProfile->getUseAdditionalNetworkRoutes() && !currentProfile->getAdditionalNetworkRoutes().isEmpty())
			addAdditionalNetworkRoutes();

		if ( currentProfile->getDoPingIP() )
		{
			sleep (3);
			doPingTest();
		}
	}
}

void KVpnc::readFromStderr_pptp()
{
	ProcessMsg_connect = QString( PptpProcess->readStderr() );
	//if ( !ProcessMsg_connect.isEmpty() )
	appendLogEntry ( ProcessMsg_connect,debug );

	bool abort = false;

	if ( ProcessMsg_connect.find( "authentication failed", 0 , FALSE ) > -1 )
	{
		KMessageBox::error( this, i18n( "Authentication has been failed." ) );
		abort = true;
	}

	if ( ProcessMsg_connect.find( "could not find module", 0 , FALSE ) > -1 )
	{
		KMessageBox::error( this, i18n( "Module not found." ) );
		abort = true;
	}

	if ( ProcessMsg_connect.find( "Connection terminated.", 0 , FALSE ) > -1 )
	{
		//   KMessageBox::error( this, i18n( "Connection has been terminated." ) );
		abort = true;
	}

	if ( ProcessMsg_connect.find( "tcflush failed: Bad file descriptor", 0 , FALSE ) > -1 )
	{
		KMessageBox::error( this, i18n( "Sending data has failed. Connection was terminated." ) );
		abort = true;
	}



	if ( abort )
	{
		if ( GlobalConfig->PppdDebugLevel > 0 )
			appendLogEntry( i18n( "There is a reason for stop connecting, terminating \"%1\" process." ).arg("pppd") ,debug);
		//terminateConnection ( PptpProcess );
		disconnectClicked();
		status = disconnected;
		setGuiStatus(disconnected);
	}
}

void KVpnc::readFromStdout_openvpn()
{
	QString ProcessMsg = QString( OpenvpnProcess->readStdout() );
	//if ( !ProcessMsg.isEmpty() )
	appendLogEntry ( ProcessMsg,info );

	//appendLogEntry ( "readFromStdout_openvpn",debug );

	bool abort = false;

	//QMessageBox::information(0,"stdout", ProcessMsg);
	if ( ProcessMsg.find( "Private Key Pass", 0, FALSE ) > -1 )
	{
		if ( GlobalConfig->OpenvpnDebugLevel > 0 )
			appendLogEntry ( i18n( "Private key password requested, send it...\n" ),debug );

		QString PrivKeyPassStr="";
		// 		if (currentProfile->getSavePsk())
		PrivKeyPassStr = currentProfile->getPrivateKeyPass();
		// 		else
		// 			PrivKeyPassStr = TmpPrivKeyPassStr;

		OpenvpnProcess->writeToStdin( PrivKeyPassStr );
		OpenvpnProcess->writeToStdin( "\n" );
	}

	if ( ProcessMsg.find( "Socket bind failed on local address", 0 , FALSE ) > -1 )
	{
		KMessageBox::error( this, i18n( "Binding to socket on local address failed." ) );
		abort = true;
	}

	if ( ProcessMsg.find( "No route to host", 0 , FALSE ) > -1 )
	{
		KMessageBox::error( this, i18n( "No route to host found." ) );
		abort = true;
	}

	if ( ProcessMsg.find( "Options error", 0 , FALSE ) != -1 )
	{
		KMessageBox::error( this, i18n( "OpenVPN configuration error." ) );
		abort = true;
	}



	if ( abort )
	{
		if ( GlobalConfig->OpenvpnDebugLevel > 0 )
			appendLogEntry( i18n( "There is a reason for stop connecting, terminating \"%1\" process." ).arg("openvpn"),debug );
		//terminateConnection ( OpenvpnProcess );
		disconnectClicked();
		timer.stop();
	}
	/*
	 if ( ProcessMsg_connect.find( "Peer Connection Initiated", 0 , FALSE ) > -1 )
	 {
	  //KMessageBox::error( this, i18n( "Host not found." ) );
	  appendLogEntry( i18n( "Peer Connection Initiated." ),debug );
	 }
	 */

	bool connectSuccess = false;
	if ( ProcessMsg.find( "Initialization Sequence Completed", 0, FALSE ) > -1 )
		connectSuccess = true;

	// success
	if ( connectSuccess )
	{

		if ( GlobalConfig->KvpncDebugLevel > 0 )
			appendLogEntry( i18n( "Connection established." ),debug );

		//connectionTime.restart();
		//uint duration = connectionTime.elapsed();

		sec = hour = min = 0;

		timer.stop();
		status = connected;

		connectionTime.start();
		connect( &durationTimer, SIGNAL( timeout() ), this, SLOT( slotDurationEvent() ) );
		durationTimer.start( 1000, FALSE );
		setGuiStatus(connected);
		connectSuccess = false;

		if ( currentProfile->getExecuteCmdAfterConnect())
			executeCommandAfterConnect();

		if (currentProfile->getUseAdditionalNetworkRoutes() && !currentProfile->getAdditionalNetworkRoutes().isEmpty())
		{
			addAdditionalNetworkRoutes();
			//	appendLogEntry( i18n( "Add additional network routes..." ),debug );
		}
		else
		{
			//	appendLogEntry( i18n( "No additional routes to add." ),debug );
		}
		if ( currentProfile->getDoPingIP() )
		{
			sleep (3);
			doPingTest();
		}
	}
}

void KVpnc::readFromStderr_openvpn()
{

	QString ProcessMsg = QString( OpenvpnProcess->readStderr() );
	//QMessageBox::information(0,"stderr", ProcessMsg);
	//if ( !ProcessMsg.isEmpty() )
	appendLogEntry ( ProcessMsg,info );
	//appendLogEntry ( "readFromStderr_openvpn",debug );

	bool abort = false;

	/*
	if ( ProcessMsg.find( "authentication failed", 0 , FALSE ) > -1 )
	{
	 KMessageBox::error( this, i18n( "Authentication has been failed." ) );
	 abort = true;
	}
	*/

	if ( ProcessMsg.find( "Enter Private Key Password", 0, FALSE ) > -1 )
	{
		if ( GlobalConfig->OpenvpnDebugLevel > 0 )
			appendLogEntry ( i18n( "Private key password requested, send it...\n" ),debug );

		QString PrivKeyPassStr="";
		// 		if (currentProfile->getSavePsk())
		PrivKeyPassStr = currentProfile->getPrivateKeyPass();
		// 		else
		// 			PrivKeyPassStr = TmpPrivKeyPassStr;

		OpenvpnProcess->writeToStdin( PrivKeyPassStr );
		OpenvpnProcess->writeToStdin( "\n" );
	}


	if ( ProcessMsg.find( "Cannot resolve host address", 0 , FALSE ) > -1 )
	{
		KMessageBox::error( this, i18n( "Host could not be resolved." ) );
		abort = true;
	}

	if ( ProcessMsg.find( "could not find module", 0 , FALSE ) > -1 )
	{
		KMessageBox::error( this, i18n( "Module not found." ) );
		abort = true;
	}

	if ( ProcessMsg.find( "Socket bind failed on local address", 0 , FALSE ) > -1 )
	{
		KMessageBox::error( this, i18n( "Binding to socket on local address failed." ) );
		abort = true;
	}

	if ( ProcessMsg.find( "No route to host", 0 , FALSE ) > -1 )
	{
		KMessageBox::error( this, i18n( "No route to host found." ) );
		abort = true;
	}


	if ( ProcessMsg.find( "Connection terminated", 0 , FALSE ) > -1 )
	{
		//   KMessageBox::error( this, i18n( "Connection has been terminated." ) );
		abort = true;
	}

	if ( ProcessMsg.find( "Cannot open file key file", 0 , FALSE ) > -1 )
	{
		KMessageBox::error( this, i18n( "Cannot open the preshared key file." ) );
		abort = true;
	}

	if ( ProcessMsg.find( "Options error", 0 , FALSE ) > -1 )
	{
		KMessageBox::error( this, i18n( "OpenVPN configuration error." ) );
		abort = true;
	}


	if ( abort )
	{
		if ( GlobalConfig->OpenvpnDebugLevel > 0 )
			appendLogEntry( i18n( "There is a reason for stop connecting, terminating \"%1\" process." ).arg("openvpn"),debug );
		terminateConnection ( OpenvpnProcess );
		disconnectClicked();
		timer.stop();
	}
}

void KVpnc::readFromStdout_ping()
{
	//while ( ConnectProcess->canReadLineStdout() ) {
	//ProcessMsg_connect = ConnectProcess->readLineStdout();
	ProcessMsg_connect = QString( PingProcess->readStdout() );

	//mw->DebugOutput->append( "raw out (pingtest): " + ProcessMsg_connect );

	if ( ProcessMsg_connect.find( "time=", 0, FALSE ) > -1 )
	{
		mw->DebugOutput->append("ping "+ i18n( "out: success: <font color=\"darkgreen\">" ) + ProcessMsg_connect + "</font>" );
		KNotifyClient::event( topLevelWidget()->winId(), "ping sucsess",i18n("Ping was successful."));
	}
	else if ( ProcessMsg_connect.find( "100 % packet loss", 0, FALSE ) > -1 )
	{
		mw->DebugOutput->append( "ping "+i18n( "out: failed: <font color=\"red\">" ) + ProcessMsg_connect + "</font>" );
		KNotifyClient::event( topLevelWidget()->winId(), "ping failed",i18n("Ping has failed."));
	}
	else
	{
		if ( ProcessMsg_connect.find( "failed to bind", 0, FALSE ) > -1 )
		{
			KMessageBox::error( this, i18n( "Binding interface failed!" ) );
		}
		if ( ProcessMsg_connect.find( "Destination Host Unreachable", 0, FALSE ) > -1 )
		{
			KMessageBox::error( this, i18n( "Destination host is unreachable!" ) );
		}


		//mw->DebugOutput->append( "out: " + ProcessMsg_connect );
	}
	// }
}

void KVpnc::readFromStderr_ping()
{
	// while ( ConnectProcess->canReadLineStderr() ) {
	// ProcessMsg_connect = ConnectProcess->readLineStderr();
	ProcessMsg_connect = QString( PingProcess->readStderr() );

	//mw->DebugOutput->append( "raw err (pingtest): " + ProcessMsg_connect );
	if ( ProcessMsg_connect.find( "time=", 0, FALSE ) > -1 )
	{
		mw->DebugOutput->append( "ping "+i18n( "out: success: <font color=\"darkgreen\">" ) + ProcessMsg_connect + "</font>" );
	}
	else if ( ProcessMsg_connect.find( "100 % packet loss", 0, FALSE ) > -1 )
	{
		mw->DebugOutput->append( "ping "+i18n( "out: failed: <font color=\"red\">" ) + ProcessMsg_connect + "</font>" );
	}
	else
	{
		if ( ProcessMsg_connect.find( "failed to bind", 0, FALSE ) > -1 )
		{
			KMessageBox::error( this, i18n( "Binding interface failed!" ) );
		}
		if ( ProcessMsg_connect.find( "Destination Host Unreachable", 0, FALSE ) > -1 )
		{
			KMessageBox::error( this, i18n( "Destination host is unreachable!" ) );
		}
	}
	//}
}

void KVpnc::readFromStdout_route()
{
	//while ( RouteProcess->canReadLineStdout() ) {
	//ProcessMsg_connect = RouteProcess->readLineStdout();
	ProcessMsg_connect = QString( RouteProcess->readStdout() );

	if (GlobalConfig->KvpncDebugLevel > 0 )
		mw->DebugOutput->append( "raw out (route): " + ProcessMsg_connect );

	// }
}

void KVpnc::readFromStderr_route()
{
	// while ( RouteProcess->canReadLineStderr() ) {
	// ProcessMsg_connect = RouteProcess->readLineStderr();
	ProcessMsg_connect = QString( RouteProcess->readStderr() );

	if (GlobalConfig->KvpncDebugLevel > 0 )
		mw->DebugOutput->append( "raw err (route): " + ProcessMsg_connect );
	//}
}

void KVpnc::readFromStdout_setVirtualIP()
{
	QString msg = QString( SetVirtualIPProcess->readStdout() );
	if (GlobalConfig->KvpncDebugLevel > 0)
		appendLogEntry ( msg,debug );
}

void KVpnc::readFromStderr_setVirtualIP()
{
	QString msg = QString( SetVirtualIPProcess->readStderr() );
	appendLogEntry ( msg,error );
	setVirtualIPSuccess=false;
}

void KVpnc::terminateConnection ( QProcess *proc )
{
	if ( proc->isRunning() )
	{
		proc->tryTerminate();
		QTimer::singleShot( 5000, proc, SLOT( kill() ) );
		//sleep(5000);
	}
	timer.stop();
	durationTimer.stop();
	status = disconnected;
	connectCounter = 0;
	setGuiStatus ( disconnected );

}

void KVpnc::connectionEnded()
{
	durationTimer.stop();
	status = disconnected;

	appendLogEntry ( i18n( "Successful disconnected." ) ,info);

	if (currentProfile->getExecuteCmdAfterConnect())
		executeCommandAfterDisconnect();

	// clean up scripts
	if ( !GlobalConfig->holdGeneratedScripts )
	{

		if ( currentProfile->getConnectionType() == VpnAccountData::racoon )
		{
			QFile file( tmpPath + "setkey." + currentProfile->getName() + ".conf" );
			file.remove();
			file.setName( tmpPath + "/setkey." + currentProfile->getName() + ".sh" );
			file.remove();
			file.setName( tmpPath + "racoon." + currentProfile->getName() + ".conf" );
			file.remove();
			file.setName( tmpPath + "iptables." + currentProfile->getName() + ".add_ipsec_racoon.sh" );
			file.remove();
			file.setName( tmpPath + "iptables." + currentProfile->getName() + ".remove_ipsec_racoon.sh" );
			file.remove();
			file.setName( tmpPath + "setkey." + currentProfile->getName() + ".remove_setkey.sh" );
			file.remove();

			if ( currentProfile->getAuthType() == VpnAccountData::psk )
			{
				file.setName( tmpPath + "psk." + currentProfile->getName() + ".key" );
				file.remove();
			}
			file.setName( tmpPath + "racoon." + currentProfile->getName() + ".log" );
			file.remove();
		} // racoon

		if ( currentProfile->getConnectionType() == VpnAccountData::freeswan )
		{
			QFile file( tmpPath + "iptables." + currentProfile->getName() + ".add_ipsec_freeswan.sh" );
			file.remove();
			file.setName( tmpPath + "iptables." + currentProfile->getName() + ".remove_ipsec_freeswan.sh" );
			file.remove();

		} // freeswan

	}

	QString durationString ( i18n( "%1:%2:%3" ).arg( QString().sprintf( "%02u", hour ) ).arg( QString().sprintf( "%02u", min ) , QString().sprintf( "%02u", sec ) ) );
	appendLogEntry ( i18n( "Connection duration was %1 hours, %2 minutes, %3 seconds" ).arg( QString().sprintf( "%02u", hour ), QString().sprintf( "%02u", min ) , QString().sprintf( "%02u", sec ) ), info );
	connectCounter = 0;
	TmpPassword="";
	TmpGroupPassword="";
}

void KVpnc::slotConnectTimedOut()
{
	this->connectCounter++;

	if ( connectCounter > connectSleepRetry )
	{
		// it dont get it connected so we kill it :)
		appendLogEntry ( i18n( "Timeout! Kill connect process!" ),info );
		ConnectProcess->kill();
		timer.stop();
		status = disconnected;
		mw->buttonConnect->setEnabled( true );
		mw->buttonDisconnect->setEnabled( false );
		this->connectAction->setEnabled( true );
		this->disconnectAction->setEnabled( false );
		//parent->statusBar() ->insertItem( i18n( "Connected" ), 2, 0, true );
		kickerMenu->setStatus( disconnected );
		appendLogEntry ( i18n( "Connection failed (timeout)." ),info );
		disconnectClicked();
		slotStatusMsg( i18n( "Connection timed out, \"%1\" killed." ).arg("vpnc"), ID_FLASH_MSG );
		slotStatusMsg( i18n( "Disconnected" ), ID_CONNECTION_MSG );
		QToolTip::remove
			( kickerMenu );
		QToolTip::add
			( kickerMenu, i18n( "Disconnected" ) );
		connectCounter = 0;

		setGuiStatus(disconnected);
	}
	else
	{
		appendLogEntry ( i18n( "Waiting %1 seconds for connect..." ).arg( connectSleepDelay ),info );
		slotStatusMsg( i18n( "Waiting for connect..." ), ID_FLASH_MSG );
	}
}

void KVpnc::newSessionClicked()
{
	// call new profile dialog...
	NewProfileDialog newdlg( parent, this, i18n("New profile"),GlobalConfig, mw->DebugOutput );
	newdlg.exec();
	if ( newdlg.newProfileCreated )
	{

		profileAddedOrDeleted();
		mw->SessionCombo->setCurrentText(newdlg.getName());

		mw->SessionCombo->setEnabled(TRUE);
		deleteSessionAction->setEnabled ( TRUE );
		saveSessionAction->setEnabled ( TRUE );
		importSessionAction->setEnabled ( TRUE );
		connectAction->setEnabled ( TRUE );
		mw->buttonConnect->setEnabled ( TRUE );

		slotStatusMsg ( i18n( "Profile \"%1\" added." ).arg( mw->SessionCombo->currentText() ), ID_FLASH_MSG );
		appendLogEntry ( i18n( "Profile \"%1\" added." ).arg( mw->SessionCombo->currentText() ) ,info);
		saveOptions();
	}
	else
	{
		slotStatusMsg ( i18n( "Creating new profile canceled." ), ID_FLASH_MSG );
		appendLogEntry ( i18n( "Creating new profile canceled." ),info );
	}
}

void KVpnc::saveSessionClicked()
{
	if ( !mw->SessionCombo->currentText().isEmpty() )
	{
		saveOptions();
	}
}

void KVpnc::deleteSessionClicked()
{
	if (currentProfile == 0)
		return;

	QString Name = currentProfile->getName();
	if ( !Name.isEmpty() )
	{
		int result = KMessageBox::questionYesNo ( this, i18n( "Really delete \"%1\"?" ).arg( currentProfile->getName() ), i18n( "Delete?" ) );
		if ( result == 3 ) // yes
		{
			VpnAccountData * it;
			if ( !GlobalConfig->AccountList->isEmpty()) {
				for ( it = GlobalConfig->AccountList->first(); it; it = GlobalConfig->AccountList->next() )
				{
					if (it->getName() == currentProfile->getName())
					{
						mw->SessionCombo->removeItem(mw->SessionCombo->currentItem());
						mw->SessionCombo->setCurrentText(GlobalConfig->AccountList->first()->getName());
						GlobalConfig->AccountList->remove(it);
						saveOptions();
						break;
					}
				}
			}

			if (GlobalConfig->AccountList->isEmpty())
			{
				currentProfile=0L;
				mw->SessionCombo->setEnabled ( FALSE );
				connectAction->setEnabled( FALSE );
				mw->buttonConnect->setEnabled ( FALSE );
			}
			else
				currentProfile = GlobalConfig->AccountList->first();

			profileAddedOrDeleted();

			slotStatusMsg ( i18n( "Profile \"%1\" deleted." ).arg( Name ), ID_FLASH_MSG );
			appendLogEntry ( i18n( "Profile \"%1\" deleted." ).arg( Name ) ,info);
			saveOptions();
			this->sessionToggled( "" );
		}
	}

}

void KVpnc::sessionToggled( const QString& )
{
	//  mw->DebugOutput->append( "sessionToggled" );
	if ( mw->SessionCombo->currentText().isEmpty() )
	{
		this->deleteSessionAction->setEnabled ( FALSE );
		//   this->advancedSessionAction->setEnabled ( FALSE );
		this->saveSessionAction->setEnabled ( FALSE );
	}
	else
	{
		VpnAccountData * it;
		currentProfile = GlobalConfig->AccountList->first();
		for ( it = GlobalConfig->AccountList->first(); it; it = GlobalConfig->AccountList->next() )
		{
			QString Name = mw->SessionCombo->currentText();
			if ( !Name.isEmpty() )
			{
				if ( it->getName() == Name )
				{
					// profile found
					currentProfile = it;

					QString type=""; // visible name
					QString bin="";  // executable name
					if ( currentProfile->getConnectionType() == VpnAccountData::cisco ){
						type = "vpnc";
						bin = "vpnc";
					}
					else if ( currentProfile->getConnectionType() == VpnAccountData::freeswan ){
						type = "freeswan";
						bin="ipsec";
					}
					else if ( currentProfile->getConnectionType() == VpnAccountData::racoon ){
						type = "racoon";
						bin="racoon";
					}
					else if ( currentProfile->getConnectionType() == VpnAccountData::pptp ){
						type = "pptp";
						bin="pppd";
					}
					else if ( currentProfile->getConnectionType() == VpnAccountData::openvpn ){
						type = "openvpn";
						bin="openvpn";
					}
					else {
						type = "other";
						bin="";
					}
	
					if ( GlobalConfig->KvpncDebugLevel > 0 )
					{
						appendLogEntry( i18n( "New type: %1." ).arg( type ),debug );
					}

					bool programAvailiable=false;
					ToolInfo *it;
					//std::cout << "Tool (bin): " << bin << std::endl;
					if (!GlobalConfig->ToolList->isEmpty()){
						for ( it = GlobalConfig->ToolList->first(); it; it = GlobalConfig->ToolList->next() ){
							//std::cout << "Tool (searching): " << it->Name << std::endl;
							if (it->Name == bin ){
								// program is installed
								this->connectAction->setEnabled ( TRUE );
								mw->buttonConnect->setEnabled ( TRUE );
					
								programAvailiable=true;
								break;
							}
						}
					}
					if (!programAvailiable) {
						this->connectAction->setEnabled ( FALSE );
						mw->buttonConnect->setEnabled ( FALSE );
					}
					
					//this->connectAction->setEnabled ( TRUE );
					break;
				}
			}
		}
	}
}

void KVpnc::slotDurationEvent()
{
	sec += 1;
	if ( sec > 59 )
	{
		sec = 0;
		min += 1;
	}
	if ( min > 59 )
	{
		min = 0;
		hour += 1;
	}


	QString durationString ( i18n( "%1:%2:%3" ).arg( QString().sprintf( "%02u", hour ), QString().sprintf( "%02u", min ), QString().sprintf( "%02u", sec ) ) );
	QString ToolTipMsg;
	QString StatusMsg;
	QString TypeString;
	QString IP = TunnelInterfaceIP;
	if (TunnelInterfaceIP.isEmpty())
		IP="none";


	if (currentProfile->getConnectionType() == VpnAccountData::cisco)
	{
		TypeString="Cisco";
		if (GlobalConfig->showInterfaceIPinTooltip)
			ToolTipMsg= i18n( "Connected to server \"%1\" (user: \"%2\", IPSec ID:\"%3\", type: %4, tunnel ip: %5), duration: %6" ).arg( currentProfile->getGateway()).arg( currentProfile->getUserName()).arg(currentProfile->getID()).arg(TypeString).arg(IP).arg( durationString );
		else
			ToolTipMsg= i18n( "Connected to server \"%1\" (user: \"%2\", IPSec ID:\"%3\", type: %4), duration: %5" ).arg( currentProfile->getGateway()).arg( currentProfile->getUserName()).arg(currentProfile->getID()).arg( durationString );
		StatusMsg=i18n( "Connected: %2@%1, %3" ).arg( currentProfile->getGateway() ).arg( currentProfile->getUserName() ).arg(TypeString).arg ( durationString );
	}
	else 	if (currentProfile->getConnectionType() == VpnAccountData::pptp)
	{
		TypeString="PPTP";
		ToolTipMsg= i18n( "Connected to server \"%1\" (user: \"%2\"), duration: %3" ).arg( currentProfile->getGateway()).arg( currentProfile->getUserName()).arg( durationString );
		StatusMsg=i18n( "Connected: %2@%1 (%3), %4" ).arg( currentProfile->getGateway() ).arg( currentProfile->getUserName() ).arg(TypeString).arg ( durationString );
	}
	else if (currentProfile->getConnectionType() == VpnAccountData::freeswan ||
	         currentProfile->getConnectionType() == VpnAccountData::racoon ||
	         currentProfile->getConnectionType() == VpnAccountData::openvpn
	        )
	{
		if (currentProfile->getConnectionType() == VpnAccountData::freeswan)
			TypeString="*S/WAN";
		else
			if (currentProfile->getConnectionType() == VpnAccountData::racoon)
				TypeString="*racoon";
		if (currentProfile->getConnectionType() == VpnAccountData::openvpn)
			TypeString="OpenVPN";
		ToolTipMsg= i18n( "Connected to server \"%1\" (%2), duration: %3" ).arg( currentProfile->getGateway()).arg(TypeString).arg( durationString );
		StatusMsg=i18n( "Connected: %1 (%2) %3" ).arg( currentProfile->getGateway() ).arg(TypeString).arg ( durationString );
	}

	QToolTip::add
		( kickerMenu, ToolTipMsg );
	slotStatusMsg( StatusMsg, ID_CONNECTION_MSG );
	/*
	runningCheckCounter += 1;
	if ( runningCheckCounter == runningCheckTimeout ) {
	if ( !ConnectProcess->isRunning() ) {
	KMessageBox::error( this, i18n( "Connection lost in las^t %1 seconds!" ).arg( QString().setNum( runningCheckTimeout ) ) );
	appendLogEntry ( i18n( "Connection to %1 lost at %2" ).arg( mw->gatewayLineEdit->text() ).arg( QDateTime::currentDateTime().toString( Qt::TextDate ) ),info );
	terminateConnection ( ConnectProcess );
	} else {
	runningCheckCounter = 0;
	}
	}
	*/
}

void KVpnc::appendLogEntry( QString msg, LogType type )
{

	// write to logfile
	if ( GlobalConfig->writeLogFile )
	{
		if ( !logfile.isOpen() )
		{
			KStandardDirs * dirs = KGlobal::dirs();
			logfileName = dirs->saveLocation( "data" );
			logfileName += "/kvpnc/kvpnc.log"; // /root.kde/share/apps/kvpnc.log
			//    KMessageBox::information( this, logfileName, "info" );
			logfile.setName( logfileName );
			if ( !logfile.open( IO_WriteOnly | IO_Append ) )
				KMessageBox::error( this, i18n( "Log file cant be opened!" ) );
			else
			{
				QTextStream s( &logfile );
				s << i18n( "Log session started at: " );
				s << QDateTime::currentDateTime().toString( Qt::TextDate );
				s << "\n";
			}
		}

		if (logfile.isOpen())
		{
			QString prefix;
			switch ( type )
			{
					case info:
					prefix=i18n("info")+": ";
					break;
					case remote:
					prefix=i18n("remote")+": ";
					break;
					case error:
					prefix=i18n("error")+": ";
					break;
					case success:
					prefix=i18n("success")+": ";
					break;
					case debug:
					prefix=i18n("debug")+": ";
					break;
			}
			s << prefix << msg << "\n";
		}


		// write into debug console
		QString prefix;
		if (GlobalConfig->useColorizedLogOutput)
			prefix+="<font color=\"";

		switch ( type )
		{
				case info:
				if (GlobalConfig->useColorizedLogOutput)
				{
					prefix+=InfoLogColor.name();
					prefix+="\">";
				}
				prefix+="info: ";
				break;
				case remote:
				if (GlobalConfig->useColorizedLogOutput)
				{
					prefix+=RemoteLogColor.name();
					prefix+="\">";
				}
				prefix+="remote: ";

				break;
				case error:
				if (GlobalConfig->useColorizedLogOutput)
				{
					prefix+=ErrorLogColor.name();
					prefix+="\">";
				}
				prefix+="error: ";

				break;
				case success:
				if (GlobalConfig->useColorizedLogOutput)
				{
					prefix+=SuccessLogColor.name();
					prefix+="\">";
				}
				prefix+="success: ";
				break;
				case debug:
				if (GlobalConfig->useColorizedLogOutput)
				{
					prefix+=DebugLogColor.name();
					prefix+="\">";
				}
				prefix+="debug: ";
				break;
		}

		QString postfix;
		if (GlobalConfig->useColorizedLogOutput)
			postfix+="</font>";
		mw->DebugOutput->append( prefix+msg+postfix );
	}
}

void KVpnc::setGuiStatus( int status )
{
	if ( status == connected )
	{
		setIcon( connectedIcon );

		QString durationString ( i18n( "%1:%2:%3" ).arg( QString().sprintf( "%02u", hour ) ).arg( QString().sprintf( "%02u", min ) ).arg( QString().sprintf( "%02u", sec ) ) );
		QString TypeString="";
		QString ConnectMsg="";
		QString LogMsg="";
		QString ToolTipMsg="";

		QToolTip::remove ( kickerMenu );

		if ( currentProfile->getConnectionType() == VpnAccountData::cisco )
		{
			TypeString="Cisco";
			ToolTipMsg= i18n( "Connected to server \"%1\" (user: \"%2\", IPSec ID:\"%3\"), duration: %4, profile \"%5\" (%6)" ).arg( currentProfile->getGateway() ).arg( currentProfile->getUserName() ).arg( currentProfile->getID() ).arg ( durationString ).arg(currentProfile->getName()).arg(TypeString);

			ConnectMsg = i18n( "Connected: %2@%1, %3" ).arg( currentProfile->getGateway() ).arg( currentProfile->getUserName() ).arg ( durationString );

			LogMsg = i18n( "Successful connected to server \"%1\" (user: \"%2\", IPSec ID: \"%3\") at %4" ).arg( currentProfile->getGateway() ).arg( currentProfile->getUserName() ).arg( currentProfile->getID() ).arg( QDateTime::currentDateTime().toString( Qt::TextDate ));
		}

		else if ( currentProfile->getConnectionType() == VpnAccountData::pptp )
		{
			TypeString="PPTP";
			ToolTipMsg= i18n( "Connected to server \"%1\" (user: \"%2\"), duration: %3, profile \"%4\" (%6)" ).arg( currentProfile->getGateway() ).arg( currentProfile->getUserName() ).arg ( durationString ).arg(currentProfile->getName()).arg(TypeString);

			ConnectMsg = i18n( "Connected: %2@%1, %3" ).arg( currentProfile->getGateway() ).arg( currentProfile->getUserName() ).arg ( durationString );

			LogMsg = i18n( "Successful connected to server \"%1\" (user: \"%2\") at %3" ).arg( currentProfile->getGateway() ).arg( currentProfile->getUserName() ).arg( QDateTime::currentDateTime().toString( Qt::TextDate ));
		}

		else if ( currentProfile->getConnectionType() == VpnAccountData::racoon || currentProfile->getConnectionType() == VpnAccountData::freeswan )
		{
			if ( currentProfile->getConnectionType() == VpnAccountData::racoon )
				TypeString = "racoon";
			if ( currentProfile->getConnectionType() == VpnAccountData::freeswan )
				TypeString = "freeswan";

			ConnectMsg = i18n( "Policy successful activated and daemon (%1) running." ).arg( TypeString );

			LogMsg = i18n( "Policy successful activated and daemon (%3) running for server \"%1\" at date %2, profile \"%3\" (%4)" ).arg( currentProfile->getGateway() ).arg( QDateTime::currentDateTime().toString( Qt::TextDate ) ).arg(currentProfile->getName()).arg( TypeString );

		}

		else if ( currentProfile->getConnectionType() == VpnAccountData::openvpn )
		{
			TypeString = "OpenVPN";
			ToolTipMsg =  i18n( "Connected to server \"%1\", duration: %2" ).arg( currentProfile->getGateway() ).arg ( durationString );

			ConnectMsg = i18n( "Connected: %1, %2, profile \"%3\"" ).arg( currentProfile->getGateway() ).arg ( durationString ).arg(currentProfile->getName());

			LogMsg = i18n( "Successful connected to server \"%1\" at %2, profile \"%3\" (%4)" ).arg( currentProfile->getGateway() ).arg( QDateTime::currentDateTime().toString( Qt::TextDate )).arg(currentProfile->getName()).arg(TypeString);
		}

		kickerMenu->setStatus( connected );
		statusColorLabel->setPixmap(*connectedStatusPixmap);
		mw->buttonConnect->setEnabled( false );
		mw->buttonDisconnect->setEnabled( true );
		this->connectAction->setEnabled( false );
		this->disconnectAction->setEnabled( true );


		ToolTipMsg = ConnectMsg;
		QToolTip::add ( kickerMenu, ToolTipMsg );

		slotStatusMsg ( i18n( "Successful connected." ), ID_FLASH_MSG );
		slotStatusMsg ( ConnectMsg , ID_CONNECTION_MSG );
		appendLogEntry ( LogMsg ,info );

		KNotifyClient::event( topLevelWidget()->winId(), "connection established",ToolTipMsg );



		sleep(2);
		if ( GlobalConfig->minimizeAfterConnect )
			hide();
	}
	else if ( status == connecting )
	{
		//setIcon( connectingIcon );
		mw->buttonConnect->setEnabled( false );
		mw->buttonDisconnect->setEnabled( false );
		this->connectAction->setEnabled( false );
		this->disconnectAction->setEnabled( false );
		kickerMenu->setStatus( connecting );
		QToolTip::remove
			( kickerMenu );
		QToolTip::add( kickerMenu, i18n( "Connecting..." ) );

		statusColorLabel->setPixmap(*connectingStatusPixmap);

		slotStatusMsg( i18n( "Connecting..." ), ID_CONNECTION_MSG );
		slotStatusMsg ( i18n( "Connecting..." ), ID_FLASH_MSG );

		connectClicked();

	}
	else if ( status == disconnected )
	{
		setIcon( disconnectedIcon );
		mw->buttonConnect->setEnabled( true );
		mw->buttonDisconnect->setEnabled( false );
		this->connectAction->setEnabled( true );
		this->disconnectAction->setEnabled( false );
		kickerMenu->setStatus( disconnected );
		QToolTip::remove( kickerMenu );
		QToolTip::add( kickerMenu, i18n( "Disconnected" ) );

		statusColorLabel->setPixmap(*disconnectedStatusPixmap);

		slotStatusMsg( i18n( "Disconnected" ), ID_CONNECTION_MSG );
		slotStatusMsg ( i18n( "Disconnected." ), ID_FLASH_MSG );
		KNotifyClient::event( topLevelWidget()->winId(), "connection finished",i18n("Connection \"%1\" finished").arg(currentProfile->getName()) );

	}

}

void KVpnc::advancedSettingsClicked()
{
	VpnAccountData * it = 0L;
	VpnAccountData *currentAccount = 0L;


	if ( mw->SessionCombo->isEnabled() )
	{

		for ( it = GlobalConfig->AccountList->first(); it; it = GlobalConfig->AccountList->next() )
		{
			if ( it->getName() == mw->SessionCombo->currentText() )
			{
				currentAccount = it;
				break;
			} // profile found
		} // for

	}

	if (currentAccount != 0)
	{
		AdvancedSettingsDialog d1(this, i18n("Advanced settings..."), currentAccount, currentAccount->getConnectionType());
		d1.main->AdvancedSettingsCheckbox->setChecked ( false );
		d1.main->UseGlobalIpsecSecretCheckbox->setChecked ( false );
		d1.main->SingleDesCheckbox->setChecked( false );
		d1.main->IkeGroupCheckbox->setChecked ( false );
		d1.main->LocalPortCheckbox->setEnabled ( false );
		d1.main->PerfectForwardSecurityCheckbox->setChecked( false );
		d1.main->NtDomainNameCheckbox->setEnabled ( false );
		d1.main->ApplicationVersionCheckbox->setEnabled ( false );
		d1.main->UseGlobalIpsecSecretCheckbox->setEnabled ( false );

		d1.main->LocalPortSpinbox->setValue( currentAccount->getLocalPort() );
		d1.main->IkeGroupCombobox->setCurrentText( currentAccount->getIkeGroup() );
		d1.main->PerfectForwardSecrecyCombobox->setCurrentText( currentAccount->getPerfectForwardSecrety() );
		d1.main->NtDomainNameLineedit->setText( currentAccount->getNtDomainName() );
		d1.main->ApplicationVersionLineedit->setText( currentAccount->getApplicationVersion() );

		if ( currentAccount->getUseAdvancedSettings() )
		{
			d1.main->LocalPortCheckbox->setEnabled ( true );
			d1.main->IkeGroupCheckbox->setEnabled ( true );
			d1.main->PerfectForwardSecurityCheckbox->setEnabled ( true );
			d1.main->SingleDesCheckbox->setEnabled ( true );
			d1.main->NtDomainNameCheckbox->setEnabled ( true );
			d1.main->ApplicationVersionCheckbox->setEnabled ( true );
			d1.main->UseGlobalIpsecSecretCheckbox->setEnabled ( true );
			d1.main->AdvancedSettingsCheckbox->setChecked ( true );

			if ( currentAccount->getUseSingleDes() )
			{
				d1.main->SingleDesCheckbox->setChecked( true );
			}

			if ( currentAccount->getUseIkeGroup() )
			{
				d1.main->IkeGroupCheckbox->setChecked ( true );
			}

			if ( currentAccount->getUseGlobalIpsecSecret() )
			{
				d1.main->UseGlobalIpsecSecretCheckbox->setChecked ( true );
			}

			if ( currentAccount->getUsePerfectForwardSecrety() )
			{
				d1.main->PerfectForwardSecrecyCombobox->setEnabled ( true );
				d1.main->PerfectForwardSecurityCheckbox->setChecked( true );
			}

			if ( currentAccount->getUseNtDomainName() )
			{
				d1.main->NtDomainNameLineedit->setEnabled ( true );
				d1.main->NtDomainNameCheckbox->setChecked( true );
			}

			if ( currentAccount->getUseApplicationVersion() )
			{
				d1.main->ApplicationVersionLineedit->setEnabled ( true );
				d1.main->ApplicationVersionCheckbox->setChecked( true );
			}

			if ( currentAccount->getUseLocalPort() )
			{
				d1.main->LocalPortSpinbox->setEnabled ( true );
				d1.main->LocalPortCheckbox->setChecked( true );
			}

			if ( currentAccount->getUseIkeGroup() )
			{
				d1.main->IkeGroupCombobox->setEnabled ( true );
			}

		} // advanced settings



		d1.exec();

		if ( !mw->SessionCombo->currentText()
		        .isEmpty() )
		{
			if ( d1.main->AdvancedSettingsCheckbox->isChecked() )
			{
				currentAccount->setUseAdvancedSettings( true );
			}
			else
			{
				currentAccount->setUseAdvancedSettings( false );
			}
			if ( d1.main->NtDomainNameCheckbox->isChecked() )
			{
				currentAccount->setNtDomainName( d1.main->NtDomainNameLineedit->text() );
				currentAccount->setUseNtDomainName( true );
			}
			else
			{
				currentAccount->setUseNtDomainName( false );
			}

			if ( d1.main->ApplicationVersionCheckbox->isChecked() )
			{
				currentAccount->setApplicationVersion( d1.main->ApplicationVersionLineedit->text() );
				currentAccount->setUseApplicationVersion( true );
			}
			else
			{
				currentAccount->setUseApplicationVersion( false );
			}

			if ( d1.main->IkeGroupCheckbox->isChecked() )
			{
				currentAccount->setIkeGroup( d1.main->IkeGroupCombobox->currentText() );
				currentAccount->setUseIkeGroup( true );
			}
			else
			{
				currentAccount->setUseIkeGroup( false );
			}

			if ( d1.main->PerfectForwardSecurityCheckbox->isChecked() )
			{
				currentAccount->setPerfectForwardSecrety( d1.main->PerfectForwardSecrecyCombobox->currentText() );
				currentAccount->setUsePerfectForwardSecrety( true );
			}
			else
			{
				currentAccount->setUsePerfectForwardSecrety( false );
			}


			if ( d1.main->SingleDesCheckbox->isChecked() )
			{
				currentAccount->setUseSingleDes( true );
			}
			else
			{
				currentAccount->setUseSingleDes( false );
			}

			if ( d1.main->UseGlobalIpsecSecretCheckbox->isChecked() )
			{
				currentAccount->setUseGlobalIpsecSecret( true );
			}
			else
			{
				currentAccount->setUseGlobalIpsecSecret( false );
			}


			if ( d1.main->LocalPortCheckbox->isChecked() )
			{
				currentAccount->setLocalPort( d1.main->LocalPortSpinbox->value() );
				currentAccount->setUseLocalPort( true );
			}
			else
			{
				currentAccount->setUseLocalPort( false );
			}

		}
		else
		{
			currentAccount->setUseAdvancedSettings( false );
		}
		saveSessionClicked();

		mw->SessionCombo->clear();
		for ( it = GlobalConfig->AccountList->first(); it; it = GlobalConfig->AccountList->next() )
		{
			mw->SessionCombo->insertItem( it->getName() );
		} // for
		mw->SessionCombo->setCurrentText(lastProfile);

	}
}

void KVpnc::importProfileClicked()
{

	ImportProfileDialog dlg(this,i18n("Import profile"));
	dlg.exec();

	bool importOK = dlg.importOk;
	if ( importOK )
	{
		VpnAccountData * account = dlg.acc;

		VpnAccountData * it;
		for ( it = GlobalConfig->AccountList->first(); it; it = GlobalConfig->AccountList->next() )
		{
			if ( it->getName() == account->getName() )
			{
				account->setName( QString( account->getName() + "_2" ) );
				KMessageBox::information ( this, i18n( "Profile name exists!\n It will be renamed to \"%1\"." ).arg( account->getName() ), i18n( "Name exist, renamed" ) );

				break;
			}
		}


		GlobalConfig->AccountList->append( account );
		mw->SessionCombo->insertItem( account->getName() );
		mw->SessionCombo->setCurrentText( account->getName() );

		if ( !mw->SessionCombo->isEnabled() )
			mw->SessionCombo->setEnabled( TRUE );

		this->connectAction->setEnabled( true );

		sessionToggled( "" );

		KMessageBox::information( 0, i18n( "Import of \"%1\" was successful." ).arg( account->getName() ) );
		slotStatusMsg ( i18n( "Import of \"%1\" was successful." ).arg( account->getName() ), ID_FLASH_MSG );
	}

}

void KVpnc::slotCancelClicked()
{
	cancelClicked = true;
}

void KVpnc::doRouteRacoon()
{

	if ( ConnectProcess->normalExit () )
	{
		disconnect( ConnectProcess, SIGNAL( processExited () ), this, SLOT( doRouteRacoon() ) );
		/*
		route add -net 192.168.1.0/24 eth0
		*/
		putenv( ( char* ) ( "LC_ALL=C" ) );
		ConnectProcess->clearArguments();
		ConnectProcess->addArgument( "/sbin/route" );
		ConnectProcess->addArgument( "add" );
		ConnectProcess->addArgument( "-net" );
		ConnectProcess->addArgument( currentProfile->getRemoteNet() );
		QString dev;

		dev = currentProfile->getNetworkDevice();


		/* QString defaultinterface="Defaultinterface: "+iface.getDefaultInterface();
		mw->DebugOutput->append( defaultinterface );
		*/

		if (!currentProfile->getUseVirtualIP())
		{
			ConnectProcess->addArgument( dev );
			tmpInterface = dev;
		}
		else
		{
			ConnectProcess->addArgument( QString(dev+":100"));
			tmpInterface = QString(dev+":100");
		}

		if ( currentProfile->getAuthType() != VpnAccountData::psk )
			connect( ConnectProcess, SIGNAL( processExited () ), this, SLOT( doIptablesRacoon() ) );
		else
			connect( ConnectProcess, SIGNAL( processExited () ), this, SLOT( doChmodPskFile() ) );

		if ( !ConnectProcess->start() )
		{
			KMessageBox::error( this, i18n( "\"%1\" start failed!" ).arg("route: \"route add -net " + currentProfile->getRemoteNet() + " " + tmpInterface + "\"") );
			disconnectClicked();
		}
		else
		{
			if ( GlobalConfig->RacoonDebugLevel > 0 )
				mw->DebugOutput->append ( "route: \"route add -net " + currentProfile->getRemoteNet() + " " + tmpInterface + "\"" );
		}
	}
}

void KVpnc::doChmodPskFile()
{
	if ( ConnectProcess )
		disconnect( ConnectProcess, SIGNAL( processExited () ), this, SLOT( doChmodPskFile() ) );

	tmpPath = locateLocal ( "data", "kvpnc/" ); // should be
	putenv( ( char* ) ( "LC_ALL=C" ) );
	ConnectProcess->clearArguments();
	ConnectProcess->addArgument( "chmod" );
	ConnectProcess->addArgument( "go-rwx" );
	ConnectProcess->addArgument( tmpPath + "psk." + currentProfile->getName() + ".key" );

	connect( ConnectProcess, SIGNAL( processExited () ), this, SLOT( doIptablesRacoon() ) );

	if ( !ConnectProcess->start() )
	{
		KMessageBox::error( this, i18n( "\"%1\" start failed!" ).arg( "chmod go-rwx "+tmpPath + "psk." + currentProfile->getName() + ".key" ) );
		disconnectClicked();
	}
	else
	{
		if ( GlobalConfig->KvpncDebugLevel > 0 )
			mw->DebugOutput->append ( i18n( "\"%1\" started." ).arg( "chmod go-rwx "+tmpPath + "psk." + currentProfile->getName() + ".key" ) );
	}

}

void KVpnc::doIptablesRacoon()
{
	if ( ConnectProcess->normalExit () )
	{
		if ( ConnectProcess )
			disconnect( ConnectProcess, SIGNAL( processExited () ), this, SLOT( doIptablesRacoon() ) );
		/*
		iptables -I INPUT -p 50 -i $IFACE -s $VPNGW_IP -j ACCEPT
		*/
		putenv( ( char* ) ( "LC_ALL=C" ) );
		ConnectProcess->clearArguments();
		ConnectProcess->addArgument( "/bin/sh" );
		ConnectProcess->addArgument( tmpPath + "iptables." + currentProfile->getName() + ".add_ipsec_racoon.sh" );

		connect( ConnectProcess, SIGNAL( processExited () ), this, SLOT( doRacoon() ) );

		if ( !ConnectProcess->start() )
		{
			KMessageBox::error( this, i18n( "\"%1\" start failed!" ).arg( "iptables."+currentProfile->getName()+".add_ipsec_racoon.sh" ) );
			disconnectClicked();
		}
		else
		{
			if ( GlobalConfig->RacoonDebugLevel > 0 )
				appendLogEntry( i18n( "\"%1\" started." ).arg( "iptables."+currentProfile->getName()+".add_ipsec_racoon.sh" ) ,debug);
		}
	}
}

void KVpnc::doTailRacoonLog()
{

	putenv( ( char* ) ( "LC_ALL=C" ) );
	TailProcess = new QProcess( this );
	TailProcess->addArgument( "tail" );
	TailProcess->addArgument( "-n" );
	TailProcess->addArgument( "100" );
	TailProcess->addArgument( "-f" );
	TailProcess->addArgument( tmpPath + "racoon." + currentProfile->getName() + ".log" );

	connect( TailProcess, SIGNAL( readyReadStdout() ), this, SLOT( readFromStdout_tail_racoon() ) );
	connect( TailProcess, SIGNAL( readyReadStderr() ), this, SLOT( readFromStderr_tail_racoon() ) );

	if ( !TailProcess->start() )
	{
		KMessageBox::error( this, i18n( "\"%1\" start failed!" ).arg( "tail -f "+tmpPath + "racoon." + currentProfile->getName() + ".log" ) );
		//   disconnectClicked();
	}
	else
	{
		if ( GlobalConfig->KvpncDebugLevel > 0 )
			appendLogEntry( i18n( "\"%1\" started." ).arg( "tail -f "+tmpPath + "racoon." + currentProfile->getName() + ".log" ),debug );

		if ( currentProfile->getExecuteCmdAfterConnect())
			executeCommandAfterConnect();

		if ( currentProfile->getDoPingIP() )
		{
			sleep (3);
			doPingTest();
		}
	}
}

void KVpnc::doRacoon()
{
	if ( ConnectProcess->normalExit () )
	{
		if ( ConnectProcess )
			disconnect( ConnectProcess, SIGNAL( processExited () ), this, SLOT( doRacoon() ) );
		/*
		/usr/sbin/racoon -f <configfile> -l <logfile>
		*/
		QFile f( tmpPath + "racoon." + currentProfile->getName() + ".log");
		if (f.exists())
			f.remove();
		QTextStream s (&f);
		if ( f.open( IO_WriteOnly ) )
		{
			s << "\n";
		}
		f.close();
		s.unsetDevice();
		putenv( ( char* ) ( "LC_ALL=C" ) );
		RacoonProcess = new QProcess ( this );
		RacoonProcess->addArgument( GlobalConfig->pathToRacoon );
		RacoonProcess->addArgument( "-F" ); // foreground we want :)
		RacoonProcess->addArgument( "-f" );
		RacoonProcess->addArgument( tmpPath + "racoon." + currentProfile->getName() + ".conf" );
		RacoonProcess->addArgument( "-l" ); // logfile (we get content with tail process)
		RacoonProcess->addArgument( tmpPath + "racoon." + currentProfile->getName() + ".log" );
		//mw->DebugOutput->append ( "racoon: " +GlobalConfig->pathToRacoon+ " -f "+tmpPath+"/racoon"+ currentProfile->getName() + ".conf" );

		connect( RacoonProcess, SIGNAL( readyReadStdout() ), this, SLOT( readFromStdout_racoon() ) );
		connect( RacoonProcess, SIGNAL( readyReadStderr() ), this, SLOT( readFromStderr_racoon() ) );


		if ( !RacoonProcess->start() )
		{
			KMessageBox::error( this, i18n( "\"%1\" start failed!" ).arg("racoon") );
			disconnectClicked();
		}
		else
		{
			racoonStarted();

			if ( GlobalConfig->RacoonDebugLevel > 0 )
				mw->DebugOutput->append ( "debug: racoon: " + GlobalConfig->pathToRacoon + " -f " + tmpPath + "racoon." + currentProfile->getName() + ".conf" );
			sec = hour = min = 0;

			status = disconnected;
			timer.stop();
			connect( &durationTimer, SIGNAL( timeout() ), this, SLOT( slotDurationEvent() ) );
			durationTimer.start( 1000, FALSE );
			setGuiStatus(connected);

			doTailRacoonLog();
		}
	}

}

void KVpnc::removeRouteRacoon()
{

	if ( DisconnectProcess->normalExit () )
	{
		if ( DisconnectProcess )
			disconnect( DisconnectProcess, SIGNAL( processExited () ), this, SLOT( removeRouteRacoon() ) );
		/*
		route del -net 192.168.1.0/24 eth0
		*/
		putenv( ( char* ) ( "LC_ALL=C" ) );
		DisconnectProcess->clearArguments();
		DisconnectProcess->addArgument( "/sbin/route" );
		DisconnectProcess->addArgument( "del" );
		DisconnectProcess->addArgument( "-net" );
		DisconnectProcess->addArgument( currentProfile->getRemoteNet() );
		QString dev;
		//   NetworkInterface iface(parent);
		//   if ( currentProfile->getNetworkDevice() == "default" ) {
		//
		//    QString defaultinterface=iface.getDefaultInterface();
		//    dev = defaultinterface;
		//    DisconnectProcess->addArgument( dev );
		//   } else

		if (!currentProfile->getUseVirtualIP())
			dev = currentProfile->getNetworkDevice();
		else
			dev = QString(currentProfile->getNetworkDevice()+":100");


		/* QString defaultinterface="Defaultinterface: "+iface.getDefaultInterface();
		mw->DebugOutput->append( defaultinterface );
		*/
		if ( GlobalConfig->KvpncDebugLevel	 > 0 )
			mw->DebugOutput->append ( i18n( "Network device: %1" ).arg( dev ) );
		DisconnectProcess->addArgument( dev );
		tmpInterface = dev;
		connect( DisconnectProcess, SIGNAL( processExited () ), this, SLOT( removeIptablesRacoon() ) );

		if ( !DisconnectProcess->start() )
		{
			KMessageBox::error( this, i18n( "\"%1\" start failed!" ).arg("route del") );
			disconnectClicked();
		}
		else
		{
			if ( GlobalConfig->KvpncDebugLevel > 0 )
				mw->DebugOutput->append ( "route: route del -net "  + currentProfile->getRemoteNet() + " " + tmpInterface );
		}
	}
}

void KVpnc::removeIptablesRacoon()
{
	if ( DisconnectProcess->normalExit () )
	{
		if ( DisconnectProcess )
			disconnect( DisconnectProcess, SIGNAL( processExited () ), this, SLOT( removeIptablesRacoon() ) );

		putenv( ( char* ) ( "LC_ALL=C" ) );
		DisconnectProcess->clearArguments();
		DisconnectProcess->addArgument( "/bin/sh" );
		DisconnectProcess->addArgument( tmpPath + "iptables." + currentProfile->getName() + ".remove_ipsec_racoon.sh" );
		if ( !DisconnectProcess->start() )
		{
			KMessageBox::error( this, i18n( "\"%1\" start failed!" ).arg( "iptables."+currentProfile->getName()+".remove_ipsec_racoon.sh" ) );
			disconnectClicked();
		}
		else
		{
			if ( GlobalConfig->KvpncDebugLevel > 0 )
				mw->DebugOutput->append ( i18n( "\"%1\" started." ).arg( "iptables."+currentProfile->getName()+".remove_ipsec_racoon.sh"  ) );
		}
	}
}

void KVpnc::removeSetkey()
{
	if ( DisconnectProcess )
		disconnect( DisconnectProcess, SIGNAL( processExited () ), this, SLOT( removeSetkey() ) );

	putenv( ( char* ) ( "LC_ALL=C" ) );
	DisconnectProcess = new QProcess ( this );
	DisconnectProcess->addArgument( "/bin/sh" );
	DisconnectProcess->addArgument( tmpPath + "setkey." + currentProfile->getName() + ".remove_setkey.sh" );

	connect( DisconnectProcess, SIGNAL( readyReadStdout() ), this, SLOT( readFromStdout_racoon() ) );
	connect( DisconnectProcess, SIGNAL( readyReadStderr() ), this, SLOT( readFromStderr_racoon() ) );
	connect( DisconnectProcess, SIGNAL( processExited () ), this, SLOT( removeRouteRacoon() ) );

	if ( !DisconnectProcess->start() )
	{
		KMessageBox::error( this, i18n( "\"%1\" start failed!" ).arg("setkey (remove)") );
		disconnectClicked();
	}
	else
	{
		if ( GlobalConfig->KvpncDebugLevel > 0 )
			appendLogEntry( i18n( "\"%1\" started." ).arg("setkey (remove)"),debug );
	}

	connectionEnded();
}

void KVpnc::doRouteFreeswan()
{

	if ( ConnectProcess->normalExit () )
	{
		disconnect( ConnectProcess, SIGNAL( processExited () ), this, SLOT( doRouteFreeswan() ) );
		/*
		route add -net 192.168.1.0/24 eth0
		*/
		putenv( ( char* ) ( "LC_ALL=C" ) );
		ConnectProcess->clearArguments();
		ConnectProcess->addArgument( "/sbin/route" );
		ConnectProcess->addArgument( "add" );
		ConnectProcess->addArgument( "-net" );
		ConnectProcess->addArgument( currentProfile->getRemoteNet() );
		ConnectProcess->addArgument( "ipsec0" );
		QString dev;

		dev = currentProfile->getNetworkDevice();

		NetworkInterface iface(parent);
		QString defaultinterface = "Defaultinterface: " + iface.getDefaultInterface();

		if ( dev.isEmpty() )
		{
			if ( defaultinterface.isEmpty() )
				dev = "lo";
			dev = defaultinterface;
		}

		if (currentProfile->getUseVirtualIP())
			dev= QString(dev+":100");

		if ( GlobalConfig->KvpncDebugLevel > 0 )
			mw->DebugOutput->append( i18n( "Default interface: %1" ).arg( defaultinterface ) );

		ConnectProcess->addArgument( dev );
		tmpInterface = dev;
		connect( ConnectProcess, SIGNAL( processExited () ), this, SLOT( doIptablesFreeswan() ) );

		if ( !ConnectProcess->start() )
		{
			KMessageBox::error( this, i18n( "\"%1\" start failed!" ).arg("route (freeswan)") );
			disconnectClicked();
		}
		else
		{
			if ( GlobalConfig->KvpncDebugLevel > 0 )
				mw->DebugOutput->append ( i18n( "route (freeswan): route add -net " ) + currentProfile->getRemoteNet() + " " + tmpInterface );
		}
	}
}

void KVpnc::doIptablesFreeswan()
{
	if ( ConnectProcess->normalExit () )
	{
		disconnect( ConnectProcess, SIGNAL( processExited () ), this, SLOT( doIptablesFreeswan() ) );
		/*
		iptables -I INPUT -p 50 -i $IFACE -s $VPNGW_IP -j ACCEPT
		*/
		putenv( ( char* ) ( "LC_ALL=C" ) );
		ConnectProcess->clearArguments();
		ConnectProcess->addArgument( "/bin/sh" );
		ConnectProcess->addArgument( tmpPath + "iptables." + currentProfile->getName() + ".add_ipsec_freeswan.sh" );

		if ( !ConnectProcess->start() )
		{
			KMessageBox::error( this, i18n( "\"%1\" start failed!" ).arg("iptables."+currentProfile->getName()+".add_ipsec_freeswan.sh" ) );
			disconnectClicked();
		}
		else
		{
			if ( GlobalConfig->KvpncDebugLevel > 0 )
				mw->DebugOutput->append ( i18n( "\"%1\" started." ).arg("iptables.%1.add_ipsec_freeswan.sh") );

			if ( currentProfile->getExecuteCmdAfterConnect())
				executeCommandAfterConnect();
		}
	}
}

void KVpnc::removeRouteFreeswan()
{

	if ( DisconnectProcess->normalExit () )
	{
		disconnect( DisconnectProcess, SIGNAL( processExited () ), this, SLOT( removeRouteFreeswan() ) );
		/*
		route del -net 192.168.1.0/24 eth0
		*/
		putenv( ( char* ) ( "LC_ALL=C" ) );
		DisconnectProcess->clearArguments();
		DisconnectProcess->addArgument( "/sbin/route" );
		DisconnectProcess->addArgument( "del" );
		DisconnectProcess->addArgument( "-net" );
		DisconnectProcess->addArgument( currentProfile->getRemoteNet() );
		DisconnectProcess->addArgument( "ipsec0" );
		QString dev;
		//   NetworkInterface iface(parent);
		//   if ( currentProfile->getNetworkDevice() == "default" ) {
		//
		//    QString defaultinterface=iface.getDefaultInterface();
		//    dev = defaultinterface;
		//    DisconnectProcess->addArgument( dev );
		//   } else
		dev = currentProfile->getNetworkDevice();


		/* QString defaultinterface="Defaultinterface: "+iface.getDefaultInterface();
		mw->DebugOutput->append( defaultinterface );
		*/
		mw->DebugOutput->append ( "Device: " + dev );
		DisconnectProcess->addArgument( dev );
		tmpInterface = dev;
		connect( DisconnectProcess, SIGNAL( processExited () ), this, SLOT( removeIptablesFreeswan() ) );

		if ( !DisconnectProcess->start() )
		{
			KMessageBox::error( this, i18n( "\"%1\" start failed!" ).arg("route (freeswan) del") );
			disconnectClicked();
		}
		else
		{
			if ( GlobalConfig->KvpncDebugLevel > 0 )
				mw->DebugOutput->append ( "route(freeswan): route del -net " + currentProfile->getRemoteNet() + " " + tmpInterface );
		}
	}
}

void KVpnc::removeIptablesFreeswan()
{
	if ( DisconnectProcess->normalExit () )
	{
		disconnect( DisconnectProcess, SIGNAL( processExited () ), this, SLOT( removeIptablesFreeswan() ) );

		putenv( ( char* ) ( "LC_ALL=C" ) );
		DisconnectProcess->clearArguments();
		DisconnectProcess->addArgument( "/bin/sh" );
		DisconnectProcess->addArgument( tmpPath + "iptables." + currentProfile->getName() + ".remove_ipsec_freeswan.sh" );
		if ( !DisconnectProcess->start() )
			KMessageBox::error( this, i18n( "\"%1\" start failed!" ).arg( "iptables."+currentProfile->getName()+".remove_ipsec_freeswan.sh" ) );
		else
		{
			if ( GlobalConfig->KvpncDebugLevel > 0 )
				mw->DebugOutput->append ( i18n( "\"%1\" started." ).arg( "iptables."+currentProfile->getName()+".remove_ipsec_freeswan.sh" ) );
		}
	}
	disconnectClicked();
}

void KVpnc::doPingTest()
{
	/*
	ping -c3 -I <interfaceIP> <gateway>
	*/
	QString LocalIP;
	QString Interface;
	NetworkInterface iface(parent);
	//LocalIP = iface.getInterfaceIP( iface.getDefaultInterface() );
	if ( currentProfile->getNetworkDevice().isEmpty() )
		Interface = iface.getInterfaceIP( iface.getDefaultInterface() );
	else
		Interface = currentProfile->getNetworkDevice();
	QString PingHostIP = currentProfile->getPingHostIP();
	if ( GlobalConfig->KvpncDebugLevel > 0 )
		mw->DebugOutput->append( i18n( "Host for ping: %1" ).arg( PingHostIP ) );

	/* first: kick up the tunnel */
	putenv( ( char* ) ( "LC_ALL=C" ) );
	PingProcess = new QProcess( this );
	PingProcess->addArgument( "/bin/ping" );
	PingProcess->addArgument( "-c1" );
	PingProcess->addArgument( PingHostIP );
	if ( !PingProcess->start() )
		KMessageBox::error( this, i18n( "\"%1\" start failed!" ).arg( "ping "+currentProfile->getPingHostIP() ) );
	else
	{
		sleep( 3 );
		PingProcess->kill();
		/* second: real ping */
		PingProcess->clearArguments();
		PingProcess->addArgument( "/bin/ping" );
		PingProcess->addArgument( "-c5" );
		PingProcess->addArgument( "-v" );
		//PingProcess->addArgument( "-I" );
		//PingProcess->addArgument( Interface );
		PingProcess->addArgument( PingHostIP );

		connect( PingProcess, SIGNAL( readyReadStdout() ), this, SLOT( readFromStdout_ping() ) );
		connect( PingProcess, SIGNAL( readyReadStderr() ), this, SLOT( readFromStderr_ping() ) );
		connect( PingProcess, SIGNAL( processExited() ), this, SLOT( pingTestExited() ) );

		pingtest = true;

		if ( !PingProcess->start() )
			KMessageBox::error( this, i18n( "\"%1\" start failed!" ).arg( "ping "+currentProfile->getPingHostIP() ) );
		else
		{
			if ( GlobalConfig->KvpncDebugLevel > 0 )
				mw->DebugOutput->append ( i18n( "\"%1\" started." ).arg("ping") );
			sleep(5);
			PingProcess->kill();
			PingProcess->start();
		}
	}
}

void KVpnc::pingTestExited()
{
	disconnect( PingProcess, SIGNAL( processExited () ), this, SLOT( pingTestExited() ) );
	//  disconnect( PingProcess, SIGNAL( readyReadStdout() ), this, SLOT( readFromStdout_ping() ) );
	//  disconnect( PingProcess, SIGNAL( readyReadStderr() ), this, SLOT( readFromStderr_ping() ) );
	// pingtest = false;
}

void KVpnc::vpncStarted()
{
	disconnect ( ConnectProcess, SIGNAL( processExited () ), this, SLOT( vpncStarted() ) );

	/* doing some after tunnel is up */

	//   ConnectProcess->clearArguments();
	//   ConnectProcess->addArgument( "/bin/sh" );
	//   ConnectProcess->addArgument( VpncConnectScript );
	//
	//   connect( ConnectProcess, SIGNAL( readyReadStdout() ), this, SLOT( readFromStdout_vpnc() ) );
	//   connect( ConnectProcess, SIGNAL( readyReadStderr() ), this, SLOT( readFromStderr_vpnc() ) );
	//
	//   if ( !ConnectProcess->start() ) {
	//    KMessageBox::sorry( this, i18n( "\"%1\" start failed!" ).arg( "vpnc-connect" ) );
	//   } else {
	//    appendLogEntry ( i18n( "\"%1\" started." ).arg("vpnc-connect"),info );
	//
	//    //                                 connect( &timer, SIGNAL( timeout() ), this, SLOT( slotConnectTimedOut() ) );
	//    //                                 timer.start( tryConnectTimeout * 1000, FALSE );
	//    //
	//    //                                 mw->buttonConnect->setEnabled( false );
	//
	//   }


}

void KVpnc::racoonStarted()
{}

void KVpnc::freeswanStarted()
{
	disconnect ( FreeswanProcess, SIGNAL( processExited () ), this, SLOT( freeswanStarted() ) );

}

void KVpnc::pppdStarted()
{
	disconnect ( PptpProcess, SIGNAL( processExited () ), this, SLOT( pppdStarted() ) );

	// do replace default route (pppd 2.4.2 doesnt support it)
	//ip route replace default dev ppp0
	if (currentProfile->getReplaceDefaultRoute())
	{
		tmpPath = locateLocal ( "data", "kvpnc/" );
		QString PppdReplaceDefaultRouteScript = tmpPath + "pppd_replace_defaultroute." + currentProfile->getName()+".sh";

		QFile file ( PppdReplaceDefaultRouteScript );
		QTextStream stream( &file );
		if ( file.open( IO_WriteOnly ) )
		{
			stream << "# generated by kvpnc. Do not edit it." << "\n";
			stream << "\n";
			stream << "ip route replace default dev ppp0\n";
			file.close();
		}

		ReplaceDefaultRouteProcess = new QProcess(this);
		ReplaceDefaultRouteProcess->addArgument("/bin/sh");
		ReplaceDefaultRouteProcess->addArgument(PppdReplaceDefaultRouteScript);
		if ( !ReplaceDefaultRouteProcess->start() )
		{
			KMessageBox::sorry( this, i18n( "\"%1\" start failed!" ).arg( i18n("pppd replace route process") ) );
			// 			disconnectClicked();
			// 			setGuiStatus(disconnected);
		}
		else
		{
			appendLogEntry ( i18n( "\"%1\" started." ).arg(i18n("pppd replace route process")),info );
		}
	}
}

void KVpnc::openvpnStarted()
{
	disconnect ( OpenvpnProcess, SIGNAL( processExited () ), this, SLOT( openvpnStarted() ) );

}

QString KVpnc::getX509CertificateID( QString Cert, bool user )
{
	issuerID = "";
	userID = "";

	/*
	issuer
	openssl x509 -noout -issuer -in mykeys.pem

	user
	openssl x509 -noout -subject -in mykeys.pem
	*/
	putenv( ( char* ) ( "LC_ALL=C" ) );
	X509CertificateExtractProcess = new QProcess ( this );
	X509CertificateExtractProcess->addArgument( GlobalConfig->pathToOpenssl );
	X509CertificateExtractProcess->addArgument( "x509" );
	X509CertificateExtractProcess->addArgument( "-noout" );
	if ( user )
		X509CertificateExtractProcess->addArgument( "-subject" );
	else
		X509CertificateExtractProcess->addArgument( "-issuer" );

	X509CertificateExtractProcess->addArgument( "-in" );
	X509CertificateExtractProcess->addArgument( Cert );

	X509CertificateExtractUser = user;

	connect( X509CertificateExtractProcess, SIGNAL( readyReadStdout() ), this, SLOT( readFromStdout_getX509CertificateID() ) );
	connect( X509CertificateExtractProcess, SIGNAL( readyReadStderr() ), this, SLOT( readFromStderr_getX509CertificateID() ) );
	connect( X509CertificateExtractProcess, SIGNAL( processExited () ), this, SLOT( openSSLEnded() ) );

	if ( !X509CertificateExtractProcess->start() )
	{
		KMessageBox::error( this, i18n( "\"%1\" start failed!" ).arg("openssl") );
		return "";
	}
	else
	{
		//KMessageBox::information( this, i18n( "\"%1\" started." ).arg("openssl") );
		if ( GlobalConfig->KvpncDebugLevel > 0 )
			mw->DebugOutput->append( i18n( "\"%1\" started." ).arg("openssl (getCertificate)") );
		X509CertificateExtractRunning = true;

		while ( X509CertificateExtractRunning )
		{
			if ( GlobalConfig->KvpncDebugLevel > 0 )
				appendLogEntry( i18n( "Waiting for process end (getCertificate)." ),debug );
			parent->QApplication::processEvents();
		}
		disconnect( X509CertificateExtractProcess, SIGNAL( readyReadStdout() ), this, SLOT( readFromStdout_getX509CertificateID() ) );
		disconnect( X509CertificateExtractProcess, SIGNAL( readyReadStderr() ), this, SLOT( readFromStderr_getX509CertificateID() ) );
		disconnect( X509CertificateExtractProcess, SIGNAL( processExited () ), this, SLOT( openSSLEnded() ) );
		delete X509CertificateExtractProcess;
		if ( user )
			return userID;
		else
			return issuerID;
	}

}

void KVpnc::readFromStdout_getX509CertificateID()
{
	QString ProcessMsg = "";
	if ( X509CertificateExtractProcess->canReadLineStdout() )
		ProcessMsg = X509CertificateExtractProcess->readLineStdout();
	if ( !ProcessMsg.isEmpty() )
	{
		if ( GlobalConfig->KvpncDebugLevel > 0 )
			mw->DebugOutput->append( i18n( "ID found: %1" ).arg( ProcessMsg.section( '=', 1 ) ) );
		if ( X509CertificateExtractUser )
		{
			userID = ProcessMsg.section( '=', 1 ); // subject = /C=DE... => /C=DE...
			if ( userID.left( 0 ) == ' ' )
				userID = userID.right( userID.length() - 1 );

		}
		else
		{
			issuerID = ProcessMsg.section( '=', 1 ); // issuer= /C=DE... => /C=DE...
			if ( issuerID.left( 0 ) == ' ' )
				issuerID = issuerID.right( issuerID.length() - 1 );
		}
	}
}

void KVpnc::readFromStderr_getX509CertificateID()
{
	QString ProcessMsg = QString( X509CertificateExtractProcess->readStderr() );
	if ( GlobalConfig->KvpncDebugLevel > 0 )
		mw->DebugOutput->append( i18n( "getX509CertificateID() err: %1" ).arg( ProcessMsg ) );
}

void KVpnc::openSSLEnded()
{
	X509CertificateExtractRunning = false;
	if ( GlobalConfig->KvpncDebugLevel > 0 )
		mw->DebugOutput->append( i18n( "OpenSSL finished.\n" ) );

}

void KVpnc::importCertClicked()
{
	ImportCertificateDialog dlg( this, i18n("Import certificate..."), GlobalConfig, mw->DebugOutput );
	//int result =
	dlg.exec();
}

void KVpnc::showProfileManagerClicked()
{
	ProfileManager manager(this, "ProfileManger", parent, mw->DebugOutput, GlobalConfig, currentProfile );
	manager.exec();

	if (manager.MyprofileWidget->profileAddedOrDeleted)
	{
		if (GlobalConfig->AccountList->isEmpty())
		{
			currentProfile=0L;
			mw->SessionCombo->setEnabled ( FALSE );
		}
		else
		{
			if (currentProfile!=0)
				sessionToggled(currentProfile->getName());
		}

		profileAddedOrDeleted();
		if (!manager.MyprofileWidget->NewProfileName.isEmpty())
			sessionToggled(manager.MyprofileWidget->NewProfileName);

	}

}

void KVpnc::executeCommandAfterConnect()
{
	if (!currentProfile->getCommandAfterConnect().isEmpty())
	{
		QString cmd = currentProfile->getCommandAfterConnect();

		// write command after connect script
		QString CmdAfterConnectScript = tmpPath + "cmd_after_connect_script." + currentProfile->getName();
		mw->DebugOutput->append( "CmdAfterConnectScript: " + CmdAfterConnectScript + "\n" );
		QFile file ( CmdAfterConnectScript );
		QTextStream stream( &file );
		if ( file.open( IO_WriteOnly ) )
			stream << cmd;
		file.close();

		CmdAfterConnectProcess = new QProcess ( this );
		CmdAfterConnectProcess->addArgument( "/bin/sh" );
		CmdAfterConnectProcess->addArgument( CmdAfterConnectScript );

		if ( GlobalConfig->KvpncDebugLevel > 0 )
			mw->DebugOutput->append( i18n( "Executing command after connect:\n%1" ).arg( cmd ) );
		if ( !CmdAfterConnectProcess->start() )
		{
			KMessageBox::sorry( this, i18n( "\"%1\" start failed!!" ).arg( CmdAfterConnectScript ) );
			disconnectClicked();
		}
		else
		{
			appendLogEntry ( i18n( "\"%1\" started." ).arg("CmdAfterConnectScript"),info );
		}

	}
}

void KVpnc::executeCommandAfterDisconnect()
{
	if (!currentProfile->getCommandAfterDisconnect().isEmpty())
	{
		QString cmd = currentProfile->getCommandAfterDisconnect();

		// write command after connect script
		QString CmdAfterDisconnectScript = tmpPath + "cmd_after_disconnect_script." + currentProfile->getName();
		mw->DebugOutput->append( "CmdAfterDisconnectScript: " + CmdAfterDisconnectScript + "\n" );
		QFile file ( CmdAfterDisconnectScript );
		QTextStream stream( &file );
		if ( file.open( IO_WriteOnly ) )
			stream << cmd;
		file.close();

		CmdAfterDisconnectProcess = new QProcess ( this );
		CmdAfterDisconnectProcess->addArgument( "/bin/sh" );
		CmdAfterDisconnectProcess->addArgument( CmdAfterDisconnectScript );

		if ( !CmdAfterDisconnectProcess->start() )
		{
			KMessageBox::sorry( this, i18n( "\"%1\" start failed!" ).arg( CmdAfterDisconnectScript ) );
			disconnectClicked();
		}
		else
		{
			appendLogEntry ( i18n( "\"%1\" started." ).arg(CmdAfterDisconnectScript) ,info);
		}

		if ( GlobalConfig->KvpncDebugLevel > 0 )
			mw->DebugOutput->append( i18n( "Executing command after disconnect:\n%1" ).arg( cmd ) );
	}
}

void KVpnc::doRoutePptp()
{
	/*
	route add -net 192.168.1.0/24 eth0
	*/
	putenv( ( char* ) ( "LC_ALL=C" ) );
	RouteProcess = new QProcess(this);
	RouteProcess->addArgument( "/sbin/route" );
	RouteProcess->addArgument( "add" );
	RouteProcess->addArgument( "-net" );
	RouteProcess->addArgument( currentProfile->getRemoteNet() );
	QString dev;

	// 	if (!currentProfile->getNetworkDevice().isEmpty())
	// 		dev = currentProfile->getNetworkDevice();
	// 	else
	dev="ppp0";

	/* QString defaultinterface="Defaultinterface: "+iface.getDefaultInterface();
	mw->DebugOutput->append( defaultinterface );
	*/


	RouteProcess->addArgument( dev );
	tmpInterface = dev;

	connect( RouteProcess, SIGNAL( readyReadStdout() ), this, SLOT( readFromStdout_pptp() ) );
	connect( RouteProcess, SIGNAL( readyReadStderr() ), this, SLOT( readFromStderr_pptp() ) );

	if ( !RouteProcess->start() )
	{
		KMessageBox::error( this, i18n( "\"%1\" start failed!" ).arg("route") );
		disconnectClicked();
	}
	else
	{
		if ( GlobalConfig->KvpncDebugLevel > 0 )
			mw->DebugOutput->append ( "route: \"route add -net "  + currentProfile->getRemoteNet() + " " + tmpInterface + "\"" );
	}

}


void KVpnc::slotConnecting()
{
	setGuiStatus(connecting);
}

void KVpnc::profileAddedOrDeleted()
{
	// need to clear list and rebuid it.

	mw->SessionCombo->clear();

	VpnAccountData * it;
	for ( it = GlobalConfig->AccountList->first(); it; it = GlobalConfig->AccountList->next() )
	{
		mw->SessionCombo->insertItem( it->getName() );
	}

	if ( !lastProfile.isEmpty() )
	{
		// 		mw->SessionCombo->setCurrentItem( 0 );
		sessionToggled( "" );
	}
	else
	{
		if ( !GlobalConfig->AccountList->isEmpty() )
		{
			GlobalConfig->AccountList->first();
			mw->SessionCombo->setCurrentItem( 0 );
		}
		else
		{
			this->deleteSessionAction->setEnabled ( FALSE );
			//    this->advancedSessionAction->setEnabled ( FALSE );
			this->saveSessionAction->setEnabled ( FALSE );
			//    this->advancedSessionAction->setEnabled ( FALSE );
			this->importSessionAction->setEnabled ( TRUE );

			this->connectAction->setEnabled ( FALSE );
			mw->buttonConnect->setEnabled ( FALSE );
		}
	}
}

bool KVpnc::setVirtualIP()
{
	/*
	if (currentProfile->getNetworkDevice().isEmpty() || currentProfile->getVirtualIP().isEmpty()){
	KMessageBox::error ( this, i18n( "Empty interface or empty virtual IP!" ), i18n( "Bad value" ) );
	return false;
	}
	if (!Utils().isValidNetworkDevice(currentProfile->getNetworkDevice())){
	KMessageBox::error ( this, i18n( "Interface doesnt exits!" ), i18n( "No interface" ) );
	return false;
	}

	setVirtualIPSuccess=true;
	SetVirtualIPProcess = new QProcess(this);
	SetVirtualIPProcess->addArgument("/sbin/ifconfig");
	SetVirtualIPProcess->addArgument(QString(NetworkDevice+":100"));
	SetVirtualIPProcess->addArgument(currentProfile->getVirtualIP());
	connect( SetVirtualIPProcess, SIGNAL( readyReadStdout() ), this, SLOT( readFromStdout_setVirtualIP() ) );
	connect( SetVirtualIPProcess, SIGNAL( readyReadStderr() ), this, SLOT( readFromStderr_setVirtualIP() ) );

	if ( !SetVirtualIPProcess->start() )
	{
	std::cerr << i18n("Unable to set virtual IP!") << std::endl;
	return false;
	}
	else {
	while (SetVirtualIPProcess->isRunning()) {}
	return setVirtualIPSuccess;
	}
	*/


	return false;
}


void KVpnc::toggleDebugConsole()
{
	if ( !GlobalConfig->showDebugConsole )
	{
		GlobalConfig->showDebugConsole = true;
		mw->DebugOutput->show();
		//resize( minimumSizeHint() );
	}
	else
	{
		GlobalConfig->showDebugConsole=false;
		mw->DebugOutput->hide();
		setMinimumSize(0,0);
		this->resize( minimumSizeHint() );
	}
}

void KVpnc::addAdditionalNetworkRoutes()
{
	if (currentProfile == 0)
		return;

	if  (!currentProfile->getAdditionalNetworkRoutes().isEmpty())
	{
		if ( GlobalConfig->KvpncDebugLevel > 0 )
			mw->DebugOutput->append( i18n( "Setting additional network routes..." ));
		QStringList AdditionalNetworkRoutes = currentProfile->getAdditionalNetworkRoutes();


		QString dev = currentProfile->getNetworkDevice();

		NetworkInterface iface(parent);
		QString defaultinterface = iface.getDefaultInterface();
		

		if (currentProfile->getConnectionType() == VpnAccountData::racoon ) {
			if ( dev.isEmpty() )
			{
				if ( defaultinterface.isEmpty() )
					dev = "lo";
				else
					dev = defaultinterface;
	
			}
		}
		else if (currentProfile->getConnectionType() == VpnAccountData::cisco ||
						currentProfile->getConnectionType() == VpnAccountData::openvpn ){
			if ( dev.isEmpty() || dev=="default" ) {
				//FIXME dynamiclly?
				dev="tun0";
			}
		}
		else if (currentProfile->getConnectionType() == VpnAccountData::freeswan ) {
			if ( dev.isEmpty() || dev=="default" ) {
				//FIXME dynamiclly?
				dev="ipsec0";
			}
		}
		else if (currentProfile->getConnectionType() == VpnAccountData::pptp ){
			if ( dev.isEmpty() || dev=="default" ) {
				//FIXME dynamiclly?
				dev="ppp0";
			}
		}

		if ( GlobalConfig->KvpncDebugLevel > 0 )
			mw->DebugOutput->append( i18n( "Default interface: %1" ).arg( dev ) );

		QString tmpPath = locateLocal ( "data", "kvpnc/" );
		QFile file( tmpPath + "addAdditionalNetworkRoutes." + currentProfile->getName() + ".sh" );
		QString Routes="";
		QTextStream stream( &file );
		if ( file.open( IO_WriteOnly ) )
		{
			for (QStringList::Iterator it = AdditionalNetworkRoutes.begin(); it != AdditionalNetworkRoutes.end();++it)
			{
				QString net = *it;
				if (net.find("/32") > 0 )
					stream << "/sbin/route add " << net.left(net.length()-3) << " " << dev << "\n";
				else
					stream << "/sbin/route add -net " << *it << " " << dev << "\n";
				Routes+=QString(*it)+"\n";
			}
			file.close();
			RouteProcess = new QProcess(this);
			RouteProcess->addArgument( "/bin/sh" );
			RouteProcess->addArgument( tmpPath + "addAdditionalNetworkRoutes." + currentProfile->getName() + ".sh" );
			tmpInterface = dev;

			connect( RouteProcess, SIGNAL( readyReadStdout() ), this, SLOT( readFromStdout_route() ) );
			connect( RouteProcess, SIGNAL( readyReadStderr() ), this, SLOT( readFromStderr_route() ) );

			if ( !RouteProcess->start() )
			{
				KMessageBox::error( this, i18n( "\"%1\" start failed!" ).arg("route (addtitional network routes)") );
				//disconnectClicked();
			}
			else
			{
				if ( GlobalConfig->KvpncDebugLevel > 0 )
					mw->DebugOutput->append ( i18n( "Adding the following additional network routes:\n" ) + Routes );
				while (RouteProcess->isRunning())
					parent->processEvents();
				disconnect( RouteProcess, SIGNAL( readyReadStdout() ), this, SLOT( readFromStdout_route() ) );
				disconnect( RouteProcess, SIGNAL( readyReadStderr() ), this, SLOT( readFromStderr_route() ) );
			}
 		}
 	}
}

void KVpnc::removeAdditionalNetworkRoutes()
{
	if( !currentProfile->getAdditionalNetworkRoutes().isEmpty())
	{
		QStringList AdditionalNetworkRoutes = currentProfile->getAdditionalNetworkRoutes();
		QString dev = currentProfile->getNetworkDevice();

		NetworkInterface iface(parent);
		QString defaultinterface = iface.getDefaultInterface();

		if ( dev.isEmpty() )
		{
			if ( defaultinterface.isEmpty() )
				dev = "lo";
		}
		dev = defaultinterface;

		if ( GlobalConfig->KvpncDebugLevel > 0 )
			mw->DebugOutput->append( i18n( "Default interface: %1" ).arg( defaultinterface ) );

		tmpPath = locateLocal ( "data", "kvpnc/" );
		QFile file( tmpPath + "delAdditionalNetworkRoutes." + currentProfile->getName() + ".sh" );
		QString Routes="";
		QTextStream stream( &file );
		if ( file.open( IO_WriteOnly ) )
		{
			for (QStringList::Iterator it = AdditionalNetworkRoutes.begin(); it != AdditionalNetworkRoutes.end();++it)
			{
				QString net = *it;
				if (net.find("/32") > 0 )
					stream << "/sbin/route del " << net.left(net.length()-3) << " " << dev << "\n";
				else
					stream << "/sbin/route del -net " << *it << " " << dev << "\n";;
				Routes+=QString(*it)+"\n";
			}
			file.close();
			RouteProcess = new QProcess(this);
			RouteProcess->addArgument( "/bin/sh" );
			RouteProcess->addArgument( tmpPath + "delAdditionalNetworkRoutes." + currentProfile->getName() + ".sh" );
			tmpInterface = dev;

			if ( !RouteProcess->start() )
			{
				KMessageBox::error( this, i18n( "\"%1\" start failed!" ).arg("route (addtitional network routes)") );
				//disconnectClicked();
			}
			else
			{
				if ( GlobalConfig->KvpncDebugLevel > 0 )
					mw->DebugOutput->append ( i18n( "Removing the following additional network routes:\n" ) + Routes );
				while (RouteProcess->isRunning())
					parent->processEvents();
			}

		}
	}
}

void KVpnc::showLogViewer()
{
	LogViewerDialog dlg(this, i18n("show log viewer"));
	dlg.exec();
}


#include "kvpnc.moc"
// --- DCOP stuff ---
QString KVpnc::getVersion()
{
	return parent->aboutData()->version();
}

void KVpnc::doConnect()
{
	connectClicked();
}

void KVpnc::doDisconnect()
{
	disconnectClicked();
}

uint KVpnc::getConnectedTime()
{

	if (status == connected)
	{
		// return secs
		return hour*60*60+min*60+sec;
	}
	else return 0;
}

QString KVpnc::getStatus()
{
	if (status == connected)
		return "connected";
	else if (status == disconnected)
		return "disconnected";
	else if (status == connecting)
		return "connecting";
	else
		return "unknown";
}

QString KVpnc::getCurrentProfile()
{
	return currentProfile->getName();
}

void KVpnc::setProfile(QString profilename)
{
	VpnAccountData * it;
	for ( it = GlobalConfig->AccountList->first(); it; it = GlobalConfig->AccountList->next() )
	{
		if ( it->getName() == profilename )
		{
			mw->SessionCombo->setCurrentText(profilename);
			break;
		}
	}
}

void KVpnc::createNewProfile()
{
	newSessionClicked();
}

void KVpnc::deleteCurrentProfile()
{
	deleteSessionClicked();
}

void KVpnc::openPreferences()
{
	slotSettings();
}

void KVpnc::doImportCert()
{
	importCertClicked();
}

void KVpnc::doImportPcf()
{
	importProfileClicked();
}

QStringList KVpnc::profiles()
{
	QStringList profiles;
	VpnAccountData * it;
	if (!GlobalConfig->AccountList->isEmpty())
	{
		for ( it = GlobalConfig->AccountList->first(); it; it = GlobalConfig->AccountList->next() )
		{
			//std::cout << it->getName() << "\n";
			profiles.append(it->getName());
		}
	}
	return profiles;
}

void KVpnc::doQuit()
{
	quitCalled();
}

